/***********************************************************************
 * Agent.hh
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   class Agent
 *
 * requires:
 *
 *
 * Created on: 27.03.2001 by wenns
 * Maintainer: harlekin
 * Group:
 * Co-authors: cyco
 *
 * Changelog:
 *
 * $Id: Agent.hh,v 1.2 2004/03/04 16:06:41 harlekin Exp $
 *
 **********************************************************************/

#ifndef AGENT_HH
#define AGENT_HH

//#define USE_PLANNER

#include "Communicator.hh"
#include "Sensors.hh"
#include "Effectors.hh"
#include "WorldModel.hh"
#ifdef DEBUG_ON
#include "Logger.hh"
#endif
#include "Behaviour.hh"
#include "SkillPool.hh"

#ifdef USE_PLANNER
#include "./panorama/AbsolutePerceptionImplVW.h"
#include "./panorama/ActorInterfaceImplVW.h"
#include "./panorama/Core.h"
#endif



/**
 * This class realizes the classical scheme of an agent with his sensors,
 * effectors, wolrdmodel etc.
 */
class Agent {
private:
  static Sensors      *sensors;
  static Effectors    *effectors;
  static WorldModel   *worldModel;
  static Communicator *communicator;
  static SkillPool    *skillPool;
  Behaviour *behaviour;

#ifdef  USE_PLANNER
  static AbsolutePerceptionImplVW* plannerPerceptionInterface;
  //static ActorInterfaceImplVW* plannerActorInterface;
  static Core* plannerCore;
  /** Set up the planner */
  void setupPlanner();
  void setPlannerBehaviour();
  public:
  static bool useNetExecutor;
  private:
#endif

 
  void setBehaviour();

   
  int behaviourType;

#ifdef DEBUG_ON
  Logger *logger;
#endif

public:
  Agent(const char *serverHost, const int serverPort,
        const char *teamname, const bool goalie);

  ~Agent();

  /** Implements the main loop. */
  void run();

  /** Initializes the Behaviour for this agent*/
  void init();

  /** for all these funny things by josh*/
  void testJosh();
  /** for testing purposes */
  void testWenns();
  /** for internal testing by majunor*/
  void testMajunor();
  /** for internal testing by pagra*/
  void testPagra();
  /** for testing stuff by tekgeist*/
  void testTekgeist();
  void testFritsche();
  /** for debug stuff by cniehaus*/
  void testCniehaus();

  /** \name Accessors */
  //@{
  static WorldModel*   getWorldModel()   { return worldModel;   }
  static SkillPool*    getSkillPool()    { return skillPool;    }
  static Effectors*    getEffectors()    { return effectors;    }
  static Sensors*      getSensors()      { return sensors;      }
  #ifdef USE_PLANNER
  static AbsolutePerceptionImplVW* getPlannerPerceptionInterface() {return plannerPerceptionInterface;}
  //static ActorInterfaceImplVW* getPlannerActorInterface() {return plannerActorInterface;}  
  static Core* getPlannerCore() {return plannerCore;}
  #endif
  //@}
};

// $Source: /home/robocup/virtualwerder/CVSRepository/VirtualWerder2004/Agent.hh,v $
#endif // AGENT_HH
