/***********************************************************************
 * AgentPlanLogger.cc
 * Copyright (c) 2003 Virtual Werder
 *
 * implements:
 *   class AgentPlanLogger
 *
 * requires:
 *
 *
 * 3B
 * Created on: 07.01.2003 by aw
 * Maintainer: aw, mangels
 * Group:
 * Co-authors:
 *
 * Changelog:
 *
 * $Id: AgentPlanLogger.cc,v 1.3 2004/03/31 10:01:46 harlekin Exp $
 *
 **********************************************************************/

#include "AgentPlanLogger.hh"
#include "WorldModel.hh"

// static members
AgentPlanLogger *AgentPlanLogger::apl = NULL;

AgentPlanLogger::AgentPlanLogger(WorldModel *worldModel) {
    wm = worldModel;
    ms = &(worldModel->myself);
    uNum = ms->getUniformNumber(); 
    openFile();
}

void AgentPlanLogger::openFile() {
    using namespace std;
    stringstream tempName;
    tempName << "./agentplanlog/agentplan-player-" << uNum << ".log";
    const char* const filename = tempName.str().c_str();
    // open file
    //out.open(filename, ios_base::out | ios_base::ate); !works with g++-3.2 only!
    out.open(filename, ( ofstream::out | ofstream::trunc) );
}


void AgentPlanLogger::writePoint( const Vector2D &point,
				  APLLevel level,
				  int r, int g, int b,
				  int uNum ) {
  if ( (!out.is_open()) || (wm->getTime()->getStoppedCount() != 0) ) {
    cout << "returning" << endl;
    return;
  }
  
  if ( !wm->getTime()->isUndefined() ) {
    if ( uNum == -1 ) uNum = this->uNum;
    writeLine( point + Vector2D(-0.5,-0.5), point + Vector2D(0.5,0.5), level, r, g, b, uNum );
    writeLine( point + Vector2D(0.5,-0.5), point + Vector2D(-0.5,0.5), level, r, g, b, uNum );
  }
}


void AgentPlanLogger::writeLine( const Vector2D &p1, const Vector2D &p2,
				 APLLevel level,
				 int r, int g, int b,
				 int uNum ) {
  if ( (!out.is_open()) || (wm->getTime()->getStoppedCount() != 0) ) return;

  if ( !wm->getTime()->isUndefined() ) {
    if ( uNum == -1 ) uNum = this->uNum;
    out << wm->getTime()->getTime() 
	<< " " 
	<< uNum 
	<< " "
	<< level 
	<< " line (" 
	<< p1.getX() << "," << p1.getY()
	<< ") ("
	<< p2.getX() << "," << p2.getY()
	<< ") # " << r << " " << g << " " << b << "\n";
  }
}
  
void AgentPlanLogger::writePolyline( vector<Vector2D> points,
				     APLLevel level,
				     int r, int g, int b,
				     int uNum ) {
  if ( (!out.is_open()) || (wm->getTime()->getStoppedCount() != 0) || 
       points.size() < 3 ) 
    return;
  
  if ( !wm->getTime()->isUndefined() ) {
    if ( uNum == -1 ) uNum = this->uNum;
    out << wm->getTime()->getTime() 
	<< " " 
	<< uNum 
	<< " "
	<< level 
	<< " polyline";
    vector<Vector2D>::iterator i;
    for (i = points.begin(); i != points.end(); ++i)
      out << " (" << i->getX() << "," << i->getY() << ")";
    out << " # " << r << " " << g << " " << b << "\n";
  }
}

void AgentPlanLogger::writeNumber( std::string number,
		    const Vector2D &upperLeft,
		    float width, float height,
		    APLLevel level, int r, int g, int b)
{
  Vector2D tmp = upperLeft;
  DEBUG2_KHUEBNER("Write: ", number);
  for (unsigned int i=0; i<number.length(); i++)
  {
    int num = atoi(number.substr(i,1).data());
    writeDigit(num, tmp, width, height, level, r, g, b);
    tmp.setX(tmp.getX() + 1.3 * width); 
  }
}

void AgentPlanLogger::writeDigit( int digit,
		    const Vector2D &upperLeft,
		    float width, float height,
		    APLLevel level, int r, int g, int b)
{
  if (digit < 0 || digit > 9) return;

  std::vector<Vector2D> points;

  std::vector<Vector2D> pos;
  pos.push_back(Vector2D(upperLeft.getX()      , upperLeft.getY()+height  ));
  pos.push_back(Vector2D(upperLeft.getX()+width, upperLeft.getY()+height  ));
  pos.push_back(Vector2D(upperLeft.getX()      , upperLeft.getY()+height/2));
  pos.push_back(Vector2D(upperLeft.getX()+width, upperLeft.getY()+height/2));
  pos.push_back(Vector2D(upperLeft.getX()      , upperLeft.getY()         ));
  pos.push_back(Vector2D(upperLeft.getX()+width, upperLeft.getY()         ));
 
  switch (digit)
  {
    case 0: 
      points.push_back(pos[1]);
      points.push_back(pos[0]);
      points.push_back(pos[4]);
      writePolyline(points, level, r, g, b);
      points.clear();
      writeDigit(7, upperLeft, width, height, level, r, g, b);
      break;
    case 1:
      points.push_back(pos[1]);
      points.push_back(pos[1]);
      points.push_back(pos[5]);
      writePolyline(points, level, r, g, b);
      points.clear();
      break;
    case 2:
      points.push_back(pos[1]);
      points.push_back(pos[0]);
      points.push_back(pos[2]);
      points.push_back(pos[3]);
      points.push_back(pos[5]);
      points.push_back(pos[4]);
      writePolyline(points, level, r, g, b);
      points.clear();
      break;
    case 3:
      points.push_back(pos[2]);
      points.push_back(pos[2]);
      points.push_back(pos[3]);
      writePolyline(points, level, r, g, b);
      points.clear();
      points.push_back(pos[0]);
      points.push_back(pos[0]);
      points.push_back(pos[1]);
      writePolyline(points, level, r, g, b);
      points.clear();
      writeDigit(7, upperLeft, width, height, level, r, g, b);
      break;
    case 4:
      points.push_back(pos[3]);
      points.push_back(pos[2]);
      points.push_back(pos[4]);
      writePolyline(points, level, r, g, b);
      points.clear();
      writeDigit(1, upperLeft, width, height, level, r, g, b);
      break;
    case 5:
      points.push_back(pos[0]);
      points.push_back(pos[1]);
      points.push_back(pos[3]);
      points.push_back(pos[2]);
      points.push_back(pos[4]);
      points.push_back(pos[5]);
      writePolyline(points, level, r, g, b);
      points.clear();
      break;
    case 6:
      points.push_back(pos[0]);
      points.push_back(pos[0]);
      points.push_back(pos[2]);
      writePolyline(points, level, r, g, b);
      points.clear();
      writeDigit(5, upperLeft, width, height, level, r, g, b);
      break;
    case 7:
      points.push_back(pos[4]);
      points.push_back(pos[4]);
      points.push_back(pos[5]);
      writePolyline(points, level, r, g, b);
      points.clear();
      writeDigit(1, upperLeft, width, height, level, r, g, b);
      break;
    case 8:
      points.push_back(pos[0]);
      points.push_back(pos[0]);
      points.push_back(pos[2]);
      writePolyline(points, level, r, g, b);
      points.clear();
      writeDigit(9, upperLeft, width, height, level, r, g, b);
      break;
    case 9:
      points.push_back(pos[3]);
      points.push_back(pos[3]);
      points.push_back(pos[5]);
      writePolyline(points, level, r, g, b);
      points.clear();
      writeDigit(5, upperLeft, width, height, level, r, g, b);
      break;
  }
}

void AgentPlanLogger::writeEllipse( const Vector2D &upperLeft,
				    float width, float height,
				    APLLevel level,
				    int r, int g, int b,
				    int uNum ) {
  if ( (!out.is_open()) || (wm->getTime()->getStoppedCount() != 0) ) return;
  
  if ( !wm->getTime()->isUndefined() ) {
    if ( uNum == -1 ) uNum = this->uNum;
    out << wm->getTime()->getTime() 
	<< " " 
	<< uNum 
	<< " "
	<< level 
	<< " ellipse (" 
	<< upperLeft.getX() 
	<< "," 
	<< upperLeft.getY()
	<< ") "
	<< width << " " << height
	<< " # " << r << " " << g << " " << b << "\n";
  }
}
 
void AgentPlanLogger::writeBall( APLLevel level, float conf ) {
  if (conf == -1) conf = ConfigFile::goodConfidenceBall;

  if ( wm->ball.getConfidence(wm->getTime()) >= conf ) {
    if ( wm->getFieldSide() == FS_Left ) {
      writeEllipse( wm->ball.getPosition() - Vector2D(0.5,0.5),
		    1,1, level, 255, 255, 255 );
      writeLine( wm->ball.getPosition(), 
		 wm->ball.getPosition() + wm->ball.getVelocity() * 3,
		 level, 255, 255, 255 );
    } else {
      writeEllipse( -(wm->ball.getPosition()) - Vector2D(0.5,0.5),
		    1,1, level, 255, 255, 255 );
      writeLine( -(wm->ball.getPosition()), 
		 -(wm->ball.getPosition()) - wm->ball.getVelocity() * 3,
		 level, 255, 255, 255 );
    }
  }
}

void AgentPlanLogger::writeInterceptionPoint( APLLevel level, float conf ) {
  if(conf == -1) conf = ConfigFile::goodConfidenceBall;
  
  if ( wm->ball.getConfidence(wm->getTime()) >= conf ) {
    Vector2D intPoint;
    int noCycles;
    wm->interceptionPoint( *ms, intPoint, noCycles, ms->isGoalKeeper() );
    if ( wm->getFieldSide() == FS_Left ) 
      writePoint( intPoint, level, 0,0,255 );
    else 
      writePoint( -intPoint, level, 0,0,255 );      
  }
}

void AgentPlanLogger::writeAllPlayerPositions( APLLevel level, float conf ) {
  if(conf == -1) conf = ConfigFile::goodConfidencePlayer;

  vector<Player>::const_iterator i;
  for ( i = wm->theirTeam.getPlayers().begin(); i != wm->theirTeam.getPlayers().end(); ++i ) {
    if(i->getConfidence(wm->getTime()) < conf)
      continue;
    if ( wm->getFieldSide() == FS_Left )
      writeEllipse( i->getPosition() - Vector2D(0.75,0.75), 1.5, 1.5,
		    level, 255, 0, 0 );
    else 
      writeEllipse( -(i->getPosition()) - Vector2D(0.75,0.75), 1.5, 1.5,
		    level, 255, 255, 0 );
    
  }
  for ( i = wm->myTeam.getPlayers().begin(); i != wm->myTeam.getPlayers().end(); ++i ) {
    if(i->getConfidence(wm->getTime()) < conf)
      continue;
    if ( wm->getFieldSide() == FS_Left )
      writeEllipse( i->getPosition() - Vector2D(0.75,0.75), 1.5, 1.5,
		    level, 255, 255, 0 );
    else
      writeEllipse( -(i->getPosition()) - Vector2D(0.75,0.75), 1.5, 1.5,
		    level, 255, 0, 0 );
  }
}





void AgentPlanLogger::writeAgentPlan() {
    
    if (! out.is_open() ) return;
    Myself &ms = wm->myself;
    Vector2D myPos = ms.getPosition();
    int time = wm->getTime()->getTime();
    if(time != -1){
    
    // Write the actual position
    out << time 
	<< " " 
	<< uNum 
	<< " 8 ellipse (" 
	<< (myPos.getX() - 1) 
	<< "," 
	<< (myPos.getY() - 1) 
	<< ") 2 2"
	<< "\n";
		    
    // get the body direction
    float bodyDir = ms.getBodyDir();
    Vector2D p2 = myPos + Vector2D(1.5,bodyDir,POLAR);
    out << time 
	<< " " 
	<< uNum 
	<< " 8 line (" 
	<< myPos.getX() 
	<< "," 
	<< myPos.getY() 
	<< ") ("
	<< p2.getX()
	<< ","
	<< p2.getY()
	<< ")\n";

    // get Pass Partner
    // 0022 (Zyklus) 3 (PlayerUnum) 8 state 3(Arrow) 4(uNUM)
    //const Player* getPassCand(bool)(offensive)
    
    
    const Player* pc = wm->getPassCand(true);
    if(pc != NULL && pc->isKnownByUnum()){
      int pcUnum = pc->getUniformNumber();    
      out << time
	  << " "
	  << uNum
	  << " 8 state 3 "
	  << pcUnum
	  <<"\n";   
    }
    
    // get the head direction
    /*int headAngle = ms.getBodyInfo().getHeadAngle();
    Vector2D p3 = myPos + Vector2D(1, headAngle, POLAR);
    out << time
	<< " "
	<< uNum
	<< " 8 line ("
	<< myPos.getX()
	<< ","
	<< myPos.getY()
	<< ") ("
	<< p3.getX()
	<< ","
	<< p3.getY()
	<< ")\n";
	*/
    }
    out.flush();
}
