import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.QuadCurve2D;
import java.awt.geom.QuadCurve2D.Double;
import javax.swing.*;
import java.awt.*;

public class Field extends JPanel{
    final static double BALL_SIZE       = FormationEditor.BALL_SIZE;
    final static double KICKABLE_MARGIN = FormationEditor.KICKABLE_MARGIN;
    final static double PLAYER_SIZE     = FormationEditor.PLAYER_SIZE;
    final static double GOAL_WIDTH      = FormationEditor.GOAL_WIDTH;
    final static double GOAL_LENGTH     = FormationEditor.GOAL_LENGTH;
    final static double CIRCLE_SIZE     = FormationEditor.CIRCLE_SIZE;
    final static double PITCH_WIDTH     = FormationEditor.PITCH_WIDTH;
    final static double PITCH_LENGTH    = FormationEditor.PITCH_LENGTH;
    final static double PENALTY_AREA_LENGTH = FormationEditor.PENALTY_AREA_LENGTH;
    final static double PENALTY_AREA_WIDTH  = FormationEditor.PENALTY_AREA_WIDTH;
    final static double PENALTY_X = PITCH_LENGTH/2.0 - PENALTY_AREA_LENGTH;
    final static double PENALTY_Y = PITCH_WIDTH/2.0 - PENALTY_AREA_WIDTH;
    final static double PITCH_MARGIN = FormationEditor.PITCH_MARGIN;
    //final static double ZOOM = FormationEditor.ZOOM; //6;

    FormationEditor f;

    public void paintComponent(Graphics go){
	if (f == null) f = FormationEditor.instance; // easier access, shorter lines ;)

	Graphics2D g = (Graphics2D)go;
	if (FormationEditor.instance.antialias.getState() == true)
	    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
			       RenderingHints.VALUE_ANTIALIAS_ON);
	if (FormationEditor.instance.antialiasText.getState() == false)
	    g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, 
			       RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
	if(FormationEditor.instance.r_quality.isSelected()){
	    g.setRenderingHint(RenderingHints.KEY_RENDERING, 
			       RenderingHints.VALUE_RENDER_QUALITY);
	}else{
	    g.setRenderingHint(RenderingHints.KEY_RENDERING, 
			       RenderingHints.VALUE_RENDER_SPEED);
	}
	/*
	  g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, 
	  RenderingHints.VALUE_FRACTIONALMETRICS_ON);
	  g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, 
	  RenderingHints.VALUE_INTERPOLATION_BICUBIC);
	*/
	super.paintComponent(g);
	setBackground(new Color(0,70,70));
	g.translate((int)(PITCH_MARGIN+f.ZOOM*(PITCH_LENGTH/2)),
		    (int)(PITCH_MARGIN+f.ZOOM*(PITCH_WIDTH /2)));

	if (f.rotateView)
	    g.rotate(Math.PI,0,0);

	if (f.mode.equals("wm")){
	    if (f.activeWorldModel == null) return;
	    if (f.activeWorldModel.left == null) return;
	    Player p_1 = f.activeWorldModel.left.getPlayerWithUNum(f.activeWorldModelUNum);
	    // View cone
	    if (p_1 != null && f.pl == null){
		Vector2D p = p_1.position;
		int vc_dist[] = new int[]{60, 40, 20};
		Color vc_col[] = new Color[]{new Color(60,60,16),
					     new Color(80,80,18),
					     new Color(100,100,20)};
		for (int i = 0; i < 3; i++){
		    Polygon vw = new Polygon(new int[]{0,0,vc_dist[i]},
					     new int[]{0,vc_dist[i],0},3);
		    double rotation = -45 + p_1.headDir;
		    if (rotation < -180) rotation += 360;
		    if (rotation >  180) rotation -= 360;
		    rotation = Math.toRadians(rotation);
		    AffineTransform at = new AffineTransform();
		    at.scale(f.ZOOM, f.ZOOM);
		    at.translate(p.x, p.y);
		    at.rotate(rotation);
		    GeneralPath gp = new GeneralPath(vw);
		    gp.append(new QuadCurve2D.Double(vc_dist[i],0,vc_dist[i],
						     vc_dist[i],0,vc_dist[i]),false);
		    gp.transform(at);
		    g.setColor(vc_col[i]);
		    g.fill(gp);
		}
	    }
	    
	    // Offside-X
	    g.setColor(new Color(255, 0, 0));
	    float ox = f.activeWorldModel.offsideX;
	    g.drawLine((int)(ox * f.ZOOM), 
		       (int)(-100 * f.ZOOM), 
		       (int)(ox * f.ZOOM), 
		       (int)(100 * f.ZOOM));
	}

	if (f.al){
	    Font lf = g.getFont();
	    g.setColor(new Color(10,80,80));
	    g.setFont(new Font("Arial",Font.BOLD,150));
	    g.drawString("UR 1337", (int)-PITCH_LENGTH*3, 0);
	    g.setFont(lf);	    
	}
	// Pong
	if (f.p != null && f.p.isAlive()){
	    Font lf = g.getFont();
	    g.setColor(new Color(10,80,80));
	    g.setFont(new Font("Arial",Font.BOLD,150));
	    g.drawString(""+ f.p.pL, (int)(-PITCH_LENGTH/2/2*f.ZOOM), 0);
	    g.drawString(""+ f.p.pR, (int)( PITCH_LENGTH/4/2*f.ZOOM), 0);
	    g.setFont(lf);	    
	}

	if (f.showLimits.getState()){
	    double minX, maxX, minY;	    
	    minX = f.aF.player[f.activePlayer].minX;
	    maxX = f.aF.player[f.activePlayer].maxX;
	    g.setColor(new Color(20,90,90));
	    Vector2D tl, br;
	    tl = new Vector2D(-PITCH_LENGTH/2,-PITCH_WIDTH/2);
	    br = new Vector2D(+PITCH_LENGTH/2,+PITCH_WIDTH/2);
	    tl = f.aF.player[f.activePlayer].getRealPredictedPosition(tl);
	    br = f.aF.player[f.activePlayer].getRealPredictedPosition(br);
	    if (br.getX() > maxX) br.setX(maxX);
	    if (tl.getX() < minX) tl.setX(minX);
	    g.fillRect((int)(tl.getX()*f.ZOOM),
		       (int)(tl.getY()*f.ZOOM),
		       (int)((br.getX()-tl.getX())*f.ZOOM),
		       (int)((br.getY()-tl.getY())*f.ZOOM));
	}
	// Draw Grid
	double gr = f.gridSize;
	if (f.tgrid.getState() && gr > 0.05 && !f.mode.equals("ball")){
	    g.setColor(new Color(0,60,60));
	    double x1 = 0;
	    double y1 = -PITCH_WIDTH/2;
	    double x2 = 0;
	    double y2 =  PITCH_WIDTH/2;
	    while(x1 < PITCH_LENGTH/2){
		if(f.mode.equals("home"))
		    g.drawLine((int)(x1 * f.ZOOM),  (int)(y1 * f.ZOOM),
			       (int)(x2 * f.ZOOM),  (int)(y2 * f.ZOOM));
		g.drawLine((int)(-x1 * f.ZOOM), (int)(y1 * f.ZOOM),
			   (int)(-x2 * f.ZOOM), (int)(y2 * f.ZOOM));
		x1 += gr;
		x2 += gr;
	    }
	    x1 = -PITCH_LENGTH/2;
	    x2 = (f.mode.equals("init") ? 0 : PITCH_LENGTH/2);
	    y1 = 0;
	    y2 = 0;
	    while(y1 < PITCH_WIDTH/2){
		g.drawLine((int)(x1 * f.ZOOM), (int)(y1  * f.ZOOM),
			   (int)(x2 * f.ZOOM), (int)(y2  * f.ZOOM));
		g.drawLine((int)(x1 * f.ZOOM), (int)(-y1 * f.ZOOM),
			   (int)(x2 * f.ZOOM), (int)(-y2 * f.ZOOM));
		y1 += gr;
		y2 += gr;
	    }
	}
	// Goals
	g.setColor(Color.BLACK);
	if (f.p != null && f.p.isAlive()){
	    g.fill(f.p.humanPaddle);
	    g.fill(f.p.compPaddle);
	}else{
	    g.fillRect(-(int)(f.ZOOM * (PITCH_LENGTH/2 + GOAL_WIDTH)),
		       -(int)(f.ZOOM * (GOAL_LENGTH/2)),
		       (int)(f.ZOOM * (GOAL_WIDTH)),
		       (int)(f.ZOOM * (GOAL_LENGTH)));
	    
	    g.fillRect(+(int)(f.ZOOM * (PITCH_LENGTH/2)),
		       -(int)(f.ZOOM * (GOAL_LENGTH/2)),
		       (int)(f.ZOOM * (GOAL_WIDTH)),
		       (int)(f.ZOOM * (GOAL_LENGTH)));
	}
	// Lines
	g.setColor(Color.WHITE);
	g.drawRect(-(int)(f.ZOOM * (PITCH_LENGTH/2)),
		   -(int)(f.ZOOM * (PITCH_WIDTH /2)),
		   (int)(f.ZOOM * (PITCH_LENGTH)),
		   (int)(f.ZOOM * (PITCH_WIDTH )));
	g.drawRect(-(int)(f.ZOOM * (PITCH_LENGTH/2)),
		   -(int)(f.ZOOM * (PENALTY_AREA_WIDTH/2)),
		   (int)(f.ZOOM * (PENALTY_AREA_LENGTH)),
		   (int)(f.ZOOM * (PENALTY_AREA_WIDTH )));
	g.drawRect(+(int)(f.ZOOM * (PITCH_LENGTH/2       - PENALTY_AREA_LENGTH)),
		   +(int)(f.ZOOM * (PENALTY_AREA_WIDTH/2 - PENALTY_AREA_WIDTH )),
		   (int)(f.ZOOM * (PENALTY_AREA_LENGTH)),
		   (int)(f.ZOOM * (PENALTY_AREA_WIDTH )));
	g.drawLine(0,-(int)(f.ZOOM * (PITCH_WIDTH/2 )),
		   0,+(int)(f.ZOOM * (PITCH_WIDTH/2 )));
	g.drawOval(-(int)(f.ZOOM * (CIRCLE_SIZE)),
		   -(int)(f.ZOOM * (CIRCLE_SIZE)),
		   (int)(f.ZOOM * (CIRCLE_SIZE*2)),
		   (int)(f.ZOOM * (CIRCLE_SIZE*2)));
	g.fillOval(-(int)(f.ZOOM * (PITCH_LENGTH/2 - 10.75)),
		   -(int)(f.ZOOM * (0.25)),
		   (int)(f.ZOOM * (0.5)),
		   (int)(f.ZOOM * (0.5)));
	g.fillOval( (int)(f.ZOOM * (PITCH_LENGTH/2 - 11.25)),
		    -(int)(f.ZOOM * (0.25)),
		    (int)(f.ZOOM * (0.5)),
		    (int)(f.ZOOM * (0.5)));
	g.drawRect(+(int)(f.ZOOM * (PITCH_LENGTH/2       - 5.5)),
		   +(int)(f.ZOOM * (                     - 9.0)),
		   (int)(f.ZOOM * ( 5.5)),
		   (int)(f.ZOOM * (18)));
	g.drawRect(-(int)(f.ZOOM * (PITCH_LENGTH/2       )),
		   +(int)(f.ZOOM * ( -9)),
		   (int)(f.ZOOM * ( 5.5)),
		   (int)(f.ZOOM * (18)));


	// Player
	for (int i = 0; i < 11; i++){
	    Vector2D p = null, p2 = null;
	    double p1BD = 0, p1HD = 0, p2BD = 0, p2HD = 0;
	    Color p1Color, p2Color;
	    String s1 = ""+ (i+1), s2 = ""+ (i+1);
	    p1Color = Color.YELLOW;
	    p2Color = Color.GRAY;
	    if (f.mode.equals("wm")){
		Player p_1, p_2;
		//Player p_1 = f.activeWorldModel.left.getPlayerWithUNum(i+1);
		//Player p_2 = f.activeWorldModel.right.getPlayerWithUNum(i+1);
		if (f.activeWorldModel.left.player.size() > i)
		    p_1 = ((Player)(f.activeWorldModel.left.player.get(i)));
		else
		    p_1 = null;
		if (f.activeWorldModel.right.player.size() > i)
		    p_2 = ((Player)(f.activeWorldModel.right.player.get(i)));
		else p_2 = null;
		if (p_1 != null) {
		    p  = p_1.position;
		    if (p_1.unum > 11)
			s1 = "??";
		    else
			s1 = ""+ p_1.unum;
		    p1BD = p_1.bodyDir;
		    p1HD = p_1.headDir;
		    if (p_1.position.confidence < 0.97)
			p1Color = p1Color.darker();
		    if (p_1.position.confidence < 0.90)
			p1Color = p1Color.darker();
		}
		if (p_2 != null) {
		    p2 = p_2.position;
		    if (p_2.unum > 11) 
			s2 = "??";
		    else
			s2 = ""+ p_2.unum;
		    p2BD = p_2.bodyDir;
		    p2HD = p_2.headDir;
		    if (p_2.position.confidence < 0.97)
			p2Color = p2Color.darker();
		    if (p_2.position.confidence < 0.90)
			p2Color = p2Color.darker();
		}
	    }else{
		if      (f.mode.equals("ball")) p = f.aF.player[i].getRealPredictedPosition(f.ballPos);
		else if (f.mode.equals("init")) p = f.aF.player[i].initPosition;
		else if (f.mode.equals("home")) p = f.aF.player[i].homePosition;
		else continue;
		
		if (f.sF != null){
		    if      (f.mode.equals("ball")) p2 = f.aF.player[i].getRevertedRealPredictedPosition(f.ballPos);
		    else if (f.mode.equals("init")) p2 = f.sF.player[i].initPosition.revert();
		    else if (f.mode.equals("home")) p2 = f.sF.player[i].homePosition.revert();
		    else p2 = null;
		}else{ p2 = null; }

	    }

	    if (f.showLimits.getState() && i == f.activePlayer){
		g.setColor(Color.RED);
		g.drawLine((int)(f.aF.player[i].maxX * f.ZOOM), (int)(f.ZOOM * -PITCH_WIDTH/2),
			   (int)(f.aF.player[i].maxX * f.ZOOM), (int)(f.ZOOM *  PITCH_WIDTH/2));
		g.drawString("maxX",
			     (int)(f.aF.player[i].maxX * f.ZOOM) - 10,
			     (int)((-PITCH_WIDTH/2)*f.ZOOM) - 1);
		g.drawString("minX",
			     (int)(f.aF.player[i].minX * f.ZOOM) - 10,
			     (int)((-PITCH_WIDTH/2)*f.ZOOM) - 1);
		g.drawLine((int)(f.aF.player[i].minX * f.ZOOM), (int)(f.ZOOM * -PITCH_WIDTH/2),
			   (int)(f.aF.player[i].minX * f.ZOOM), (int)(f.ZOOM *  PITCH_WIDTH/2));
	    }

	    if (p2 != null){
		g.setColor(p2Color);
		g.fillOval((int)(p2.getX() * f.ZOOM - (PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM), 
			   (int)(p2.getY() * f.ZOOM - (PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM), 
			   (int)((PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM*2), 
			   (int)((PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM*2));
		g.setColor(Color.BLACK);
		g.drawOval((int)(p2.getX() * f.ZOOM - (PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM), 
			   (int)(p2.getY() * f.ZOOM - (PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM), 
			   (int)((PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM*2), 
			   (int)((PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM*2));
		if (f.mode.equals("wm")){
		    g.setColor(Color.WHITE);
		    g.drawString(s2,
				 (int)(p2.getX() * f.ZOOM - (PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM - 1),
				 (int)(p2.getY() * f.ZOOM - (PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM - 1));
		    g.setColor(Color.BLACK);
		    g.drawLine((int)(p2.getX() * f.ZOOM), (int)(p2.getY() * f.ZOOM),
			       (int)((p2.getX()+ Math.cos(Math.toRadians(p2BD))*(PLAYER_SIZE+KICKABLE_MARGIN)) * f.ZOOM), 
			       (int)((p2.getY()+ Math.sin(Math.toRadians(p2BD))*(PLAYER_SIZE+KICKABLE_MARGIN)) * f.ZOOM));
		}
	    }

	    if (p != null){
		if (f.activeWorldModel != null && f.activeWorldModelUNum == (i+1)) g.setColor(Color.RED);
		else if (f.activeWorldModel == null && i == f.activePlayer) g.setColor(Color.RED);	       
		else                   g.setColor(p1Color);
		g.fillOval((int)(p.getX() * f.ZOOM - (PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM), 
			   (int)(p.getY() * f.ZOOM - (PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM), 
			   (int)((PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM*2), 
			   (int)((PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM*2));
		g.setColor(Color.BLACK);
		g.drawOval((int)(p.getX() * f.ZOOM - (PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM), 
			   (int)(p.getY() * f.ZOOM - (PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM), 
			   (int)((PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM*2), 
			   (int)((PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM*2));
		
		g.setColor(Color.WHITE);
		g.drawString(s1,
			     (int)(p.getX() * f.ZOOM - (PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM - 1),
			     (int)(p.getY() * f.ZOOM - (PLAYER_SIZE+KICKABLE_MARGIN)*f.ZOOM - 1));
		if (f.mode.equals("wm")){
		    g.setColor(Color.BLACK);
		    g.drawLine((int)(p.getX() * f.ZOOM), (int)(p.getY() * f.ZOOM),
			       (int)((p.getX()+ Math.cos(Math.toRadians(p1BD))*(PLAYER_SIZE+KICKABLE_MARGIN)) * f.ZOOM), 
			       (int)((p.getY()+ Math.sin(Math.toRadians(p1BD))*(PLAYER_SIZE+KICKABLE_MARGIN)) * f.ZOOM));
		}
	    }
	}
	Vector2D b = null;
	if (f.mode.equals("ball")){ b = f.ballPos; }
	if (f.mode.equals("wm")){ 
	    Ball b_1 = f.activeWorldModel.ball;
	    if (b_1 != null) b = b_1.position;
	}
	if (b != null){
	    g.setColor(Color.WHITE);
	    g.fillOval((int)((b.getX() - BALL_SIZE) * f.ZOOM),
		       (int)((b.getY() - BALL_SIZE) * f.ZOOM),
		       (int)((BALL_SIZE*2*f.ZOOM)),
		       (int)((BALL_SIZE*2*f.ZOOM)));
	}	
   }
}
