/*****************************************************************************
 * Parser.cc
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   class Parser
 *
 * requires:
 *
 * Created on: 29.08.2002 by wenns
 * Maintainer: wenns
 * Group:
 * Co-authors:
 *
 *
 * $Id: Parser.cc,v 1.7 2004/05/13 16:28:24 aw Exp $
 *****************************************************************************/

#include "Parser.hh"
#include "debug.hh"
#include "Configuration.hh"

Parser::Parser(){
  newVisual_synch    = false;
  newFullstate_synch = false;
  newSenseBody_synch = false;
  newInitMsg_synch   = false;
  
  newVisual        = false;
  newFullstate     = false;
  newSenseBody     = false;
  newRefAudio      = false;
  newSelfAudio     = false;
  newPlayerAudio   = false;
  newCoachAudio    = false;
  newScore         = false;
  newSubstitution  = false;
  newError         = false;
  newInitMsg       = false;
  newServerParam   = false;
  newPlayerParam   = false;
  newPlayerType    = false;
  newWarning       = false;
  newCoachFreeform = false;

  complexLocation = 0;
}


Parser::~Parser(){
}


void Parser::setTeamname(const char *tn){
  strcpy(teamname, tn);
}


void Parser::buildLeftSide()  { side = Sense::S_Left; }
void Parser::buildRightSide() { side = Sense::S_Right; }
void Parser::buildOurSide()   { side = Sense::S_Our; }
void Parser::buildOppSide()   { side = Sense::S_Opp; }

//playmodes
void Parser::buildBeforeKickOffPMode() { pmode = PM_BeforeKickOff; }
void Parser::buildTimeOverPMode()      { pmode = PM_TimeOver; }
void Parser::buildPlayOnPMode()        { pmode = PM_PlayOn; }
void Parser::buildKickOffLPMode()
{ fside == FS_Left ? pmode = PM_OurKickOff : pmode = PM_TheirKickOff; }
void Parser::buildKickOffRPMode()
{ fside == FS_Right ? pmode = PM_OurKickOff : pmode = PM_TheirKickOff; }
void Parser::buildKickInLPMode()
{ fside == FS_Left ? pmode = PM_OurKickIn : pmode = PM_TheirKickIn; }
void Parser::buildKickInRPMode()
{ fside == FS_Right ? pmode = PM_OurKickIn : pmode = PM_TheirKickIn; }
void Parser::buildFreeKickLPMode()
{ fside == FS_Left ? pmode = PM_OurFreeKick : pmode = PM_TheirFreeKick; }
void Parser::buildFreeKickRPMode()
{ fside == FS_Right ? pmode = PM_OurFreeKick : pmode = PM_TheirFreeKick; }
void Parser::buildCornerKickLPMode()
{ fside == FS_Left ? pmode = PM_OurCornerKick : pmode = PM_TheirCornerKick; }
void Parser::buildCornerKickRPMode()
{ fside == FS_Right ? pmode = PM_OurCornerKick : pmode = PM_TheirCornerKick; }
void Parser::buildGoalKickLPMode()
{ fside == FS_Left ? pmode = PM_OurGoalKick : pmode = PM_TheirGoalKick; }
void Parser::buildGoalKickRPMode()
{ fside == FS_Right ? pmode = PM_OurGoalKick : pmode = PM_TheirGoalKick; }

void Parser::buildGoalLPMode(){
  fside == FS_Left ? pmode = PM_OurGoal : pmode = PM_TheirGoal;
}
void Parser::buildGoalRPMode(){
  fside == FS_Right ? pmode = PM_OurGoal : pmode = PM_TheirGoal;
}
void Parser::buildGoalLPMode( const int goals ){
  fside == FS_Left ? pmode = PM_OurGoal : pmode = PM_TheirGoal;
  refAudio.setGoals(goals);
}
void Parser::buildGoalRPMode( const int goals ){
  fside == FS_Right ? pmode = PM_OurGoal : pmode = PM_TheirGoal;
  refAudio.setGoals(goals);
}
void Parser::buildDropBallPMode() { pmode = PM_DropBall; }
void Parser::buildOffsideLPMode()
{ fside == FS_Left ? pmode = PM_OurOffside : pmode = PM_TheirOffside; }
void Parser::buildOffsideRPMode()
{ fside == FS_Right ? pmode = PM_OurOffside : pmode = PM_TheirOffside; }
void Parser::buildPenaltyKickLPMode()
{ fside == FS_Left ? pmode = PM_OurPenaltyKick : pmode = PM_TheirPenaltyKick; }
void Parser::buildPenaltyKickRPMode()
{ fside == FS_Right ? pmode = PM_OurPenaltyKick : pmode = PM_TheirPenaltyKick; }
void Parser::buildPausePMode() { pmode = PM_Pause; }
void Parser::buildHumanJudgePMode() { pmode = PM_HumanJudge; }
void Parser::buildFoulLPMode()
{ fside == FS_Left ? pmode = PM_OurFoul : pmode = PM_TheirFoul; }
void Parser::buildFoulRPMode()
{ fside == FS_Right ? pmode = PM_OurFoul : pmode = PM_TheirFoul; }
void Parser::buildFoulChargeLPMode()
{ fside == FS_Left ? pmode = PM_OurFoulCharge : pmode = PM_TheirFoulCharge; }
void Parser::buildFoulChargeRPMode()
{ fside == FS_Right ? pmode = PM_OurFoulCharge : pmode = PM_TheirFoulCharge; }
void Parser::buildFoulPushLPMode()
{ fside == FS_Left ? pmode = PM_OurFoulPush : pmode = PM_TheirFoulPush; }
void Parser::buildFoulPushRPMode()
{ fside == FS_Right ? pmode = PM_OurFoulPush : pmode = PM_TheirFoulPush; }
void Parser::buildFoulMultipleAttackLPMode()
{ fside == FS_Left ? pmode = PM_OurFoulMultipleAttack : pmode = PM_TheirFoulMultipleAttack; }
void Parser::buildFoulMultipleAttackRPMode()
{ fside == FS_Right ? pmode = PM_OurFoulMultipleAttack : pmode = PM_TheirFoulMultipleAttack; }
void Parser::buildFoulBallOutLPMode()
{ fside == FS_Left ? pmode = PM_OurFoulBallout : pmode = PM_TheirFoulBallout; }
void Parser::buildFoulBallOutRPMode()
{ fside == FS_Right ? pmode = PM_OurFoulBallout : pmode = PM_TheirFoulBallout; }
void Parser::buildBackPassLPMode()
{ fside == FS_Left ? pmode = PM_OurBackPass : pmode = PM_TheirBackPass; }
void Parser::buildBackPassRPMode()
{ fside == FS_Right ? pmode = PM_OurBackPass : pmode = PM_TheirBackPass; }
void Parser::buildFreeKickFaultLPMode()
{ fside == FS_Left ? pmode = PM_OurFreeKickFault : pmode = PM_TheirFreeKickFault; }
void Parser::buildFreeKickFaultRPMode()
{ fside == FS_Right ? pmode = PM_OurFreeKickFault : pmode = PM_TheirFreeKickFault; }
void Parser::buildGoalieCatchBallLPMode()
{ fside == FS_Left ? pmode = PM_OurGoalieCatchBall : pmode = PM_TheirGoalieCatchBall; }
void Parser::buildGoalieCatchBallRPMode()
{ fside == FS_Right ? pmode = PM_OurGoalieCatchBall : pmode = PM_TheirGoalieCatchBall; }
void Parser::buildHalfTimePMode() { pmode = PM_HalfTime; }
void Parser::buildTimeUpRPMode()  { pmode = PM_TimeUp;   }
void Parser::buildIndirectFreeKickLPMode()
{ fside == FS_Left ? pmode = PM_OurIndirectFreeKick : pmode = PM_TheirIndirectFreeKick; }
void Parser::buildIndirectFreeKickRPMode()
{ fside == FS_Right ? pmode = PM_OurIndirectFreeKick : pmode = PM_TheirIndirectFreeKick; }

void Parser::buildPenaltySetupLPMode()
{ fside == FS_Left ? pmode = PM_OurPenaltySetup : pmode = PM_TheirPenaltySetup; }
void Parser::buildPenaltySetupRPMode()
{ fside == FS_Right ? pmode = PM_OurPenaltySetup : pmode = PM_TheirPenaltySetup; }
void Parser::buildPenaltyReadyLPMode()
{ fside == FS_Left ? pmode = PM_OurPenaltyReady : pmode = PM_TheirPenaltyReady; }
void Parser::buildPenaltyReadyRPMode()
{ fside == FS_Right ? pmode = PM_OurPenaltyReady : pmode = PM_TheirPenaltyReady; }
void Parser::buildPenaltyTakenLPMode()
{ fside == FS_Left ? pmode = PM_OurPenaltyTaken : pmode = PM_TheirPenaltyTaken; }
void Parser::buildPenaltyTakenRPMode()
{ fside == FS_Right ? pmode = PM_OurPenaltyTaken : pmode = PM_TheirPenaltyTaken; }
void Parser::buildPenaltyMissLPMode()
{ fside == FS_Left ? pmode = PM_OurPenaltyMiss : pmode = PM_TheirPenaltyMiss; }
void Parser::buildPenaltyMissRPMode()
{ fside == FS_Right ? pmode = PM_OurPenaltyMiss : pmode = PM_TheirPenaltyMiss; }
void Parser::buildPenaltyScoreLPMode()
{ fside == FS_Left ? pmode = PM_OurPenaltyScore : pmode = PM_TheirPenaltyScore; }
void Parser::buildPenaltyScoreRPMode()
{ fside == FS_Right ? pmode = PM_OurPenaltyScore : pmode = PM_TheirPenaltyScore; }
void Parser::buildPenaltySideL()
{ fside == FS_Left ? pside = PS_OurGoal : pside = PS_TheirGoal; }
void Parser::buildPenaltySideR()
{ fside == FS_Right ? pside = PS_OurGoal : pside = PS_TheirGoal; }


void Parser::buildViewQualityHigh() { vqual = VQ_HIGH; }
void Parser::buildViewQualityLow()  { vqual = VQ_LOW; }

void Parser::buildViewWidthNarrow() { vwidth = VW_NARROW; }
void Parser::buildViewWidthNormal() { vwidth = VW_NORMAL; }
void Parser::buildViewWidthWide()   { vwidth = VW_WIDE; }

void Parser::buildArmInfo( const int movable, const int expires,
                           const float targetDist, const float targetDir,
                           const int count) {
  armInfo.setMovable(movable);
  armInfo.setExpires(expires);
  armInfo.setTarget(targetDist, targetDir);
  armInfo.setCount(count);
}


//----------------------------- init ----------------------------------
//uses also side and playmode objects
void Parser::buildInit( const int unum ) {
  fside = (FieldSide)side;  //needed later (e.g. by playmodes)

  initMsg.setSide(side);
  initMsg.setPlayMode(pmode);
  initMsg.setUnum(unum);
  newInitMsg = true;
  newInitMsg_synch = true;
}


//----------------------------- server param ----------------------------
void Parser::buildServerParam(){ newServerParam = true; }
void Parser::buildServerParam(const ServerParam sp, const int    val){
  switch(sp){
  case SP_CatchBanCycle: 
    serverParam.setCatchBanCycle(val);
    break;
  case SP_CLangAdviceWin: 
    serverParam.setCLangAdviceWin(val);
    break;
  case SP_CLangDefineWin: 
    serverParam.setCLangDefineWin(val);
    break;
  case SP_CLangDelWin: 
    serverParam.setCLangDelWin(val);
    break;
  case SP_CLangInfoWin: 
    serverParam.setCLangInfoWin(val);
    break;
  case SP_CLangMessDelay: 
    serverParam.setCLangMessDelay(val);
    break;
  case SP_CLangMessPerCycle: 
    serverParam.setCLangMessPerCycle(val);
    break;
  case SP_CLangMetaWin: 
    serverParam.setCLangMetaWin(val);
    break;
  case SP_CLangRuleWin: 
    serverParam.setCLangRuleWin(val);
    break;
  case SP_CLangWinSize: 
    serverParam.setCLangWinSize(val);
    break;
  case SP_CoachPort: 
    serverParam.setCoachPort(val);
    break;
  case SP_DropBallTime: 
    serverParam.setDropBallTime(val);
    break;
  case SP_FreeformSendPeriod: 
    serverParam.setFreeformSendPeriod(val);
    break;
  case SP_FreeformWaitPeriod: 
    serverParam.setFreeformWaitPeriod(val);
    break;
  case SP_GameLogCompression: 
    serverParam.setGameLogCompression(val);
    break;
  case SP_GameLogVersion: 
    serverParam.setGameLogVersion(val);
    break;
  case SP_GoalieMaxMoves: 
    serverParam.setGoalieMaxMoves(val);
    break;
  case SP_HalfTime: 
    serverParam.setHalfTime(val);
    break;
  case SP_HearDecay: 
    serverParam.setHearDecay(val);
    break;
  case SP_HearInc: 
    serverParam.setHearInc(val);
    break;
  case SP_HearMax: 
    serverParam.setHearMax(val);
    break;
  case SP_MaxGoalKicks: 
    serverParam.setMaxGoalKicks(val);
    break;
  case SP_OLCoachPort: 
    serverParam.setOLCoachPort(val);
    break;
  case SP_PointToBan: 
    serverParam.setPointToBan(val);
    break;
  case SP_PointToDuration: 
    serverParam.setPointToDuration(val);
    break;
  case SP_Port: 
    serverParam.setPort(val);
    break;
  case SP_RecvStep: 
    serverParam.setRecvStep(val);
    break;
  case SP_SayCoachCntMax: 
    serverParam.setSayCoachCntMax(val);
    break;
  case SP_SayCoachMsgSize: 
    serverParam.setSayCoachMsgSize(val);
    break;
  case SP_SayMsgSize: 
    serverParam.setSayMsgSize(val);
    break;
  case SP_SendStep: 
    serverParam.setSendStep(val);
    break;
  case SP_SendVIStep: 
    serverParam.setSendVIStep(val);
    break;
  case SP_SenseBodyStep: 
    serverParam.setSenseBodyStep(val);
    break;
  case SP_SimulatorStep: 
    serverParam.setSimulatorStep(val);
    break;
  case SP_SlowDownFactor: 
    serverParam.setSlowDownFactor(val);
    break;
  case SP_StartGoalL: 
    serverParam.setStartGoalL(val);
    break;
  case SP_StartGoalR: 
    serverParam.setStartGoalR(val);
    break;
  case SP_SynchMicroSleep: 
    serverParam.setSynchMicroSleep(val);
    break;
  case SP_SynchOffset: 
    serverParam.setSynchOffset(val);
    break;
  case SP_TackleCycles: 
    serverParam.setTackleCycles(val);
    break;
  case SP_TextLogCompression: 
    serverParam.setTextLogCompression(val);
    break;
    // Bools, needed here because they are send as 1 and 0, not as true and false 
  case SP_Coach:
    serverParam.setCoach(val);
    break;
  case SP_CoachWReferee:
    serverParam.setCoachWReferee(val);
    break;
  case SP_OldCoachHear:
    serverParam.setOldCoachHear(val);
    break;
  case SP_WindNone:
    serverParam.setWindNone(val);
    break;
  case SP_WindRandom:
    serverParam.setWindRandom(val);
    break;
  case SP_BackPasses:
    serverParam.setBackPasses(val);
    break;
  case SP_ForbidKickOffOffside:
    serverParam.setForbidKickOffOffside(val);
    break;
  case SP_FreeKickFaults:
    serverParam.setFreeKickFaults(val);
    break;
  case SP_FullstateL:
    serverParam.setFullstateL(val);
    break;
  case SP_FullstateR:
    serverParam.setFullstateR(val);
    break;
  case SP_GameLogDated:
    serverParam.setGameLogDated(val);
    break;
  case SP_GameLogFixed:
    serverParam.setGameLogFixed(val);
    break;
  case SP_GameLogging:
    serverParam.setGameLogging(val);
    break;
  case SP_LogTimes:
    serverParam.setLogTimes(val);
    break;
  case SP_Profile:
    serverParam.setProfile(val);
    break;
  case SP_ProperGoalKicks:
    serverParam.setProperGoalKicks(val);
    break;
  case SP_RecordMessages:
    serverParam.setRecordMessages(val);
    break;
  case SP_SendComms:
    serverParam.setSendComms(val);
    break;
  case SP_SynchMode:
    serverParam.setSynchMode(val);
    break;
  case SP_TeamActuatorNoise:
    serverParam.setTeamActuatorNoise(val);
    break;
  case SP_TextLogDated:
    serverParam.setTextLogDated(val);
    break;
  case SP_TextLogFixed:
    serverParam.setTextLogFixed(val);
    break;
  case SP_TextLogging:
    serverParam.setTextLogging(val);
    break;
  case SP_UseOffside:
    serverParam.setUseOffside(val);
    break;
  case SP_Verbose:
    serverParam.setVerbose(val);
    break;


  default:
    DEBUG5("Parser::buildServerParam(", sp, ",int ", val, "): UNKNOWN!");
    break;
  }
}
void Parser::buildServerParam(const ServerParam sp, const double val){
  switch(sp){
  case SP_AudioCutDist: 
    serverParam.setAudioCutDist(val);
    break;
  case SP_BallAccelMax: 
    serverParam.setBallAccelMax(val);
    break;
  case SP_BallDecay: 
    serverParam.setBallDecay(val);
    break;
  case SP_BallRand: 
    serverParam.setBallRand(val);
    break;
  case SP_BallSize: 
    serverParam.setBallSize(val);
    break;
  case SP_BallSpeedMax: 
    serverParam.setBallSpeedMax(val);
    break;
  case SP_BallWeight: 
    serverParam.setBallWeight(val);
    break;
  case SP_CatchProbability: 
    serverParam.setCatchProbability(val);
    break;
  case SP_CatchableAreaL: 
    serverParam.setCatchableAreaL(val);
    break;
  case SP_CatchableAreaW: 
    serverParam.setCatchableAreaW(val);
    break;
  case SP_CKickMargin: 
    serverParam.setCKickMargin(val);
    break;
  case SP_ControlRadius: 
    serverParam.setControlRadius(val);
    break;
  case SP_DashPowerRate: 
    serverParam.setDashPowerRate(val);
    break;
  case SP_EffortDec: 
    serverParam.setEffortDec(val);
    break;
  case SP_EffortDecThr: 
    serverParam.setEffortDecThr(val);
    break;
  case SP_EffortInc: 
    serverParam.setEffortInc(val);
    break;
  case SP_EffortIncThr: 
    serverParam.setEffortIncThr(val);
    break;
  case SP_EffortInit: 
    serverParam.setEffortInit(val);
    break;
  case SP_EffortMin: 
    serverParam.setEffortMin(val);
    break;
  case SP_GoalWidth: 
    serverParam.setGoalWidth(val);
    break;
  case SP_InertiaMoment: 
    serverParam.setInertiaMoment(val);
    break;
  case SP_KickPowerRate: 
    serverParam.setKickPowerRate(val);
    break;
  case SP_KickRand: 
    serverParam.setKickRand(val);
    break;
  case SP_KickRandFactorL: 
    serverParam.setKickRandFactorL(val);
    break;
  case SP_KickRandFactorR: 
    serverParam.setKickRandFactorR(val);
    break;
  case SP_KickableMargin: 
    serverParam.setKickableMargin(val);
    break;
  case SP_MaxMoment:
    serverParam.setMaxMoment(val);
    break;
  case SP_MaxNeckAng: 
    serverParam.setMaxNeckAng(val);
    break;
  case SP_MaxNeckMoment: 
    serverParam.setMaxNeckMoment(val);
    break;
  case SP_MaxPower: 
    serverParam.setMaxPower(val);
    break;
  case SP_MinMoment: 
    serverParam.setMinMoment(val);
    break;
  case SP_MinNeckAng: 
    serverParam.setMinNeckAng(val);
    break;
  case SP_MinNeckMoment: 
    serverParam.setMinNeckMoment(val);
    break;
  case SP_MinPower: 
    serverParam.setMinPower(val);
    break;
  case SP_OffsideActiveAreaSize: 
    serverParam.setOffsideActiveAreaSize(val);
    break;
  case SP_OffsideKickMargin: 
    serverParam.setOffsideKickMargin(val);
    break;
  case SP_PlayerAccelMax: 
    serverParam.setPlayerAccelMax(val);
    break;
  case SP_PlayerDecay: 
    serverParam.setPlayerDecay(val);
    break;
  case SP_PlayerRand: 
    serverParam.setPlayerRand(val);
    break;
  case SP_PlayerSize: 
    serverParam.setPlayerSize(val);
    break;
  case SP_PlayerSpeedMax: 
    serverParam.setPlayerSpeedMax(val);
    break;
  case SP_PlayerWeight: 
    serverParam.setPlayerWeight(val);
    break;
  case SP_PRandFactorL: 
    serverParam.setPRandFactorL(val);
    break;
  case SP_PRandFactorR: 
    serverParam.setPRandFactorR(val);
    break;
  case SP_QuantizeStep: 
    serverParam.setQuantizeStep(val);
    break;
  case SP_QuantizeStepL: 
    serverParam.setQuantizeStepL(val);
    break;
  case SP_RecoverDec: 
    serverParam.setRecoverDec(val);
    break;
  case SP_RecoverDecThr: 
    serverParam.setRecoverDecThr(val);
    break;
  case SP_RecoverMin: 
    serverParam.setRecoverMin(val);
    break;
  case SP_SlownessOnTopForLeftTeam: 
    serverParam.setSlownessOnTopForLeftTeam(val);
    break;
  case SP_SlownessOnTopForRightTeam: 
    serverParam.setSlownessOnTopForRightTeam(val);
    break;
  case SP_StaminaIncMax: 
    serverParam.setStaminaIncMax(val);
    break;
  case SP_StaminaMax: 
    serverParam.setStaminaMax(val);
    break;
  case SP_StoppedBallVel: 
    serverParam.setStoppedBallVel(val);
    break;
  case SP_TackleBackDist: 
    serverParam.setTackleBackDist(val);
    break;
  case SP_TackleDist: 
    serverParam.setTackleDist(val);
    break;
  case SP_TackleExponent: 
    serverParam.setTackleExponent(val);
    break;
  case SP_TacklePowerRate: 
    serverParam.setTacklePowerRate(val);
    break;
  case SP_TackleWidth: 
    serverParam.setTackleWidth(val);
    break;
  case SP_TeamActuatorNoise:
    serverParam.setTeamActuatorNoise(val);
    break;
  case SP_VisibleAngle: 
    serverParam.setVisibleAngle(val);
    break;
  case SP_VisibleDistance: 
    serverParam.setVisibleDistance(val);
    break;
  case SP_WindAng: 
    serverParam.setWindAng(val);
    break;
  case SP_WindDir: 
    serverParam.setWindDir(val);
    break;
  case SP_WindForce: 
    serverParam.setWindForce(val);
    break;
  case SP_WindRand: 
    serverParam.setWindRand(val);
    break;
  case SP_WindRandom:
    serverParam.setWindRandom(val);
    break;
  default:
    DEBUG5("Parser::buildServerParam(", sp, ",double ", val, "): UNKNOWN!");
    break;
  }  
}
// Problem: Server sends rather 1 and 0 than true and false
void Parser::buildServerParam(const ServerParam sp, const bool   val){
  switch(sp){
  case SP_Coach:
    serverParam.setCoach(val);
    break;
  case SP_CoachWReferee:
    serverParam.setCoachWReferee(val);
    break;
  case SP_OldCoachHear:
    serverParam.setOldCoachHear(val);
    break;
  case SP_WindNone:
    serverParam.setWindNone(val);
    break;
  case SP_WindRandom:
    serverParam.setWindRandom(val);
    break;
  case SP_BackPasses:
    serverParam.setBackPasses(val);
    break;
  case SP_ForbidKickOffOffside:
    serverParam.setForbidKickOffOffside(val);
    break;
  case SP_FreeKickFaults:
    serverParam.setFreeKickFaults(val);
    break;
  case SP_FullstateL:
    serverParam.setFullstateL(val);
    break;
  case SP_FullstateR:
    serverParam.setFullstateR(val);
    break;
  case SP_GameLogDated:
    serverParam.setGameLogDated(val);
    break;
  case SP_GameLogFixed:
    serverParam.setGameLogFixed(val);
    break;
  case SP_GameLogging:
    serverParam.setGameLogging(val);
    break;
  case SP_LogTimes:
    serverParam.setLogTimes(val);
    break;
  case SP_Profile:
    serverParam.setProfile(val);
    break;
  case SP_ProperGoalKicks:
    serverParam.setProperGoalKicks(val);
    break;
  case SP_RecordMessages:
    serverParam.setRecordMessages(val);
    break;
  case SP_SendComms:
    serverParam.setSendComms(val);
    break;
  case SP_SynchMode:
    serverParam.setSynchMode(val);
    break;
  case SP_TeamActuatorNoise:
    serverParam.setTeamActuatorNoise(val);
    break;
  case SP_TextLogDated:
    serverParam.setTextLogDated(val);
    break;
  case SP_TextLogFixed:
    serverParam.setTextLogFixed(val);
    break;
  case SP_TextLogging:
    serverParam.setTextLogging(val);
    break;
  case SP_UseOffside:
    serverParam.setUseOffside(val);
    break;
  case SP_Verbose:
    serverParam.setVerbose(val);
    break;
  default:
    DEBUG5("Parser::buildServerParam(", sp, ",bool ", val, "): UNKNOWN!");
    break;
  }
}
void Parser::buildServerParam(const ServerParam sp, const char  *val){
  switch(sp){
  case SP_GameLogDir:
    serverParam.setGameLogDir(val);
    break;
  case SP_GameLogFixedName:
    serverParam.setGameLogFixedName(val);
    break;
  case SP_LandmarkFile:
    serverParam.setLandmarkFile(val);
    break;
  case SP_LogDateFormat:
    serverParam.setLogDateFormat(val);
    break;
  case SP_TextLogDir:
    serverParam.setTextLogDir(val);
    break;
  case SP_TextLogFixedName:
    serverParam.setTextLogFixedName(val);
    break;
  default:
    DEBUG5("Parser::buildServerParam(", sp, ",char* ", val, "): UNKNOWN!");
    break;
  }
}
void Parser::buildServerParam(
                              const int catch_ban_cycle,
                              const int clang_advice_win,
                              const int clang_define_win,
                              const int clang_del_win,
                              const int clang_info_win,
                              const int clang_mess_delay,
                              const int clang_mess_per_cycle,
                              const int clang_meta_win,
                              const int clang_rule_win,
                              const int clang_win_size,
                              const int coach_port,
                              const int drop_ball_time,
                              const int freeform_send_period,
                              const int freeform_wait_period,
                              const int game_log_compression,
                              const int game_log_version,
                              const int goalie_max_moves,
                              const int half_time,
                              const int hear_decay,
                              const int hear_inc,
                              const int hear_max,
                              const int max_goal_kicks,
                              const int olcoach_port,
                              const int point_to_ban,
                              const int point_to_duration,
                              const int port,
                              const int recv_step,
                              const int say_coach_cnt_max,
                              const int say_coach_msg_size,
                              const int say_msg_size,
                              const int send_step,
                              const int send_vi_step,
                              const int sense_body_step,
                              const int simulator_step,
                              const int slow_down_factor,
                              const int start_goal_l,
                              const int start_goal_r,
                              const int synch_micro_sleep,
                              const int synch_offset,
                              const int tackle_cycles,
                              const int text_log_compression,
                              const char *game_log_dir,
                              const char *game_log_fixed_name,
                              const char *landmark_file,
                              const char *log_date_format,
                              const char *text_log_dir,
                              const char *text_log_fixed_name,
                              const bool coach,
                              const bool coach_w_referee,
                              const bool old_coach_hear,
                              const bool wind_none,
                              const float wind_random,
                              const bool back_passes,
                              const bool forbid_kick_off_offside,
                              const bool free_kick_faults,
                              const bool fullstate_l,
                              const bool fullstate_r,
                              const bool game_log_dated,
                              const bool game_log_fixed,
                              const bool game_logging,
                              const bool log_times,
                              const bool profile,
                              const bool proper_goal_kicks,
                              const bool record_messages,
                              const bool send_comms,
                              const bool synch_mode,
                              const bool team_actuator_noise,
                              const bool text_log_dated,
                              const bool text_log_fixed,
                              const bool text_logging,
                              const bool use_offside,
                              const bool verbose,
                              const float audio_cut_dist,
                              const float ball_accel_max,
                              const float ball_decay,
                              const float ball_rand,
                              const float ball_size,
                              const float ball_speed_max,
                              const float ball_weight,
                              const float catch_probability,
                              const float catchable_area_l,
                              const float catchable_area_w,
                              const float ckick_margin,
                              const float control_radius,
                              const float dash_power_rate,
                              const float effort_dec,
                              const float effort_dec_thr,
                              const float effort_inc,
                              const float effort_inc_thr,
                              const float effort_init,
                              const float effort_min,
                              const float goal_width,
                              const float inertia_moment,
                              const float kick_power_rate,
                              const float kick_rand,
                              const float kick_rand_factor_l,
                              const float kick_rand_factor_r,
                              const float kickable_margin,
                              const float maxmoment,
                              const float maxneckang,
                              const float maxneckmoment,
                              const float maxpower,
                              const float minmoment,
                              const float minneckang,
                              const float minneckmoment,
                              const float minpower,
                              const float offside_active_area_size,
                              const float offside_kick_margin,
                              const float player_accel_max,
                              const float player_decay,
                              const float player_rand,
                              const float player_size,
                              const float player_speed_max,
                              const float player_weight,
                              const float prand_factor_l,
                              const float prand_factor_r,
                              const float quantize_step,
                              const float quantize_step_l,
                              const float recover_dec,
                              const float recover_dec_thr,
                              const float recover_min,
                              const float slowness_on_top_for_left_team,
                              const float slowness_on_top_for_right_team,
                              const float stamina_inc_max,
                              const float stamina_max,
                              const float stopped_ball_vel,
                              const float tackle_back_dist,
                              const float tackle_dist,
                              const float tackle_exponent,
                              const float tackle_power_rate,
                              const float tackle_width,
                              const float visible_angle,
                              const float visible_distance,
                              const float wind_ang,
                              const float wind_dir,
                              const float wind_force,
                              const float wind_rand
                             ){
  serverParam.setAll(
                      catch_ban_cycle,
                      clang_advice_win,
                      clang_define_win,
                      clang_del_win,
                      clang_info_win,
                      clang_mess_delay,
                      clang_mess_per_cycle,
                      clang_meta_win,
                      clang_rule_win,
                      clang_win_size,
                      coach_port,
                      drop_ball_time,
                      freeform_send_period,
                      freeform_wait_period,
                      game_log_compression,
                      game_log_version,
                      goalie_max_moves,
                      half_time,
                      hear_decay,
                      hear_inc,
                      hear_max,
                      max_goal_kicks,
                      olcoach_port,
                      point_to_ban,
                      point_to_duration,
                      port,
                      recv_step,
                      say_coach_cnt_max,
                      say_coach_msg_size,
                      say_msg_size,
                      send_step,
                      send_vi_step,
                      sense_body_step,
                      simulator_step,
                      slow_down_factor,
                      start_goal_l,
                      start_goal_r,
                      synch_micro_sleep,
                      synch_offset,
                      tackle_cycles,
                      text_log_compression,
                      game_log_dir,
                      game_log_fixed_name,
                      landmark_file,
                      log_date_format,
                      text_log_dir,
                      text_log_fixed_name,
                      coach,
                      coach_w_referee,
                      old_coach_hear,
                      wind_none,
                      wind_random,
                      back_passes,
                      forbid_kick_off_offside,
                      free_kick_faults,
                      fullstate_l,
                      fullstate_r,
                      game_log_dated,
                      game_log_fixed,
                      game_logging,
                      log_times,
                      profile,
                      proper_goal_kicks,
                      record_messages,
                      send_comms,
                      synch_mode,
                      team_actuator_noise,
                      text_log_dated,
                      text_log_fixed,
                      text_logging,
                      use_offside,
                      verbose,
                      audio_cut_dist,
                      ball_accel_max,
                      ball_decay,
                      ball_rand,
                      ball_size,
                      ball_speed_max,
                      ball_weight,
                      catch_probability,
                      catchable_area_l,
                      catchable_area_w,
                      ckick_margin,
                      control_radius,
                      dash_power_rate,
                      effort_dec,
                      effort_dec_thr,
                      effort_inc,
                      effort_inc_thr,
                      effort_init,
                      effort_min,
                      goal_width,
                      inertia_moment,
                      kick_power_rate,
                      kick_rand,
                      kick_rand_factor_l,
                      kick_rand_factor_r,
                      kickable_margin,
                      maxmoment,
                      maxneckang,
                      maxneckmoment,
                      maxpower,
                      minmoment,
                      minneckang,
                      minneckmoment,
                      minpower,
                      offside_active_area_size,
                      offside_kick_margin,
                      player_accel_max,
                      player_decay,
                      player_rand,
                      player_size,
                      player_speed_max,
                      player_weight,
                      prand_factor_l,
                      prand_factor_r,
                      quantize_step,
                      quantize_step_l,
                      recover_dec,
                      recover_dec_thr,
                      recover_min,
                      slowness_on_top_for_left_team,
                      slowness_on_top_for_right_team,
                      stamina_inc_max,
                      stamina_max,
                      stopped_ball_vel,
                      tackle_back_dist,
                      tackle_dist,
                      tackle_exponent,
                      tackle_power_rate,
                      tackle_width,
                      visible_angle,
                      visible_distance,
                      wind_ang,
                      wind_dir,
                      wind_force,
                      wind_rand
                     );
  newServerParam = true;
}


//----------------------------- player param ----------------------------
void Parser::buildPlayerParam(){ newPlayerParam = true; }
void Parser::buildPlayerParam(PlayerParam pp, int val){
  switch(pp){
  case PP_PlayerTypes:
    playerParam.setPlayerTypes(val);
    break;
  case PP_PTMax:
    playerParam.setPTMax(val);
    break;
  case PP_SubsMax:
    playerParam.setSubsMax(val);
    break;
  default:
    DEBUG4("Parser::buildPlayerParam> Error! PlayerParam: ", pp, " Value: ", val);
    break;
  }
}
void Parser::buildPlayerParam(PlayerParam pp, float val){
  switch(pp){
  case PP_RandomSeed:
    playerParam.setRandomSeed(val);
    break;
  case PP_DashPowerRateDeltaMax:
    playerParam.setDashPowerRateDeltaMax(val);
    break;
  case PP_DashPowerRateDeltaMin:
    playerParam.setDashPowerRateDeltaMin(val);
    break;
  case PP_EffortMaxDeltaFactor:
    playerParam.setEffortMaxDeltaFactor(val);
    break;
  case PP_EffortMinDeltaFactor:
    playerParam.setEffortMinDeltaFactor(val);
    break;
  case PP_ExtraStaminaDeltaMax:
    playerParam.setExtraStaminaDeltaMax(val);
    break;
  case PP_ExtraStaminaDeltaMin:
    playerParam.setExtraStaminaDeltaMin(val);
    break;
  case PP_InertiaMomentDeltaFactor:
    playerParam.setInertiaMomentDeltaFactor(val);
    break;
  case PP_KickRandDeltaFactor:
    playerParam.setKickRandDeltaFactor(val);
    break;
  case PP_KickableMarginDeltaMax:
    playerParam.setKickableMarginDeltaMax(val);
    break;
  case PP_KickableMarginDeltaMin:
    playerParam.setKickableMarginDeltaMin(val);
    break;
  case PP_NewDashPowerRateDeltaMax:
    playerParam.setNewDashPowerRateDeltaMax(val);
    break;
  case PP_NewDashPowerRateDeltaMin:
    playerParam.setNewDashPowerRateDeltaMin(val);
    break;
  case PP_NewStaminaIncMaxDeltaFactor:
    playerParam.setNewStaminaIncMaxDeltaFactor(val);
    break;
  case PP_PlayerDecayDeltaMax:
    playerParam.setPlayerDecayDeltaMax(val);
    break;
  case PP_PlayerDecayDeltaMin:
    playerParam.setPlayerDecayDeltaMin(val);
    break;
  case PP_PlayerSizeDeltaFactor:
    playerParam.setPlayerSizeDeltaFactor(val);
    break;
  case PP_PlayerSpeedMaxDeltaMax:
    playerParam.setPlayerSpeedMaxDeltaMax(val);
    break;
  case PP_PlayerSpeedMaxDeltaMin:
    playerParam.setPlayerSpeedMaxDeltaMin(val);
    break;
  case PP_StaminaIncMaxDeltaFactor:
    playerParam.setStaminaIncMaxDeltaFactor(val);
    break;
  default:
    DEBUG4("Parser::buildPlayerParam> Error! PlayerParam: ", pp, " Value: ", val);
    break;
  }    
}

void Parser::buildPlayerParam(
                              const int player_types,
                              const int pt_max,
                              const float random_seed,
                              const int subs_max,
                              const float dash_power_rate_delta_max,
                              const float dash_power_rate_delta_min,
                              const float effort_max_delta_factor,
                              const float effort_min_delta_factor,
                              const float extra_stamina_delta_max,
                              const float extra_stamina_delta_min,
                              const float inertia_moment_delta_factor,
                              const float kick_rand_delta_factor,
                              const float kickable_margin_delta_max,
                              const float kickable_margin_delta_min,
                              const float new_dash_power_rate_delta_max,
                              const float new_dash_power_rate_delta_min,
                              const float new_stamina_inc_max_delta_factor,
                              const float player_decay_delta_max,
                              const float player_decay_delta_min,
                              const float player_size_delta_factor,
                              const float player_speed_max_delta_max,
                              const float player_speed_max_delta_min,
                              const float stamina_inc_max_delta_factor
                             )
{
  playerParam.setAll(
                      player_types,
                      pt_max,
                      random_seed,
                      subs_max,
                      dash_power_rate_delta_max,
                      dash_power_rate_delta_min,
                      effort_max_delta_factor,
                      effort_min_delta_factor,
                      extra_stamina_delta_max,
                      extra_stamina_delta_min,
                      inertia_moment_delta_factor,
                      kick_rand_delta_factor,
                      kickable_margin_delta_max,
                      kickable_margin_delta_min,
                      new_dash_power_rate_delta_max,
                      new_dash_power_rate_delta_min,
                      new_stamina_inc_max_delta_factor,
                      player_decay_delta_max,
                      player_decay_delta_min,
                      player_size_delta_factor,
                      player_speed_max_delta_max,
                      player_speed_max_delta_min,
                      stamina_inc_max_delta_factor
                     );
  newPlayerParam = true;
}


//----------------------------- player type ---------------------------
void Parser::buildPlayerType(
                              const int id,
                              const float player_speed_max,
                              const float stamina_inc_max,
                              const float player_decay,
                              const float inertia_moment,
                              const float dash_power_rate,
                              const float player_size,
                              const float kickable_margin,
                              const float kick_rand,
                              const float extra_stamina,
                              const float effort_max,
                              const float effort_min
                            )
{
  Sense::PlayerType pt;
  pt.setAll(id,
	    player_speed_max,
	    stamina_inc_max,
	    player_decay,
	    inertia_moment,
	    dash_power_rate,
	    player_size,
	    kickable_margin,
	    kick_rand,
	    extra_stamina,
	    effort_max,
	    effort_min
	    );
  playerType.push_back(pt);
  /*
  DEBUG6_HARLEKIN("New PlayerType received, ID: ", id, 
		  "-", pt.getTypeId(), 
		  "-", (--playerType.end())->getTypeId());
  */
  newPlayerType = true;
}


//----------------------------- sense body ---------------------------
//uses also ViewQuality, ViewWidth, ArmInfo objects
void Parser::buildSenseBody(
                            const int time,
                            const float stamina,
                            const float effort,
                            const float speedAmount,
                            const float speedDir,
                            const float headAngle,
                            const int kickCount,
                            const int dashCount,
                            const int turnCount,
                            const int sayCount,
                            const int turnNeckCount,
                            const int catchCount,
                            const int moveCount,
                            const int changeViewCount) {

  senseBody.setAll(
                    time,
                    vqual,  /* NIY: view quality parsed but not used */
                    vwidth, /* NIY: view width   parsed but not used */
                    stamina,
                    effort,
                    speedAmount,
                    speedDir,
                    headAngle,
                    kickCount,
                    dashCount,
                    turnCount,
                    sayCount,
                    turnNeckCount,
                    catchCount,
                    moveCount,
                    changeViewCount
                   );
  senseBody.armInfo = armInfo; /* NIY: not used */
  newSenseBody = true;
  newSenseBody_synch = true;
}

void Parser::buildFocusInfo( const int count ) {
  senseBody.focusInfo.setCount(count);
}
void Parser::buildFocusTargetNone() {
  senseBody.focusInfo.setTargetSet(false);
}
//uses side object
void Parser::buildFocusTarget( const int unum ) {
  senseBody.focusInfo.setTargetSet(true);
  senseBody.focusInfo.setTargetSide((FieldSide)side);
  senseBody.focusInfo.setTargetUnum(unum);
}
void Parser::buildTackleInfo( const int expires, const int count ) {
  senseBody.tackleInfo.setExpires(expires);
  senseBody.tackleInfo.setCount(count);
}


//----------------------------- visual -----------------------------
void Parser::buildVisual( const int time ) {
  seeTime = time;
  newVisual = true;
  newVisual_synch = true;
}
void Parser::buildGlobalVisual( const int time ) {
  seeTime = time;
  newVisual = true;
  newVisual_synch = true;
}

//uses location object
void Parser::buildLine( const float dir ) {
  line.setRelPos(Vector2D::getVector2DFromPolar(UNDEFINED_FLOAT, dir));
  line.setId(locationToLineId(complexLocation));
  complexLocation = 0;   //reset
  lines.push_back(line);
}

//uses location object
void Parser::buildLine( const float dist, const float dir ) {
  line.setRelPos(Vector2D::getVector2DFromPolar(dist, dir));
  line.setId(locationToLineId(complexLocation));
  complexLocation = 0;   //reset
  lines.push_back(line);
}

//uses location object
void Parser::buildFlag( const bool close,
                        const float dist, const float dir,
                        const float distChg, const float dirChg ) {
  setMobObjAttributes( flag, close, dist, dir, distChg, dirChg );
  flag.setId(MarkerId(complexLocation));
  complexLocation = 0;   //reset
  flags.push_back(flag);
}
void Parser::buildFlag( const bool close,
                        const float dist, const float dir ) {
  setMobObjAttributes( flag, close, dist, dir );
  flag.setId(MarkerId(complexLocation));
  complexLocation = 0;   //reset
  flags.push_back(flag);
}
void Parser::buildFlag( const bool close, const float dir ) {
  setMobObjAttributes( flag, close, dir );
  flag.setId(MarkerId(complexLocation));
  complexLocation = 0;   //reset
  flags.push_back(flag);
}

void Parser::buildGoal( const bool close,
                        const float dist, const float dir,
                        const float distChg, const float dirChg ) {
  setMobObjAttributes( goal, close, dist, dir, distChg, dirChg );
  goal.setId(MarkerId(complexLocation));
  complexLocation = 0;   //reset
  goals.push_back(goal);
}
void Parser::buildGoal( const bool close,
                        const float dist, const float dir ) {
  setMobObjAttributes( goal, close, dist, dir );
  goal.setId(MarkerId(complexLocation));
  complexLocation = 0;   //reset
  goals.push_back(goal);
}
void Parser::buildGoal( const bool close, const float dir ) {
  setMobObjAttributes( goal, close, dir );
  goal.setId(MarkerId(complexLocation));
  complexLocation = 0;   //reset
  goals.push_back(goal);
}

void Parser::buildGlobalGoal( const float posx, const float posy ) {
  setGlobalMobObjAttributes( goal, posx, posy );
  goal.setId(MarkerId(complexLocation));
  complexLocation = 0;   //reset
  goals.push_back(goal);
}

void Parser::buildBall( const bool close,
                        const float dist, const float dir,
                        const float distChg, const float dirChg ) {
  setMobObjAttributes( ball, close, dist, dir, distChg, dirChg );
  balls.push_back(ball);
}
void Parser::buildBall( const bool close,
                        const float dist, const float dir ) {
  setMobObjAttributes( ball, close, dist, dir );
  balls.push_back(ball);
}
void Parser::buildBall( const bool close, const float dir ) {
  setMobObjAttributes( ball, close, dir );
  balls.push_back(ball);
}

void Parser::buildGlobalBall( const float posx, const float posy,
                              const float velx, const float vely ) {
  setGlobalMobObjAttributes( ball, posx, posy, velx, vely );
  balls.push_back(ball);
}

void Parser::buildPlayer( const bool close,
                          const float dist, const float dir,
                          const float distChg, const float dirChg,
                          const float bodyDir, const float faceDir ) {
  setMobObjAttributes( player, close, dist, dir, distChg, dirChg );
  player.setBodyDir(bodyDir);
  player.setHeadDir(faceDir);

  player.setStaminaInfos(UNDEFINED_FLOAT, UNDEFINED_FLOAT, UNDEFINED_FLOAT);

  //dependent of teamname, push_back in allies, neuts or enemies
  if(!strcmp(player.getTeamname(), ""))
    neuts.push_back(player);
  else if(!strcmp(player.getTeamname(), teamname))
    allies.push_back(player);
  else
    enemies.push_back(player);

  player.setTackling(false);
  player.setPointDir(UNDEFINED_FLOAT);
}
void Parser::buildPlayer( const bool close,
                          const float dist, const float dir,
                          const float distChg, const float dirChg ) {
  setMobObjAttributes( player, close, dist, dir, distChg, dirChg );
  player.setBodyDir(UNDEFINED_FLOAT);
  player.setHeadDir(UNDEFINED_FLOAT);

  player.setStaminaInfos(UNDEFINED_FLOAT, UNDEFINED_FLOAT, UNDEFINED_FLOAT);

  //dependent of teamname, push_back in allies, neuts or enemies
  if(!strcmp(player.getTeamname(), ""))
    neuts.push_back(player);
  else if(!strcmp(player.getTeamname(), teamname))
    allies.push_back(player);
  else
    enemies.push_back(player);

  player.setTackling(false);
  player.setPointDir(UNDEFINED_FLOAT);
}
void Parser::buildPlayer( const bool close,
                          const float dist, const float dir ) {
  setMobObjAttributes( player, close, dist, dir );
  player.setBodyDir(UNDEFINED_FLOAT);
  player.setHeadDir(UNDEFINED_FLOAT);

  player.setStaminaInfos(UNDEFINED_FLOAT, UNDEFINED_FLOAT, UNDEFINED_FLOAT);

  //dependent of teamname, push_back in allies, neuts or enemies
  if(!strcmp(player.getTeamname(), ""))
    neuts.push_back(player);
  else if(!strcmp(player.getTeamname(), teamname))
    allies.push_back(player);
  else
    enemies.push_back(player);

  player.setTackling(false);
  player.setPointDir(UNDEFINED_FLOAT);
}
void Parser::buildPlayer( const bool close, const float dir) {
  setMobObjAttributes( player, close, dir );
  player.setBodyDir(UNDEFINED_FLOAT);
  player.setHeadDir(UNDEFINED_FLOAT);

  player.setStaminaInfos(UNDEFINED_FLOAT, UNDEFINED_FLOAT, UNDEFINED_FLOAT);

  //dependent of teamname, push_back in allies, neuts or enemies
  if(!strcmp(player.getTeamname(), ""))
    neuts.push_back(player);
  else if(!strcmp(player.getTeamname(), teamname))
    allies.push_back(player);
  else
    enemies.push_back(player);

  player.setTackling(false);
  player.setPointDir(UNDEFINED_FLOAT);
}

void Parser::buildPlayerExtraInfo( const bool tackling ) {
  player.setTackling(tackling);
  player.setPointDir(UNDEFINED_FLOAT);
}
void Parser::buildPlayerExtraInfo( const float pointdir ) {
  player.setTackling(false);
  player.setPointDir(pointdir);
}
void Parser::buildPlayerExtraInfo( const float pointdir, const bool tackling ) {
  player.setPointDir(pointdir);
  player.setTackling(tackling);
}

void Parser::buildGlobalPlayer( const float posx,
                                const float posy, const float velx,
                                const float vely, const float bodyDir,
                                const float faceDir) {
  setGlobalMobObjAttributes( player, posx, posy, velx, vely );
  player.setBodyDir(bodyDir);
  player.setHeadDir(faceDir);

  //REVISIT whats about tackling and pointdir ??
  //setting temporary (??) to following values
  player.setTackling(false);
  player.setPointDir(UNDEFINED_FLOAT);

  player.setStaminaInfos(UNDEFINED_FLOAT, UNDEFINED_FLOAT, UNDEFINED_FLOAT);

  //dependent of teamname, push_back in allies, neuts or enemies
  if(!strcmp(player.getTeamname(), ""))
    neuts.push_back(player);
  else if(!strcmp(player.getTeamname(), teamname))
    allies.push_back(player);
  else
    enemies.push_back(player);
}

void Parser::buildPlayerName() {
  player.setTeamname("");
  player.setUnum(UNDEFINED_INT);
  player.setGoalie(false);
}
void Parser::buildPlayerName( const char *teamname ) {
  player.setTeamname(teamname);
  player.setUnum(UNDEFINED_INT);
  player.setGoalie(false);
}
void Parser::buildPlayerName( const char *teamname, int unum ) {
  player.setTeamname(teamname);
  player.setUnum(unum);
  player.setGoalie(unum == 1);
}
void Parser::buildPlayerName( const char *teamname,
                              const int unum, const bool goalie ) {
  player.setTeamname(teamname);
  player.setUnum(unum);
  player.setGoalie(goalie || unum == 1);
}

void Parser::buildFlagOffset( const int offset ) {
  complexLocation = (complexLocation * 100) + offset;
}

void Parser::buildTopLocation()     {
  complexLocation = (complexLocation * 10) + Sense::Loc_Top;
}
void Parser::buildBottomLocation()  {
  complexLocation = (complexLocation * 10) + Sense::Loc_Bottom;
}
void Parser::buildLeftLocation()    {
  complexLocation = (complexLocation * 10) + Sense::Loc_Left;
}
void Parser::buildRightLocation()   {
  complexLocation = (complexLocation * 10) + Sense::Loc_Right;
}
void Parser::buildCenterLocation()  {
  complexLocation = (complexLocation * 10) + Sense::Loc_Center;
}
void Parser::buildPenaltyLocation() {
  complexLocation = (complexLocation * 10) + Sense::Loc_Penalty;
}
void Parser::buildGoalLocation()    {
  complexLocation = (complexLocation * 10) + Sense::Loc_Goal;
}


//----------------------------- score ---------------------------
void Parser::buildScore( const int time, const int our, const int opp ) {
  score.setTime(time);
  score.setOurScore(our);
  score.setOppScore(opp);

  newScore = true;
}


//----------------------------- fullstate ---------------------------
void Parser::buildFullState( const int time ) {
  fullstate.setTime(time);
  fullstate.setPlayMode(pmode);
  fullstate.setViewMode(vqual, vwidth);
  fullstate.armInfo = armInfo;

  newFullstate = true;
  newFullstate_synch = true;
}
void Parser::buildFsCount(
                          const int kickCount,
                          const int dashCount,
                          const int turnCount,
                          const int sayCount,
                          const int turnNeckCount,
                          const int catchCount,
                          const int moveCount,
                          const int changeViewCount
                         ) {
  fullstate.setCounters( kickCount, dashCount, turnCount, sayCount, turnNeckCount,
                          catchCount, moveCount, changeViewCount );
}
void Parser::buildFsScore( const int our, const int their) {
  fullstate.setScores( our, their );
}
void Parser::buildFsBall( const float posx, const float posy,
                          const float velx, const float vely ) {
  setGlobalMobObjAttributes( ball, posx, posy, velx, vely );
  fs_balls.push_back(ball);
}
void Parser::buildFsPlayer(
                           const float posx,
                           const float posy,
                           const float velx,
                           const float vely,
                           const float bodyDir,
                           const float faceDir
                          ) {
  setGlobalMobObjAttributes( player, posx, posy, velx, vely );
  player.setBodyDir(bodyDir);
  player.setHeadDir(faceDir);

  //REVISIT whats about tackling and pointdir ??
  //setting temporary (??) to following values
  player.setTackling(false);
  player.setPointDir(UNDEFINED_FLOAT);

  if( ((FieldSide)side) == fside )
    fs_allies.push_back(player);
  else
    fs_enemies.push_back(player);

  //reset
  player.setGoalie(false);
  player.setType(0);
}

void Parser::buildFsPlayername( const int unum ) {
  player.setUnum(unum);
}
void Parser::buildFsPlayerType( const bool goalie ) {
  player.setGoalie(goalie);
}
void Parser::buildFsPlayerType( const int type ) {
  player.setType(type);
}
void Parser::buildFsStaminaInfo( const float stamina, const float effort,
                                 const float recovery ) {
  player.setStaminaInfos( stamina, effort, recovery );
}


//----------------------------- audio ---------------------------
void Parser::buildCoachAudio( const int time, const char *msg ) {
  coachAudio.setTime(time);
  coachAudio.setMsg(msg);

  newCoachAudio = true;
}
void Parser::buildNeutralSide() { side = Sense::S_Neutral; }

void Parser::buildRefAudio( const int time ){
  refAudio.setTime(time);
  refAudio.setPlaymode(pmode);
  refAudio.setPenaltySide(pside);

  newRefAudio = true;
}
void Parser::buildUnknownRefAudio( const int time, const char *msg ) {
  refAudio.setTime(time);
  refAudio.setUnknownRefMsg(msg);

  newRefAudio = true;
}

void Parser::buildPlayerAudio( const int time, const float dir,
                               const int unum, const char *msg ){
  playerAudio.setTime(time);
  playerAudio.setDir(dir);
  playerAudio.setUnum(unum);
  playerAudio.setMsg(msg);
  playerAudio.setSide(side);
  newPlayerAudio = true;
}
void Parser::buildPlayerAudio( const int time, const float dir,
                               const char *msg ) {
  playerAudio.setTime(time);
  playerAudio.setDir(dir);
  playerAudio.setUnum(UNDEFINED_INT);
  playerAudio.setMsg(msg);
  playerAudio.setSide(side);
  newPlayerAudio = true;
}
void Parser::buildPlayerAudio( const int time, const int unum ) {
  playerAudio.setTime(time);
  playerAudio.setUnum(unum);
  playerAudio.setSide(side);
  newPlayerAudio = true;
}

void Parser::buildPlayerAudio( const int time ) {
  playerAudio.setTime(time);
  playerAudio.setSide(side);
  newPlayerAudio = true;
}

void Parser::buildSelfAudio( const int time, const char *msg ) {
  selfAudio.setTime(time);
  selfAudio.setMsg(msg);

  newSelfAudio = true;
}


//----------------------------- change ---------------------------
void Parser::buildSubstitution( const int unum, const int type ) {
  Sense::Substitution sub;
  sub.setUnum(unum);
  sub.setType(type);
  substitution.push_back(sub);

  newSubstitution = true;
}
void Parser::buildSubstitution( const int unum ) {
  if (unum) return;
  // HACK: Need to tell them apart from own messages
  /*
  substitution.setUnum(unum);
  substitution.setType(UNDEFINED_INT);

  newSubstitution = true;
  */
}


//----------------------------- error ---------------------------
void Parser::buildCantReconnect()
{ error.setType(E_Reconnect); newError = true; }
void Parser::buildInitError()
{ error.setType(E_Init); newError = true; }
void Parser::buildNoMoreTeamOrPlayerOrGoalieError()
{ error.setType(E_NoMoreTeamOrPlayerOrGoalie); newError = true; }
void Parser::buildNoSuchTeamOrAlreadyHaveCoachError()
{ error.setType(E_NoSuchTeamOrAlreadyHaveCoach); newError = true; }
void Parser::buildTooManyMovesError()
{ error.setType(E_TooManyMoves); newError = true; }
void Parser::buildUnknownCommandError()
{ error.setType(E_UnknownCommand); newError = true; }
void Parser::buildIllegalCommandError()
{ error.setType(E_IllegalCommandForm); newError = true; }
void Parser::buildSayMessageTooLongError()
{ error.setType(E_SayMessageTooLong); newError = true; }
void Parser::buildIllegalModeError()
{ error.setType(E_IllegalMode); newError = true; }
void Parser::buildIllegalObjectFormError()
{ error.setType(E_IllegalObjectForm); newError = true; }
void Parser::buildSaidTooManyFreeformMessagesError()
{ error.setType(E_SaidTooManyFreeformMessages); newError = true; }
void Parser::buildCannotSayFreeformWhilePlayonError()
{ error.setType(E_CannotSayFreeformWhilePlayon); newError = true; }
void Parser::buildSaidTooManyMetaMessagesError()
{ error.setType(E_SaidTooManyMetaMessages); newError = true; }
void Parser::buildSaidTooManyAdviceMessagesError()
{ error.setType(E_SaidTooManyAdviceMessages); newError = true; }
void Parser::buildSaidTooManyInfoMessagesError()
{ error.setType(E_SaidTooManyInfoMessages); newError = true; }
void Parser::buildSaidTooManyDefineMessagesError()
{ error.setType(E_SaidTooManyDefineMessages); newError = true; }
void Parser::buildCouldNotParseSayError()
{ error.setType(E_CouldNotParseSay); newError = true; }
void Parser::buildSaidTooManyMessagesError()
{ error.setType(E_SaidTooManyMessages); newError = true; }
void Parser::buildUnknownError( const char *err )
{
  error.setType(E_UnknownError);
  error.setUnknownErrorMsg(err);
  newError = true;
}


//----------------------------- warning ---------------------------
void Parser::buildNoTeamFoundWarning()
{ warning.setType(W_NoTeamFound); newWarning = true; }
void Parser::buildNoSuchPlayerWarning()
{ warning.setType(W_NoSuchPlayer); newWarning = true; }
void Parser::buildCannotSubWhilePlayOnWarning()
{ warning.setType(W_CannotSubWhilePlayOn); newWarning = true; }
void Parser::buildNoSubsLeftWarning()
{ warning.setType(W_NoSubsLeft); newWarning = true; }
void Parser::buildMaxOfThatPlayerTypeOnFieldWarning()
{ warning.setType(W_MaxOfThatPlayerTypeOnField); newWarning = true; }
void Parser::buildCannotChangeGoalieWarning()
{ warning.setType(W_CannotChangeGoalie); newWarning = true; }
void Parser::buildUnknownWarning( const char *warn )
{
  warning.setType(W_UnknownWarning);
  warning.setUnknownWarningMsg(warn);
  newWarning = true;
}


//----------------------------- ok ---------------------------
void Parser::buildClangVerOK( const int min, const int max ) {
  DEBUG5("Parser: clang version ok message with: min=", min, "max=", max,
         " ignored");
  // Unused parameter if no debug used
  if (min == max) ;
}
void Parser::buildEyeOK( const bool eyeOn ) {
  DEBUG3("Parser: eye ok message with: eyeOn=", eyeOn, " ignored");
  // Unused parameter if no debug used
  if (eyeOn) ;
}
void Parser::buildUnknownOK( const char *ok ) {
  DEBUG3("Parser: \"", ok, "\" message ignored");
  // Unused parameter if no debug used
  if (ok) ;
}

//----------------------------- sync ---------------------------
void Parser::buildThink(){
  newThink = true;
}

//---------------------------- playertypes ----------------------
/*void Parser::buildPlayerTypeDetection(){
  newPlayerTypeDetected = true;
}
*/
void Parser::buildCoachFreeform(const char *msg){
  newCoachFreeform = true;

  char *str = ((char *)msg); //remove first '('
  char delims[] = " ";
  char *result = NULL;
  result = strtok( str,  delims ); // remove first number (0) which
                                   // states which sort of msg we
                                   // received
  int type = atoi(result + 1);
  result = strtok( NULL, delims);

  if (type == 0){ // Playertypes
      while( result != NULL ) {
	  int unum = atoi(result + 1);    
	  //DEBUG4_HARLEKIN(" string: ", result, " unum: ", unum);
	  result = strtok( NULL, delims);
	  int type = atoi(result);
	  //DEBUG4_HARLEKIN(" string: ", result, " type: ", type);
	  result = strtok( NULL, delims);
	  coachFreeform.setPlayerType(unum, type);
      }
  } else if (type == 1){
      float x = atof(result + 1);
      coachFreeform.setOffsideX(x);
  } else if (type == 2) {
    int fm = atoi (result + 1);
    coachFreeform.setFormation(fm);
  } else {
      DEBUG2("Parser::buildCoachFreeform(): unknown messagetype, msg: ", msg);
  }
}


LineId Parser::locationToLineId(int location){
  LineId lid = L_Top;
  switch(location){
    case Sense::Loc_Top:    { lid = L_Top;    break; }
    case Sense::Loc_Bottom: { lid = L_Bottom; break; }
    case Sense::Loc_Left:   { lid = L_Left;   break; }
    case Sense::Loc_Right:  { lid = L_Right;  break; }
  default:
    DEBUG2("Parser::locationToLineId: unknown location: ", location);
    break;
  }
  return lid;
}


//----------------------- ParsedDataProvider --------------------
bool Parser::getVisual( vector < Sense::Line >   *_lines  ,
                        vector < Sense::Flag >   *_flags  ,
                        vector < Sense::Goal >   *_goals  ,
                        vector < Sense::Player > *_allies ,
			vector < Sense::Player > *_enemies,
                        vector < Sense::Player > *_neuts  ,
                        vector < Sense::Ball >   *_balls  )
{
  bool rc=false;

  if(newVisual){
    *_lines   = lines;
    *_flags   = flags;
    *_goals   = goals;
    *_allies  = allies;
    *_enemies = enemies;
    *_neuts   = neuts;
    *_balls   = balls;

    if(ASSERT_DEBUG){
      checkLines(*_lines);
      checkFlags(*_flags);
      checkGoals(*_goals);
      checkAllies(*_allies);
      checkEnemies(*_enemies);
      checkNeuts(*_neuts);
      checkBalls(*_balls);
    }

    lines.clear(), flags.clear(), goals.clear(), allies.clear(),
      enemies.clear(), neuts.clear(), balls.clear();

    rc = true;
    newVisual = false;
  }
  return rc;
}
bool Parser::getFullstate   (Sense::Fullstate     **_fullstate,
                             vector<Sense::Goal>   *_goals  ,
                             vector<Sense::Player> *_allies ,
                             vector<Sense::Player> *_enemies,
                             vector<Sense::Ball>   *_balls  )
{
  bool rc=false;
  if(newFullstate){
    *_fullstate = &fullstate;
    *_goals   = fs_goals;
    *_allies  = fs_allies;
    *_enemies = fs_enemies;
    *_balls   = fs_balls;

    //REVISIT
    //check fullstate
    //check goals
    //check allies
    //check enemies
    //check balls

    fs_goals.clear(), fs_allies.clear(), fs_enemies.clear(), fs_balls.clear();

    rc=!(newFullstate=false);
  }
  return rc;
}
bool Parser::getSenseBody(Sense::SenseBody **data){
  //check sb
  return (newSenseBody? !(*data=&senseBody,newSenseBody=false) :false);
}
bool Parser::getRefAudio(Sense::RefAudio **data){
  //check refAudio
  return (newRefAudio?!(*data=&refAudio,newRefAudio=false):false);
}
bool Parser::getSelfAudio(Sense::SelfAudio **data){
  //check selfAudio
  return (newSelfAudio?!(*data=&selfAudio,newSelfAudio=false):false);
}
bool Parser::getPlayerAudio(Sense::PlayerAudio **data){
  //check playerAudio
  return (newPlayerAudio?!(*data=&playerAudio,newPlayerAudio=false):false);
}
bool Parser::getCoachAudio(Sense::CoachAudio **data){
  //check coachAudio
  return (newCoachAudio?!(*data=&coachAudio,newCoachAudio=false):false);
}
bool Parser::getScore(Sense::Score **data){
  //check score
  return (newScore?!(*data=&score,newScore=false):false);
}
bool Parser::getSubstitution(Sense::Substitution **data){
  //check substitution
  if (newSubstitution){
    *data=&(substitution.front());
    substitution.erase(substitution.begin());
    newSubstitution = !substitution.empty();
    return true;
  }else{
    return false;
  }
  
  
  //return (newSubstitution?!(*data=&substitution,newSubstitution=false):false);
}


bool Parser::getError(Sense::Error **data){
  //check error
  return (newError?!(*data=&error,newError=false):false);
}
bool Parser::getInitMsg(Sense::InitMsg **data){
  //check initMsg
  return (newInitMsg?!(*data=&initMsg,newInitMsg=false):false);
}
bool Parser::getServerParam(Sense::ServerParam **data){
  return (newServerParam?!(*data=&serverParam,newServerParam=false):false);
}
bool Parser::getPlayerParam(Sense::PlayerParam **data){
  return (newPlayerParam?!(*data=&playerParam,newPlayerParam=false):false);
}
bool Parser::getPlayerType(Sense::PlayerType **data){
  //check substitution
  if (playerType.empty()) return newPlayerType = false;
  if (newPlayerType){
    //DEBUG2_HARLEKIN("PlayerType-Vektor, Gre vorher: ", playerType.size()); 
    *data=&(playerType.back());
    playerType.pop_back();
    newPlayerType = !playerType.empty();
    //DEBUG2_HARLEKIN("PlayerType-Vektor, Gre danach: ", playerType.size()); 
    return true;
  }else{
    return false;
  }
  //return (newPlayerType?!(*data=&playerType,newPlayerType=false):false);
}

bool Parser::getCoachPlayerTypes( int **type ){
    return false;
}

bool Parser::getCoachFreeform( Sense::CoachFreeform **data){
    return (newCoachFreeform?!(*data=&coachFreeform,newCoachFreeform=false):false);
}


//----------------------- /ParsedDataProvider --------------------


void Parser::setMobObjAttributes( Sense::MobObject &mo, const bool close,
                                  const float dist, const float dir,
                                  const float distChg, const float dirChg ){
  mo.setClose(close);
  mo.setRelPos(Vector2D::getVector2DFromPolar(dist, dir));
  mo.setRelVel(Sense::MobObject::calcVel(dist, dir, distChg, dirChg));
}
void Parser::setMobObjAttributes( Sense::MobObject &mo, const bool close,
                                  const float dist, const float dir ){
  mo.setClose(close);
  mo.setRelPos(Vector2D::getVector2DFromPolar(dist, dir));
  mo.setRelVel(Vector2D(UNDEFINED_FLOAT, UNDEFINED_FLOAT));
}
void Parser::setMobObjAttributes( Sense::MobObject &mo,
                                  const bool close, const float dir ){
  mo.setClose(close);
  mo.setRelPos(Vector2D::getVector2DFromPolar(UNDEFINED_FLOAT, dir));
  mo.setRelVel(Vector2D(UNDEFINED_FLOAT, UNDEFINED_FLOAT));
}

void Parser::setGlobalMobObjAttributes( Sense::MobObject &mo,
                                        const float posx, const float posy,
                                        const float velx, const float vely ){
  mo.setClose(false);
  mo.setRelPos(Vector2D(posx, posy));
  mo.setRelVel(Vector2D(velx, vely));
}

//is used in the case of a global sensing a goal,
//the velocity is *not* unsure. it is 0
void Parser::setGlobalMobObjAttributes( Sense::MobObject &mo,
                                        const float posx, const float posy ){
  mo.setClose(false);
  mo.setRelPos(Vector2D(posx, posy));
  mo.setRelVel(Vector2D(0, 0));
}


//-------------------------- bug chasing -----------------------
#define MAX_DISTANCE 150
#define MAX_MARKERID MARKER_UNKNOWN       //currently the maximum (5555)
#define MAX_REL_PLAYERVEL_WITHNOISE 1.2
#define MAX_REL_BALLVEL_WITHNOISE 3.5
#define MAX_REL_VEL_WITHNOISE MAX_REL_BALLVEL_WITHNOISE
#define MAX_UNUM 11

void Parser::checkStatObject(const Sense::StatObject &so) const {
  Vector2D pos = so.getRelPos();
  assert(isValidAngle(pos.getDirection()));
  assert(isUndefined(pos.getLength())
         ||
         pos.getLength() >= 0 && pos.getLength() < MAX_DISTANCE);
}
void Parser::checkLine(const Sense::Line &l) const {
  checkStatObject(l);
  LineId li = l.getId();
  assert(li==L_Top || li==L_Bottom || li==L_Left || li==L_Right);
}
void Parser::checkMobObject(const Sense::MobObject &mo) const {
  checkStatObject(mo);
  Vector2D vel = mo.getRelVel();
  assert(isValidBool(mo.isClose()));
  if(mo.isClose())
    //the undefined distance cannot be ensured, because it is often seen by
    //close players
    assert(isUndefined(vel.getX()) && isUndefined(vel.getY()));
  else{
    //the defined distance cannot be ensured, because of information lacks
    //in LOW viewmodes
    assert((isUndefined(vel.getX()) && isUndefined(vel.getY()))
           ||
           (vel.getLength() >= 0 && vel.getLength() <= MAX_REL_VEL_WITHNOISE
            && isValidAngle(vel.getDirection())));
  }
}
void Parser::checkFlag(const Sense::Flag &f) const {
  checkMobObject(f);
  Vector2D vel = f.getRelVel();
  MarkerId mi = f.getId();
  assert(mi >= 0 && mi <= MAX_MARKERID);
  if(!f.isClose())
    assert((isUndefined(vel.getX()) && isUndefined(vel.getY()))
           ||
           (vel.getLength() <= MAX_REL_PLAYERVEL_WITHNOISE));
}
void Parser::checkGoal(const Sense::Goal &g) const {
  checkMobObject(g);
  MarkerId mi = g.getId();
  assert(mi == 0 || mi == GOAL_L || mi == GOAL_R);
}
void Parser::checkBall(const Sense::Ball &b) const {
  checkMobObject(b);
}
void Parser::checkPlayer(const Sense::Player &p) const {
  checkMobObject(p);
  Vector2D vel = p.getRelVel();
  float hDir   = p.getHeadDir();
  float bDir   = p.getBodyDir();
  float pDir   = p.getPointDir();
  int unum     = p.getUnum();
  bool goalie  = p.isGoalie();
  bool tackl   = p.isTackling();

  if(p.isClose())
    assert(isUndefined(bDir) && isUndefined(hDir) && isUndefined(unum)
           && isUndefined(pDir));
  else{
    assert((isUndefined(vel.getX()) && isUndefined(vel.getY()))
           ||
           (vel.getLength()<=MAX_REL_PLAYERVEL_WITHNOISE));
    assert(isUndefined(bDir) || isValidAngle(bDir));
    assert(isUndefined(hDir) || isValidAngle(hDir));
    assert(isUndefined(unum) || (unum >=1 && unum <= MAX_UNUM));
    assert(isUndefined(pDir) || isValidAngle(pDir));
  }

  assert(isValidBool(goalie));
  assert(isValidBool(tackl));

  //REVISIT: check type, staminainfos
}
void Parser::checkAlly(const Sense::Player &p) const {
  checkPlayer(p);
  assert(strcmp(teamname, p.getTeamname()) == 0);
}
void Parser::checkEnemy(const Sense::Player &p) const {
  checkPlayer(p);
  assert(strcmp(teamname, p.getTeamname()) != 0);
}
void Parser::checkNeut(const Sense::Player &p) const {
  checkPlayer(p);
  assert(strcmp("", p.getTeamname()) == 0);
}


void Parser::checkLines(const vector< Sense::Line > &lines) const {
  vector< Sense::Line>::const_iterator i;
  for(i = lines.begin(); i != lines.end(); ++i)
    checkLine(*i);
}
void Parser::checkFlags(const vector< Sense::Flag > &flags) const {
  vector< Sense::Flag>::const_iterator i;
  for(i = flags.begin(); i != flags.end(); ++i)
    checkFlag(*i);
}
void Parser::checkGoals(const vector< Sense::Goal > &goals) const {
  vector< Sense::Goal>::const_iterator i;
  for(i = goals.begin(); i != goals.end(); ++i)
    checkGoal(*i);
}
void Parser::checkBalls(const vector< Sense::Ball > &balls) const {
  vector< Sense::Ball >::const_iterator i;
  for(i = balls.begin(); i != balls.end(); ++i)
    checkBall(*i);
}
void Parser::checkAllies(const vector< Sense::Player > &players) const {
  vector< Sense::Player>::const_iterator i;
  for(i = players.begin(); i != players.end(); ++i)
    checkAlly(*i);
}
void Parser::checkEnemies(const vector< Sense::Player > &players) const {
  vector< Sense::Player>::const_iterator i;
  for(i = players.begin(); i != players.end(); ++i)
    checkEnemy(*i);
}
void Parser::checkNeuts(const vector< Sense::Player > &players) const {
  vector< Sense::Player>::const_iterator i;
  for(i = players.begin(); i != players.end(); ++i)
    checkNeut(*i);
}


// $Source: /home/robocup/virtualwerder/CVSRepository/VirtualWerder2004/Parser.cc,v $








