/*****************************************************************************
 * Parser.hh
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   class Parser
 *
 * requires:
 *
 * Created on: 28.08.2002 by wenns
 * Maintainer: wenns
 * Group:
 * Co-authors:
 *
 *
 * $Id: Parser.hh,v 1.4 2004/04/22 10:13:44 harlekin Exp $
 *****************************************************************************/

#ifndef PARSER_HH
#define PARSER_HH

#include "ParserIntfc.hh"
#include "ParsedDataProvider.hh"

/**
 * Implements the interface of the parser, contains and manages all the
 * objects that contain the parsed data
 */
class Parser : public ParserIntfc, public ParsedDataProvider
{
public:
  Parser::Parser();
  virtual ~Parser();

  //Accessor
  void setTeamname(const char *tn);

  virtual void buildLeftSide();
  virtual void buildRightSide();
  virtual void buildOurSide();
  virtual void buildOppSide();

  //playmodes
  virtual void buildBeforeKickOffPMode();
  virtual void buildTimeOverPMode();
  virtual void buildPlayOnPMode();
  virtual void buildKickOffLPMode();
  virtual void buildKickOffRPMode();
  virtual void buildKickInLPMode();
  virtual void buildKickInRPMode();
  virtual void buildFreeKickLPMode();
  virtual void buildFreeKickRPMode();
  virtual void buildCornerKickLPMode();
  virtual void buildCornerKickRPMode();
  virtual void buildGoalKickLPMode();
  virtual void buildGoalKickRPMode();
  virtual void buildGoalLPMode();
  virtual void buildGoalRPMode();
  virtual void buildGoalLPMode(const int goals);
  virtual void buildGoalRPMode(const int goals);
  virtual void buildDropBallPMode();
  virtual void buildOffsideLPMode();
  virtual void buildOffsideRPMode();
  virtual void buildPenaltyKickLPMode();
  virtual void buildPenaltyKickRPMode();
  virtual void buildPausePMode();
  virtual void buildHumanJudgePMode();
  virtual void buildFoulLPMode();
  virtual void buildFoulRPMode();
  virtual void buildFoulChargeLPMode();
  virtual void buildFoulChargeRPMode();
  virtual void buildFoulPushLPMode();
  virtual void buildFoulPushRPMode();
  virtual void buildFoulMultipleAttackLPMode();
  virtual void buildFoulMultipleAttackRPMode();
  virtual void buildFoulBallOutLPMode();
  virtual void buildFoulBallOutRPMode();
  virtual void buildBackPassLPMode();
  virtual void buildBackPassRPMode();
  virtual void buildFreeKickFaultLPMode();
  virtual void buildFreeKickFaultRPMode();
  virtual void buildGoalieCatchBallLPMode();
  virtual void buildGoalieCatchBallRPMode();
  virtual void buildHalfTimePMode();
  virtual void buildTimeUpRPMode();
  virtual void buildIndirectFreeKickLPMode();
  virtual void buildIndirectFreeKickRPMode();
  virtual void buildPenaltySetupLPMode();
  virtual void buildPenaltySetupRPMode();
  virtual void buildPenaltyReadyLPMode();
  virtual void buildPenaltyReadyRPMode();
  virtual void buildPenaltyTakenLPMode();
  virtual void buildPenaltyTakenRPMode();
  virtual void buildPenaltyScoreLPMode();
  virtual void buildPenaltyScoreRPMode();
  virtual void buildPenaltyMissLPMode();
  virtual void buildPenaltyMissRPMode();
  virtual void buildPenaltySideL();
  virtual void buildPenaltySideR();


  virtual void buildViewQualityHigh();
  virtual void buildViewQualityLow();

  virtual void buildViewWidthNarrow();
  virtual void buildViewWidthNormal();
  virtual void buildViewWidthWide();

  virtual void buildArmInfo(const int movable, const int expires,
			    const float targetDist, const float targetDir,
			    const int count);


  //----------------------------- init ----------------------------------
  //uses also side and playmode objects
  virtual void buildInit(const int unum);


  //----------------------------- server param ----------------------------
  virtual void buildServerParam();
  virtual void buildServerParam(const ServerParam sp, const int    val);
  virtual void buildServerParam(const ServerParam sp, const double val);
  virtual void buildServerParam(const ServerParam sp, const bool   val);
  virtual void buildServerParam(const ServerParam sp, const char  *val);
  virtual void buildServerParam(
				const int catch_ban_cycle,
				const int clang_advice_win,
				const int clang_define_win,
				const int clang_del_win,
				const int clang_info_win,
				const int clang_mess_delay,
				const int clang_mess_per_cycle,
				const int clang_meta_win,
				const int clang_rule_win,
				const int clang_win_size,
				const int coach_port,
				const int drop_ball_time,
				const int freeform_send_period,
				const int freeform_wait_period,
				const int game_log_compression,
				const int game_log_version,
				const int goalie_max_moves,
				const int half_time,
				const int hear_decay,
				const int hear_inc,
				const int hear_max,
				const int max_goal_kicks,
				const int olcoach_port,
				const int point_to_ban,
				const int point_to_duration,
				const int port,
				const int recv_step,
				const int say_coach_cnt_max,
				const int say_coach_msg_size,
				const int say_msg_size,
				const int send_step,
				const int send_vi_step,
				const int sense_body_step,
				const int simulator_step,
				const int slow_down_factor,
				const int start_goal_l,
				const int start_goal_r,
				const int synch_micro_sleep,
				const int synch_offset,
				const int tackle_cycles,
				const int text_log_compression,
				const char *game_log_dir,
				const char *game_log_fixed_name,
				const char *landmark_file,
				const char *log_date_format,
				const char *text_log_dir,
				const char *text_log_fixed_name,
				const bool coach,
				const bool coach_w_referee,
				const bool old_coach_hear,
				const bool wind_none,
				const float wind_random,
				const bool back_passes,
				const bool forbid_kick_off_offside,
				const bool free_kick_faults,
				const bool fullstate_l,
				const bool fullstate_r,
				const bool game_log_dated,
				const bool game_log_fixed,
				const bool game_logging,
				const bool log_times,
				const bool profile,
				const bool proper_goal_kicks,
				const bool record_messages,
				const bool send_comms,
				const bool synch_mode,
				const bool team_actuator_noise,
				const bool text_log_dated,
				const bool text_log_fixed,
				const bool text_logging,
				const bool use_offside,
				const bool verbose,
				const float audio_cut_dist,
				const float ball_accel_max,
				const float ball_decay,
				const float ball_rand,
				const float ball_size,
				const float ball_speed_max,
				const float ball_weight,
				const float catch_probability,
				const float catchable_area_l,
				const float catchable_area_w,
				const float ckick_margin,
				const float control_radius,
				const float dash_power_rate,
				const float effort_dec,
				const float effort_dec_thr,
				const float effort_inc,
				const float effort_inc_thr,
				const float effort_init,
				const float effort_min,
				const float goal_width,
				const float inertia_moment,
				const float kick_power_rate,
				const float kick_rand,
				const float kick_rand_factor_l,
				const float kick_rand_factor_r,
				const float kickable_margin,
				const float maxmoment,
				const float maxneckang,
				const float maxneckmoment,
				const float maxpower,
				const float minmoment,
				const float minneckang,
				const float minneckmoment,
				const float minpower,
				const float offside_active_area_size,
				const float offside_kick_margin,
				const float player_accel_max,
				const float player_decay,
				const float player_rand,
				const float player_size,
				const float player_speed_max,
				const float player_weight,
				const float prand_factor_l,
				const float prand_factor_r,
				const float quantize_step,
				const float quantize_step_l,
				const float recover_dec,
				const float recover_dec_thr,
				const float recover_min,
				const float slowness_on_top_for_left_team,
				const float slowness_on_top_for_right_team,
				const float stamina_inc_max,
				const float stamina_max,
				const float stopped_ball_vel,
				const float tackle_back_dist,
				const float tackle_dist,
				const float tackle_exponent,
				const float tackle_power_rate,
				const float tackle_width,
				const float visible_angle,
				const float visible_distance,
				const float wind_ang,
				const float wind_dir,
				const float wind_force,
				const float wind_rand
				);


  //----------------------------- player param ----------------------------
  virtual void buildPlayerParam();
  virtual void buildPlayerParam(const PlayerParam pp, const int    val);
  virtual void buildPlayerParam(const PlayerParam pp, const float val);
  //virtual void buildPlayerParam(const PlayerParam pp, const bool   val);
  //virtual void buildPlayerParam(const PlayerParam pp, const char  *val);
  virtual void buildPlayerParam(
				const int player_types,
				const int pt_max,
				const float random_seed,
				const int subs_max,
				const float dash_power_rate_delta_max,
				const float dash_power_rate_delta_min,
				const float effort_max_delta_factor,
				const float effort_min_delta_factor,
				const float extra_stamina_delta_max,
				const float extra_stamina_delta_min,
				const float inertia_moment_delta_factor,
				const float kick_rand_delta_factor,
				const float kickable_margin_delta_max,
				const float kickable_margin_delta_min,
				const float new_dash_power_rate_delta_max,
				const float new_dash_power_rate_delta_min,
				const float new_stamina_inc_max_delta_factor,
				const float player_decay_delta_max,
				const float player_decay_delta_min,
				const float player_size_delta_factor,
				const float player_speed_max_delta_max,
				const float player_speed_max_delta_min,
				const float stamina_inc_max_delta_factor
				);


  //----------------------------- player type ---------------------------
  virtual void buildPlayerType(
			       const int id,
			       const float player_speed_max,
			       const float stamina_inc_max,
			       const float player_decay,
			       const float inertia_moment,
			       const float dash_power_rate,
			       const float player_size,
			       const float kickable_margin,
			       const float kick_rand,
			       const float extra_stamina,
			       const float effort_max,
			       const float effort_min
			       );


  //----------------------------- sense body ---------------------------
  virtual void buildSenseBody(
			      const int time,
			      const float stamina,
			      const float effort,
			      const float speedAmount,
			      const float speedDir,
			      const float headAngle,
			      const int kickCount,
			      const int dashCount,
			      const int turnCount,
			      const int sayCount,
			      const int turnNeckCount,
			      const int catchCount,
			      const int moveCount,
			      const int changeViewCount );

  virtual void buildFocusInfo( const int count );
  virtual void buildFocusTargetNone();
  virtual void buildFocusTarget( const int unum );
  virtual void buildTackleInfo( const int expires, const int count );


  //----------------------------- visual -----------------------------
  virtual void buildVisual( const int time );
  virtual void buildGlobalVisual( const int time );

  virtual void buildLine( const float dir );
  virtual void buildLine( const float dist, const float dir );

  virtual void buildFlag( const bool close, const float dist,
			  const float dir, const float distChg,
			  const float dirChg );
  virtual void buildFlag( const bool close, const float dist,
			  const float dir );
  virtual void buildFlag( const bool close, const float dir );

  virtual void buildGoal( const bool close, const float dist,
			  const float dir, const float distChg,
			  const float dirChg );
  virtual void buildGoal( const bool close, const float dist,
			  const float dir );
  virtual void buildGoal( const bool close, const float dir );

  virtual void buildGlobalGoal( const float posx, const float posy );

  virtual void buildBall( const bool close, const float dist,
			  const float dir, const float distChg,
			  const float dirChg );
  virtual void buildBall( const bool close, const float dist,
			  const float dir );
  virtual void buildBall( const bool close, const float dir );

  virtual void buildGlobalBall( const float posx, const float posy,
				const float vely, const float vely );

  virtual void buildPlayer( const bool close,
			    const float dist, const float dir,
			    const float distChg, const float dirChg,
			    const float bodyDir, const float faceDir );
  virtual void buildPlayer( const bool close,
			    const float dist, const float dir,
			    const float distChg, const float dirChg );
  virtual void buildPlayer( const bool close,
			    const float dist, const float dir );
  virtual void buildPlayer( const bool close, const float dir);

  virtual void buildPlayerExtraInfo( const bool tackling );
  virtual void buildPlayerExtraInfo( const float pointdir );
  virtual void buildPlayerExtraInfo( const float pointdir, const bool tackling );

  virtual void buildGlobalPlayer( const float posx, const float posy,
				  const float velx, const float vely,
				  const float bodyDir, const float faceDir );

  virtual void buildPlayerName();
  virtual void buildPlayerName( const char *teamname );
  virtual void buildPlayerName( const char *teamname,
				const int unum );
  virtual void buildPlayerName( const char *teamname,
				const int unum, const bool goalie );

  virtual void buildFlagOffset( const int offset );

  virtual void buildTopLocation()    ;
  virtual void buildBottomLocation() ;
  virtual void buildLeftLocation()   ;
  virtual void buildRightLocation()  ;
  virtual void buildCenterLocation() ;
  virtual void buildPenaltyLocation();
  virtual void buildGoalLocation()   ;


  //----------------------------- score ---------------------------
  virtual void buildScore( const int time, const int our, const int their );


  //----------------------------- fullstate ---------------------------
  virtual void buildFullState( const int time );
  virtual void buildFsCount(
			    const int kickCount,
			    const int dashCount,
			    const int turnCount,
			    const int sayCount,
			    const int turnNeckCount,
			    const int catchCount,
			    const int moveCount,
			    const int changeViewCount
			    );
  virtual void buildFsScore( const int our, const int their);

  virtual void buildFsBall( const float posx, const float posy,
			    const float velx, const float vely );
  virtual void buildFsPlayer(
			     const float posx,
			     const float posy,
			     const float velx,
			     const float vely,
			     const float bodyDir,
			     const float faceDir
			     );

  virtual void buildFsPlayername( const int unum );
  virtual void buildFsPlayerType( const bool goalie );
  virtual void buildFsPlayerType( const int type );
  virtual void buildFsStaminaInfo( const float stamina, const float effort,
				   const float recovery );


  //----------------------------- audio ---------------------------
  virtual void buildCoachAudio( const int time, const char *msg );
  virtual void buildNeutralSide();

  virtual void buildRefAudio( const int time );
  virtual void buildUnknownRefAudio( const int time, const char *msg );

  virtual void buildPlayerAudio( const int time, const float dir,
				 const int unum, const char *msg );
  virtual void buildPlayerAudio( const int time, const float dir,
				 const char *msg );
  virtual void buildPlayerAudio( const int time, const int unum );
  virtual void buildPlayerAudio( const int time );

  virtual void buildSelfAudio( const int time, const char *msg );


  //----------------------------- change ---------------------------
  virtual void buildSubstitution( const int unum, const int type );
  virtual void buildSubstitution( const int unum );


  //----------------------------- error ---------------------------
  virtual void buildCantReconnect();
  virtual void buildInitError();
  virtual void buildNoMoreTeamOrPlayerOrGoalieError();
  virtual void buildNoSuchTeamOrAlreadyHaveCoachError();
  virtual void buildTooManyMovesError();
  virtual void buildUnknownCommandError();
  virtual void buildIllegalCommandError();
  virtual void buildSayMessageTooLongError();
  virtual void buildIllegalModeError();
  virtual void buildIllegalObjectFormError();
  virtual void buildSaidTooManyFreeformMessagesError();
  virtual void buildCannotSayFreeformWhilePlayonError();
  virtual void buildSaidTooManyMetaMessagesError();
  virtual void buildSaidTooManyAdviceMessagesError();
  virtual void buildSaidTooManyInfoMessagesError();
  virtual void buildSaidTooManyDefineMessagesError();
  virtual void buildCouldNotParseSayError();
  virtual void buildSaidTooManyMessagesError();
  virtual void buildUnknownError( const char *error );


  //----------------------------- warning ---------------------------
  virtual void buildNoTeamFoundWarning();
  virtual void buildNoSuchPlayerWarning();
  virtual void buildCannotSubWhilePlayOnWarning();
  virtual void buildNoSubsLeftWarning();
  virtual void buildMaxOfThatPlayerTypeOnFieldWarning();
  virtual void buildCannotChangeGoalieWarning();
  virtual void buildUnknownWarning( const char *warning );


  //----------------------------- ok ---------------------------
  virtual void buildClangVerOK( const int min, const int max );
  virtual void buildEyeOK( const bool eyeOn );
  virtual void buildUnknownOK( const char *ok );

  //---------------------------- sync ---------------------------
  virtual void buildThink();

  //-------------------------- playertypes -----------------------
  //virtual void buildPlayerTypeDetection();
  virtual void buildCoachFreeform(const char *msg);


  //----------------------- ParsedDataProvider --------------------
  bool getVisual      ( vector<Sense::Line>   *_lines  ,
			vector<Sense::Flag>   *_flags  ,
			vector<Sense::Goal>   *_goals  ,
			vector<Sense::Player> *_allies ,
			vector<Sense::Player> *_enemies,
			vector<Sense::Player> *_neuts  ,
			vector<Sense::Ball>   *_balls  );
  bool getFullstate   (Sense::Fullstate     **_fullstate,
		       vector<Sense::Goal>   *_goals  ,
		       vector<Sense::Player> *_allies ,
		       vector<Sense::Player> *_enemies,
		       vector<Sense::Ball>   *_balls  );
  bool getSenseBody   ( Sense::SenseBody    **data );
  bool getRefAudio    ( Sense::RefAudio     **data );
  bool getSelfAudio   ( Sense::SelfAudio    **data );
  bool getPlayerAudio ( Sense::PlayerAudio  **data );
  bool getCoachAudio  ( Sense::CoachAudio   **data );
  bool getScore       ( Sense::Score        **data );
  bool getSubstitution( Sense::Substitution **data );
  bool getError       ( Sense::Error        **data );
  bool getInitMsg     ( Sense::InitMsg      **data );
  bool getServerParam ( Sense::ServerParam  **data );
  bool getPlayerParam ( Sense::PlayerParam  **data );
  bool getPlayerType  ( Sense::PlayerType   **data );
  bool getCoachPlayerTypes( int **type );
    bool getCoachFreeform( Sense::CoachFreeform **data );

  bool seeReceived()  {
    bool rc = newVisual_synch;
    newVisual_synch = false;
    return rc;
  }
  bool sbReceived()   {
    bool rc = newSenseBody_synch;
    newSenseBody_synch = false;
    return rc;
  }
  bool fsReceived()   {
    bool rc = newFullstate_synch;
    newFullstate_synch = false;
    return rc;
  }
  bool initReceived()   {
    bool rc = newInitMsg_synch;
    newInitMsg_synch = false;
    return rc;
  }
  bool thinkReceived() {
    bool tc = newThink;
    newThink = false;
    return tc;
  }


private:
  LineId locationToLineId( const int location );
  void setMobObjAttributes( Sense::MobObject &mo, const bool close,
			    const float dist, const float dir,
			    const float distChg, const float dirChg );
  void setMobObjAttributes( Sense::MobObject &mo, const bool close,
			    const float dist, const float dir );
  void setMobObjAttributes( Sense::MobObject &mo, const bool close,
			    const float dir );

  void setGlobalMobObjAttributes( Sense::MobObject &mo,
				  const float posx, const float posy,
				  const float velx, const float vely );
  void setGlobalMobObjAttributes( Sense::MobObject &mo,
				  const float posx, const float posy );


  void checkStatObject(const Sense::StatObject &so) const;
  void checkLine(const Sense::Line &l) const;
  void checkMobObject(const Sense::MobObject &mo) const;
  void checkFlag(const Sense::Flag &f) const;
  void checkGoal(const Sense::Goal &g) const;
  void checkBall(const Sense::Ball &b) const;
  void checkPlayer(const Sense::Player &p) const;
  void checkAlly(const Sense::Player &p) const;
  void checkEnemy(const Sense::Player &p) const;
  void checkNeut(const Sense::Player &p) const;

  void checkLines(const vector< Sense::Line > &lines) const;
  void checkFlags(const vector< Sense::Flag > &flags) const;
  void checkGoals(const vector< Sense::Goal > &goals) const;
  void checkBalls(const vector< Sense::Ball > &balls) const;
  void checkAllies(const vector< Sense::Player > &players) const;
  void checkEnemies(const vector< Sense::Player > &players) const;
  void checkNeuts(const vector< Sense::Player > &players) const;

  //------------------------------- parsed data ----------------------------

  /** \name visual information  */
  //@{
  vector< Sense::Line >    lines;   // 0, 1 or many
  vector< Sense::Flag >    flags;   // 0, 1 or many
  vector< Sense::Player >  neuts;   // 0, 1 or many
  vector< Sense::Player >  allies;  // 0, 1 or many
  vector< Sense::Player >  enemies; // 0, 1 or many
  vector< Sense::Goal >    goals;   // 0, 1 or 2
  vector< Sense::Ball >    balls;   // 0 or 1

  //--- temporarily needed objects -----
  //in the case of a flag can be constructed of several Sense::Locations
  int complexLocation;
  Sense::Line line;
  Sense::Flag flag;
  //--- /temporarily needed objects -----

  int seeTime;
  //@}

  Sense::SenseBody         senseBody;

  /** \name fullstate information */
  //@{
  Sense::Fullstate         fullstate;
  vector< Sense::Player >  fs_allies;  // 1..11
  vector< Sense::Player >  fs_enemies; // 0..11
  vector< Sense::Goal >    fs_goals;   // 2
  vector< Sense::Ball >    fs_balls;   // 1
  //@}

  /** \name varios audio messages */
  //@{
  Sense::RefAudio          refAudio;
  Sense::SelfAudio         selfAudio;
  Sense::PlayerAudio       playerAudio;
  Sense::CoachAudio        coachAudio;
    Sense::CoachFreeform   coachFreeform;
  //@}

  /** \name initial messages */
  //@{
  Sense::InitMsg           initMsg;
  Sense::ServerParam       serverParam;
  Sense::PlayerParam       playerParam;
  vector<Sense::PlayerType> playerType;
  //@}

  /** \name other messages */
  //@{
  Sense::Score             score;
  vector<Sense::Substitution> substitution;
  Sense::Error             error;
  Sense::Warning           warning;
  //@}

  /** \name object that are shared */
  //@{
  Sense::Side              side;
  Playmode                 pmode;
  ViewQuality              vqual;
  ViewWidth                vwidth;
  Sense::ArmInfo           armInfo;

  int detectedTypes[11];

  //--- temporarily needed objects -----
  Sense::Goal   goal;
  Sense::Ball   ball;
  Sense::Player player;
  //--- /temporarily needed objects -----
  //@}
  
  bool volatile newVisual_synch;
  bool volatile newFullstate_synch;
  bool volatile newSenseBody_synch;
  bool volatile newInitMsg_synch;
  
  bool volatile newVisual;
  bool volatile newFullstate;
  bool volatile newSenseBody;
  bool volatile newRefAudio;
  bool volatile newSelfAudio;
  bool volatile newPlayerAudio;
  bool volatile newCoachAudio;
  bool volatile newScore;
  bool volatile newSubstitution;
  bool volatile newError;
  bool volatile newInitMsg;
  bool volatile newServerParam;
  bool volatile newPlayerParam;
  bool volatile newPlayerType;
  bool volatile newWarning;

  bool volatile newCoachFreeform;

  bool volatile newThink;

  //-------------------- other --------------------
  FieldSide   fside;
  PenaltySide pside;
  char teamname[TEAMNAME_LENGTH];
};

// $Source: /home/robocup/virtualwerder/CVSRepository/VirtualWerder2004/Parser.hh,v $
#endif //PARSER_HH
