/*****************************************************************************
 * Sensors.hh
 * Copyright (c) 2001 Virtual Werder
 *
 * part of simple agent
 *
 * implements:
 *   class Sensors
 *
 * requires:
 *   Robocup Client Command Parser lib
 *
 * Created on: 25.03.2001 by wenns
 * Maintainer: cyco
 * Group:
 * Co-authors: wenns
 *
 *
 * $Id: Sensors.hh,v 1.3 2004/04/03 20:18:29 harlekin Exp $
 *****************************************************************************/

#ifndef SENSORS_HH
#define SENSORS_HH

#include "common.hh"
#include "Communicator.hh"

#include "ParsedDataProvider.hh"
#include "Parser.hh"

/**
 * The only interface to the environment (the soccerserver) which can be used by
 * the agent to sense the environment. The server messages are awaited, parsed and
 * stored temporarily until WorldModel requests them.
 */
class Sensors : public ParsedDataProvider {
  public:
    Sensors(Communicator*, const char*);
    virtual ~Sensors();

    /** Waits for input and parses the messages using the parser */
    void sense();

    /**
     * \name The interface for the worldmodel
     *
     * the following methods are the interface to the worldmodel. Using them the
     * WorldModel can request information of a certain kind by passing a double
     * pointer and evaluate the return value. If information of this kind was
     * received, "true" is returned and the pointer is "filled"
     */
    //@{
    inline bool getVisual      (vector<Sense::Line>  *_lines  ,
	         	       vector<Sense::Flag>   *_flags  ,
	         	       vector<Sense::Goal>   *_goals  ,
	         	       vector<Sense::Player> *_allies ,
	         	       vector<Sense::Player> *_enemies,
	         	       vector<Sense::Player> *_neuts  ,
	         	       vector<Sense::Ball>   *_balls  );
    inline bool getFullstate   (Sense::Fullstate     **_fullstate,
                                vector<Sense::Goal>   *_goals  ,
                                vector<Sense::Player> *_allies ,
                                vector<Sense::Player> *_enemies,
                                vector<Sense::Ball>   *_balls  );
    inline bool getSenseBody   (Sense::SenseBody    **data);
    inline bool getRefAudio    (Sense::RefAudio     **data);
    inline bool getSelfAudio   (Sense::SelfAudio    **data);
    inline bool getPlayerAudio (Sense::PlayerAudio  **data);
    inline bool getCoachAudio  (Sense::CoachAudio   **data);
    inline bool getScore       (Sense::Score        **data);
    inline bool getSubstitution(Sense::Substitution **data);
    inline bool getError       (Sense::Error        **data);
    inline bool getInitMsg     (Sense::InitMsg      **data);
    inline bool getServerParam (Sense::ServerParam  **data);
    inline bool getPlayerParam (Sense::PlayerParam  **data);
    inline bool getPlayerType  (Sense::PlayerType   **data);
    inline bool getCoachPlayerTypes(int **type);
    inline bool getCoachFreeform(Sense::CoachFreeform **data);
    //@}

    /**
     * @return true, if see was received since last integrating of the sensor
     * infos into the worldmodel
     */
    inline bool seeReceived() { return parser.seeReceived(); }
    /**
     * @return true, if sense body was received since last integrating of the
     * sensor infos into the worldmodel
     */
    inline bool sbReceived()  { return parser.sbReceived();   }
    /**
     * @return true, if fullstate was received since last integrating of the
     * sensor infos into the worldmodel
     */
    inline bool fsReceived() { return parser.fsReceived(); }
    /**
     * @return true, if init reply was received since last integrating of the
     * sensor infos into the worldmodel
     */
    inline bool initReceived() { return parser.initReceived(); }
  
  inline bool thinkReceived() { return parser.thinkReceived();   }
    

  private:
    Communicator  *comm;
    char           message[SS_MSG_MAX_SIZE];
    Parser parser;
};


bool Sensors::getVisual(vector < Sense::Line >   *_lines  ,
			vector < Sense::Flag >   *_flags  ,
			vector < Sense::Goal >   *_goals  ,
			vector < Sense::Player > *_allies ,
			vector < Sense::Player > *_enemies,
			vector < Sense::Player > *_neuts  ,
                        vector < Sense::Ball >   *_balls  )
{
  return parser.getVisual(_lines, _flags, _goals, _allies, _enemies, _neuts,
                          _balls  );
}
bool Sensors::getFullstate(Sense::Fullstate        **_fullstate,
                           vector < Sense::Goal >   *_goals  ,
                           vector < Sense::Player > *_allies ,
                           vector < Sense::Player > *_enemies,
                           vector < Sense::Ball >   *_balls  )
{
  return parser.getFullstate(_fullstate, _goals, _allies, _enemies, _balls  );
}
bool Sensors::getSenseBody(Sense::SenseBody **data){
  return parser.getSenseBody(data);
}
bool Sensors::getRefAudio(Sense::RefAudio **data)
{
  return parser.getRefAudio(data);}
bool Sensors::getSelfAudio(Sense::SelfAudio **data){
  return parser.getSelfAudio(data);
}
bool Sensors::getPlayerAudio(Sense::PlayerAudio **data){
  return parser.getPlayerAudio(data);
}
bool Sensors::getCoachAudio(Sense::CoachAudio **data){
  return parser.getCoachAudio(data);
}
bool Sensors::getScore(Sense::Score **data){
  return parser.getScore(data);
}
bool Sensors::getSubstitution(Sense::Substitution **data){
  return parser.getSubstitution(data);
}
bool Sensors::getError(Sense::Error **data){
  return parser.getError(data);
}
bool Sensors::getInitMsg(Sense::InitMsg **data){
  return parser.getInitMsg(data);
}
bool Sensors::getServerParam(Sense::ServerParam **data){
  return parser.getServerParam(data);
}
bool Sensors::getPlayerParam(Sense::PlayerParam **data){
  return parser.getPlayerParam(data);
}
bool Sensors::getPlayerType(Sense::PlayerType **data){
  return parser.getPlayerType(data);
}
bool Sensors::getCoachPlayerTypes(int **type){
  return parser.getCoachPlayerTypes(type);
}
bool Sensors::getCoachFreeform(Sense::CoachFreeform **data){
    return parser.getCoachFreeform(data);
}

// $Source: /home/robocup/virtualwerder/CVSRepository/VirtualWerder2004/Sensors.hh,v $
#endif //SENSORS_HH
