#include "CenterAttackerBehaviour.hh"
#include "../debug.hh"

/** Constructs and initializes the Center Attacker Behaviour*/
CenterAttackerBehaviour::CenterAttackerBehaviour() : Behaviour()
{
}

/** Destroys the Center Attacker Behaviour*/
CenterAttackerBehaviour::~CenterAttackerBehaviour()
{
}

void CenterAttackerBehaviour::switchAttackerView(){
  
  if (worldModel->ball.isHighConfident(worldModel->getTime())){
    skillPool->faceOnlyBodyToBall();
    skillPool->faceOnlyNeckToPoint(Vector2D(myself->getPosition().getX(),0));
  }else{
    skillPool->faceOnlyNeckToBall();
  }
  return;
}

void CenterAttackerBehaviour::playOnWithBall(){   

  // This should be done if the ball wouldnt be kickable next turn
  // eg. leave the kickable area
  if (ball->getVelocity().isInvalid()){
    skillPool->holdBall();
    skillPool->faceOnlyNeckToBall();
    return;
  }

  
  if (tryToForceScoreGoal()){
    lookAround(true);
    APL_ELLIPSE3_MOSH(Vector2D(50,30),1,1, (APLLevel) 4  , 255, 255, 255);

    return;
  
  }
  
  // Shoot a goal if possible
  if (tryToScoreGoal()){
    lookAround(true);
    APL_ELLIPSE3_MOSH(Vector2D(50,30),1,1, (APLLevel) 4  , 255, 0, 0);
    return;
  }
    
  //try to Dribble behind enemys Defense 
  if(ball->getPosition().getX() > 0){
    static float passSpeed, passPower;
    Vector2D passPoint    = Vector2D(NAN,NAN);
    Vector2D passVelocity = Vector2D(NAN,NAN);
    if(worldModel->canDribbleBehindDefense(passPoint,passVelocity)){
      passSpeed = passVelocity.getLength();
      passPower = worldModel->calcKickPowerForSpeed( passSpeed );
      skillPool->kickToPoint( passPoint, passPower, false );      
      return;
    }
  }
   

  if (considerShouldPass()){
    lookAround(true);
    return;
  }
  
  if (considerDribbling()){
    currentlyDribbling = true;
    lookAround(false);
    return;
  }
  
  // prepare for pass  
  if(fabs(ball->getPosition().getY())>15 && ball->getPosition().getX()>44){
    if (worldModel->isAttacked(worldModel->myself)){
      skillPool->kickAway();
      lookAround(true); 
       return;
    }    
     
    if(skillPool->turnWithBallTo((Vector2D(42,0)-
				  myself->getPosition()).getDirection() ) )
      ;//go On
    else{
      skillPool->faceOnlyNeckToPoint(Vector2D(40,0));    
      return;
    }
  }
  

  if (considerCouldPass()){
    lookAround(true);
    return;
  }

  if (isStopBallNeeded()){
    skillPool->stopBall();
    lookAround(true);
    return;
  }

  skillPool->holdBall();
  skillPool->scanFieldWithNeck();
  //lookAround();
  return;

}

void CenterAttackerBehaviour::reportStatus(){
  Playmode pm = worldModel->getPlaymode();
  if (pm == PM_BeforeKickOff)
    return;

  if (worldModel->passPointToSayCycles != 0){
    DEBUG1_HARLEKIN("Behaviour::reportStatus(): communicating good pass position");
    worldModel->passPointToSayCycles--;
    skillPool->reportPassPoint();
    return;
  }

  // report the ball data in every cycle!
  if (worldModel->isClosestToBall(*myself)){
    if (ball->isHighConfident(worldModel->getTime())){
      skillPool->reportBallStatus();
    }
    return;
  }

  if (worldModel->isAGoodPointToScore(myself->getPosition())){
    skillPool->reportPosition();
    return;
  }

  if (ball->isHighConfident(worldModel->getTime())){
    if (worldModel->hasBallTeam()){
      if (ball->getPosition().distanceTo(myself->getPosition()) < 15.0){
	skillPool->reportPosition();    
	return;
      }
    }
  }
  
}

















