#include "CenterMidfielderBehaviour.hh"
#include "debug.hh"

/** Creates and initializes the center midfielder Behaviour*/
CenterMidfielderBehaviour::CenterMidfielderBehaviour() : Behaviour()
{
}

/** destroys the Center Midfielder behaviour*/
CenterMidfielderBehaviour::~CenterMidfielderBehaviour()
{
}

void CenterMidfielderBehaviour::playOnWithBall(){
  //Formation *formation = Formations::getCurrentFormation();
  //int unum = myself->getUniformNumber();
  //int myBehaviour = formation->getBehaviour(unum);

  // This should be done if the ball wouldnt be kickable next turn
  // eg. leave the kickable area
  if (ball->getVelocity().isInvalid()){
    skillPool->holdBall();
    skillPool->faceOnlyNeckToBall();
    return;
  }
  
  if (tryToForceScoreGoal()){
    lookAround(true);
    return;
  }
  
  // Shoot a goal if possible
  if (tryToScoreGoal()){
    lookAround(true);
    return;
  }

  
    //try to Dribble behind enemys Defense   
  if(ball->getPosition().getX() > 0){
    static float passSpeed, passPower;
    Vector2D passPoint    = Vector2D(NAN,NAN);
    Vector2D passVelocity = Vector2D(NAN,NAN);
    if(worldModel->canDribbleBehindDefense(passPoint,passVelocity)){
      passSpeed = passVelocity.getLength();
      passPower = worldModel->calcKickPowerForSpeed( passSpeed );
      skillPool->kickToPoint( passPoint, passPower, false );      
      return;
    }
  }
   

  if (considerShouldPass()){
    lookAround(true);
    return;
  }


  // prepare for pass  
  if(fabs(ball->getPosition().getY())>15 && ball->getPosition().getX()>44){
    if (worldModel->isAttacked(worldModel->myself)){
      skillPool->kickAway();
      //skillPool->kickToPoint(Vector2D(40,0),100);
      lookAround(true); 
       return;
    }    
     
    if(skillPool->turnWithBallTo((Vector2D(42,0)-
				  myself->getPosition()).getDirection() ) )
      ;//go On
    else{
      skillPool->faceOnlyNeckToPoint(Vector2D(40,0));    
      return;
    }
  }
   
  if (considerDribbling()){
    currentlyDribbling = true;
    lookAround(false);
    return;
  }
 

  if (considerCouldPass()){
    lookAround(true);
    return;
  }

  if (isStopBallNeeded()){
    skillPool->stopBall();
    lookAround(true);
    return;
  }

  skillPool->holdBall();
  lookAround(false);
  return;
}

/** Handles the behaviour if the Agent does not have the Ball (the Ball is not kickable)*/
void CenterMidfielderBehaviour::playOnWithoutBall(){
  //Configuration::printPlayerTypes();

  // if the player looses the ball while dribbling he would
  // otherwise still think to be dribbling if he gets into the
  // reach of another ball... later... uhm... and would not stop it...
  currentlyDribbling = false;

  const Player *fastest = worldModel->fastestInSetToBall(PS_Teammates);
  if(fastest != NULL) {
    if(*fastest == *myself){
      if(skillPool->getBall(100,false,true)){
	//DEBUG1_HARLEKIN("Turn to Ball and stop it. BOTH?!");
	skillPool->stopBall();
	skillPool->faceOnlyNeckToBall();
      }else{
	//skillPool->faceOnlyNeckToBall();
	//DEBUG1_HARLEKIN("Look Around.");
	lookAround(true);
      }
      return;
    }
  }else{  //no one is fastest to ball
    if(worldModel->isClosestTeammateToBall(*myself)){
      if(skillPool->getBall()){
	//DEBUG1_HARLEKIN("Turn to Ball.");
	skillPool->faceOnlyNeckToBall();
	skillPool->stopBall();
      }else{
	//skillPool->faceOnlyNeckToBall();
	//DEBUG1_HARLEKIN("Look Around.");
	lookAround(true);
      }
      return;
    }
  }
  //DEBUG1_HARLEKIN("Go to strategic Position...");
  doPositioning();
}




