#include "GoalieBehaviour.hh"


/** Constructor initializing the Behaviour and calling the super Constructor*/
GoalieBehaviour::GoalieBehaviour() : Behaviour()
{
  now = *(worldModel->getTime());
  passCand = false;
  freeKickStep = 0;
  zaehler = 0;
  scoreGoal = false;
  oneTime = false;
}

/** Destructor doing nothing at the moment*/
GoalieBehaviour::~GoalieBehaviour()
{
}

/** Handles Free Kicks, e.g. if the Goalie caught the Ball
    @return true if the freeKick was successful, false otherwise
    @todo this should be protected and maybe void
*/

bool GoalieBehaviour::freeKickBehaviour(){


  static const Vector2D middle      = Vector2D(worldModel->field.getMarkerPosition(FLAG_PLC) - Vector2D(2,  0.0));
  static Vector2D kickOffPoint;
  //static const Vector2D *goalieLeft  = &(Field::GOALIE_KICK_OFF_POINT_LEFT );
  //static const Vector2D *goalieRight = &(Field::GOALIE_KICK_OFF_POINT_RIGHT);
  float stamina = myself->bodyInfo.getStamina();
  bool perform = false;

  //DEBUG2_HARLEKIN("FreeKickStep: ", freeKickStep);

  // only do something if we get a see message
  if (!isLastSeeAtCurrentTime())
    return false;

  zaehler++;

  // just to be sure
  if(zaehler > 100){
    skillPool->kickAway(false,true);
    return true;
  }
  
  // recover
  if(stamina < 2800.0) 
    return false;

  // we waited long enough, we should to something
  if (zaehler > 30){
    perform = true;
  }

  // check if the move-command was really sent
  // we could also check the last sent command in the worldModel
  // but this should be good enough
  if (freeKickStep == 1 && 
      myself->getPosition().distanceToSq(middle) > 1.0f){
    freeKickStep = 0;
  }
  if (freeKickStep == 2 && 
      myself->getPosition().distanceToSq(kickOffPoint) > 1.0f){
    freeKickStep = 1;
  }
  
   
  if (freeKickStep == 0){
    skillPool->moveToPosition(middle);
    freeKickStep = 1;
    return true;
  }
  
  if (!perform) 
    return false;
  
  if (freeKickStep == 1){
    kickOffPoint = worldModel->findGoalieKickOffPoint();
    skillPool->moveToPosition(kickOffPoint);
    freeKickStep = 2;
    return true;
  }

  passOrKick();
  return true;    
}

void GoalieBehaviour::passOrKick(){
  const float PASS_END_SPEED = ConfigFile::passEndSpeed;
  const Player *passCand = goaliePass();
  
  if (passCand != NULL){
    skillPool->passBallToPlayer(passCand, PASS_END_SPEED);
    return;
  }else{
    skillPool->kickAway(false, true);
    return;
  }
}

// look left... look right...
bool GoalieBehaviour::switchView(){
  static bool direction = true;

  bool see = isLastSeeAtCurrentTime();
  
  if(see) direction = !direction;

  Vector2D flag_one = worldModel->field.getMarkerPosition(FLAG_T0);
  Vector2D flag_two = worldModel->field.getMarkerPosition(FLAG_B0);

  if(!direction){
    skillPool->faceNeckToPoint(flag_one);
  }else{
    skillPool->faceNeckToPoint(flag_two);
  }

  return see;
}

const Player* GoalieBehaviour::goaliePass(){
  Player *passCand = NULL;
  vector<Player>::iterator i;
  Vector2D myPos = myself->getPosition();
  Formation *f = Formations::getCurrentFormation();

  for (i = worldModel->myTeam.getPlayers().begin(); i < worldModel->myTeam.getPlayers().end(); i++){
    // dont pass to yourself
    if (i->getUniformNumber() == myself->getUniformNumber()) continue;
    // dont pass to players which you havent seen for "long" time
    if (!i->isGoodConfident(worldModel->getTime())) continue;
    // dont pass to midfielders or attackers
    if (f->getBehaviour(i->getUniformNumber()) >= B_WingMidfielderLeft) continue;
    // dont pass to players which are behind you
    if (myPos.getX() > i->getPosition().getX()) continue;
    // dont pass to the other side (inside) of the field
    // only to wing defenders
    if (myPos.getY() > 0 && i->getPosition().getY() <  10) continue;
    if (myPos.getY() < 0 && i->getPosition().getY() > -10) continue;
    // dont pass to player whose position is unknown
    // well, he wouldnt be confident
    if(i->getPosition().isInvalid()) continue;
    // dont pass to players you cant pass to
    float dist = (i->getPosition() - ball->getPosition()).getLength();
    float kstp = worldModel->calcKickSpeedToTravel(dist, ConfigFile::passEndSpeed);
    if(worldModel->isIntersected_NH(i->getPosition(), ball->getPosition(), kstp)) continue;
    // TODO: Decide which player to pass to. This method only passes to the last found player
    passCand = &(*i);
  }
  return passCand;
}

void GoalieBehaviour::goalKickBehaviour(){
  int type = myself->getType();
  double prefferedDistance = Configuration::getPlayerSize(type) +
    Configuration::getKickableMargin(type)/2 +
    Configuration::ballSize;
  // Should be done anyway
  const Vector2D &ballPos = ball->getPosition();
  if(ballPos.isInvalid()){
    skillPool->scanFieldWithBody();
  }else{
    // go to ball
    // Go behind it, so you can achieve higher kick power
    static Vector2D freeKickPosition;
    freeKickPosition = ballPos - Vector2D(prefferedDistance,0); 
    if(skillPool->goToPoint(freeKickPosition, prefferedDistance/2, 80)){
      zaehler++;
      if( // wait a bit and face to the right position (otherway around)
	 skillPool->faceOnlyBodyToPoint(Vector2D(0,myself->getPosition().getY()))
	 && zaehler > 35 
	 ){
	// pass to player or if none available kickAway... 
	passOrKick();
      }
      switchView();
      return;
    }
    lookAround(true);
  } 
}

void GoalieBehaviour::generalBehaviour(){
  static bool fsUsed = ConfigFile::useFullstate;
  Playmode    pm = worldModel->getPlaymode();
  now = *(worldModel->getTime());
  //PenaltySide ps = worldModel->getPenaltySide(); 

  static Vector2D opponent_goal_mid = worldModel->field.getMarkerPosition(GOAL_R );
  static Vector2D opponent_rc       = worldModel->field.getMarkerPosition(FLAG_RB);
  static Vector2D opponent_lc       = worldModel->field.getMarkerPosition(FLAG_RT);

  if (ConfigFile::useDifferentViewAngles){
    if (pm != PM_BeforeKickOff){
      if (worldModel->switchViewNeeded())
	skillPool->doSwitchView();
    }
  }

  Vector2D ballPos = worldModel->ball.getPosition();
  switch(pm){
      //cases where the goalie must pay attention, normal "behaviour"
    case PM_PlayOn:
      zaehler = 0;      
      freeKickStep = 0;
      playOnBehaviour();
      break;
    case PM_OurIndirectFreeKick:
    case PM_TheirIndirectFreeKick:
    case PM_OurOffside:
    case PM_TheirOffside:
    case PM_OurKickIn:
    case PM_TheirKickIn:
    case PM_TheirFreeKick:
    case PM_OurCornerKick:
    case PM_TheirCornerKick:
    case PM_TheirGoalKick:
    case PM_OurKickOff:
    case PM_TheirKickOff:
      if(ballPos.isInvalid()){
	scanFieldWithBodyGoalie();
      }else{
	doPositioning();
	//goaliePositioning(GOALIENOTHING,ballPos);
      }
      break;
    case PM_OurGoalKick:
      goalKickBehaviour();
      break;     
    case PM_OurFreeKick:
      if(ballPos.isInvalid()){
	// scan also behind yourself
	skillPool->scanFieldWithBody();
      }else{
	  if( Field::OWN_PENALTY_AREA.isInside(ball->getPosition()) 
	      /*worldModel->distanceToBall() <= 2.0*/ ){
	  //skillPool->performGoalieFreeKick();
	  // If no action is taken (recovering stamina)
	  // turn to the mid of the field to update worldmodel
	  if (!freeKickBehaviour()){
	    skillPool->faceOnlyBodyToPoint(Vector2D(0,0));
	    //skillPool->scanFieldWithNeckGoalie();
	    switchView();
	  }
	}else{
	  doPositioning();
      	  //goaliePositioning(goaliePosBeh(ballPos), ballPos);
	}
      }
      break;
    case PM_OurGoalieCatchBall:
      ; break;//the case pm==8 will follow
    case PM_HalfTime:
      break;
    case PM_BeforeKickOff:
      // change_view hack to begin of the game
      if( !fsUsed ){
	if(!worldModel->haveOptimalSeeTimes()){
	  skillPool->doSSProfiling();
	  lookAround(true);
	  return;
	}
      }      
      if(!skillPool->moveToInitialPosition()){
	lookAround(false);
      }else{
	skillPool->faceNeckAndBodyToPoint(Vector2D(0,0));
      }
      break;
      //goalie should only go to his goal's center
    case PM_TheirGoal:
    case PM_OurGoal: 
      if (worldModel->getTime()->getStoppedCount() + 8 >= 50){
	if(myself->getPosition().distanceToSq(myself->getInitPosition()) > 1){
	  skillPool->moveToInitialPosition();
	}
	skillPool->faceNeckAndBodyToPoint(Vector2D(0,0));
      }else{
	skillPool->moveLikeAnIdiot();
      }
      break;
    case PM_OurPenaltySetup:
    case PM_OurPenaltyReady:
    case PM_OurPenaltyTaken:
    case PM_OurPenaltyScore:
    case PM_OurPenaltyMiss:
      ourPenGoaliePositioning();
      break;
    case PM_TheirPenaltySetup:
      goaliePenaltySetup();
      break;
    case PM_TheirPenaltyReady:
    case PM_TheirPenaltyTaken:
    case PM_TheirPenaltyScore:
    case PM_TheirPenaltyMiss:
      //skillPool->doPenaltyGoalie();
      goaliePenaltyBehaviour();
      break;
    case PM_TimeOver:
      break;
    default:
      DEBUG2("GoalieBehaviour::generalBehaviour: unknown playmode: ", pm);
      // The goalie should ALWAYS know where the ball is
      if (ballPos.isInvalid())
	scanFieldWithBodyGoalie();
      else
	doPositioning();
      break;
    }
  // REVISIT
  
}

void GoalieBehaviour::doPositioning(){
  /*
  const Vector2D *ballPos = ball->getPositionPt();
  goaliePositioning(goaliePosBeh(*ballPos), *ballPos);
  return;
  */
  //ermittle den positionierungspunkt
  //beispielsweise duch Schnittpunkt eines um das Tor gelegten Kreises mit
  //einer Geraden zu der Ballpos
  //float distGoalBall = Field::GOAL_CENTER.distanceTo(ballPos);
  //int cycles = (int)(distGoalBall / Configuration::ballSpeedMax);
  // Shouldn't consider this. Won't work for vertical dribbling
  //Vector2D ballPredPos = ball->predictedPosition(cycles);
  // use this instead:
  Vector2D ballPredPos = ball->predictedPosition();

  Line line = Line::makeLineFromTwoPoints(ballPredPos, Field::GOAL_CENTER);
  Vector2D InterSecPoint_1;
  Vector2D InterSecPoint_2;
  Vector2D PointToGo;
  line.getCircleIntersectionPoints(Field::GOALIE_POSITIONING_CIRCLE,
				   &InterSecPoint_1, &InterSecPoint_2);
  
  
  //der positioning circle hat mit der (geraden zw. ball und tormitte) zwei
  //schnittpunkte; der goalie positioniert sich an dem, der auf dem feld
  //liegt. im unguenstigen falle liegen beide auf dem feld: dann positioniere
  //dich an dem punkt, der naeher zum ball ist
  if(Field::FIELD.isInside(InterSecPoint_1) && !Field::FIELD.isInside(InterSecPoint_2))
    PointToGo = InterSecPoint_1;
  else if(!Field::FIELD.isInside(InterSecPoint_1) && Field::FIELD.isInside(InterSecPoint_2))
    PointToGo = InterSecPoint_2;
  else if(Field::FIELD.isInside(InterSecPoint_1) && Field::FIELD.isInside(InterSecPoint_2)){
    if(worldModel->distanceFromPointToBall(InterSecPoint_1) <
       worldModel->distanceFromPointToBall(InterSecPoint_2))
      PointToGo = InterSecPoint_1;
    else
      PointToGo = InterSecPoint_2;
  }
                                    
  AngDeg ang1 = worldModel->angleToBall();
  AngDeg ang2 = worldModel->angleToPoint(PointToGo);

  AngDeg angDiff = getAbsAngleDifference(ang1, ang2);
  int dashPower;
  if(angDiff > 100.0) dashPower = -100;
  else                dashPower =  100;

  // This is used to avoid positioning mistakes during a free kick
  // check wether the point to and the predicted Ball Positions are 
  // on the same side of the field
  /*
  if (playMode == PM_TheirFreeKick){
    if (PointToGo.y * ballPredPos.y < 0)
      PointToGo.y *= -1;
  }
  */

  skillPool->goToPoint(PointToGo, 1, dashPower);
  lookAround(true);
}

void GoalieBehaviour::ourPenGoaliePositioning(){
 
  PenaltySide ps = worldModel->getPenaltySide();
  Vector2D posGoalie = myself->getPosition();
  Vector2D penPos;
 
  if(fabs(posGoalie.getX()) < 35 || scoreGoal){
    //doPenalty();
    scoreGoal = true;
    return;
  }
  skillPool->faceNeckAndBodyToPoint(Vector2D(0.0,0.0));
  if(ps == PS_OurGoal){
    penPos = Vector2D(-54.0,-25.0);
  }else{
    penPos = Vector2D(54.0,25.0);
  }
  
  if(worldModel->distanceToPoint(penPos) > 1){
    skillPool->goToPoint(penPos,1,100);
  }
}

void GoalieBehaviour::goaliePenaltySetup(){

  PenaltySide ps = worldModel->getPenaltySide();
  //Vector2D posGoalie = myself->getPosition();
  Vector2D penPos;
  scoreGoal = false;

  if(ps == PS_OurGoal){
    penPos = Vector2D(-44.0,0.0);
  }else{
    penPos = Vector2D(44.0,0.0);
  }
 
  if(worldModel->distanceToPoint(penPos) > 1){
    skillPool->goToPoint(penPos,1,100);
  }else{
    skillPool->faceNeckAndBodyToPoint(Vector2D(0.0,0.0));
  }
}

void GoalieBehaviour::goaliePenaltyBehaviour(){

  PenaltySide ps = worldModel->getPenaltySide();
  const Vector2D &ballPos = worldModel->ball.getPosition();
  const Vector2D &myPos   = myself->getPosition();
  Vector2D penaltyPosition;


  if(ps == PS_OurGoal){
    penaltyPosition = Vector2D(-44.0,0.0);
  }else{
    penaltyPosition = Vector2D(44.0,0.0);
  }
  
  if (ballPos.isInvalid()){
    skillPool->scanFieldWithBody();
  }else{
    if (worldModel->distanceToPoint(ballPos) < 13){
      if(!worldModel->isBallKickableOppTeam()){
      skillPool->getBall(100,true,true);
      skillPool->catchBall();
      skillPool->faceOnlyNeckToBall();
      }else{
	skillPool->goToPoint(Vector2D(myPos.getX()-1,myPos.getY()),1,100);
      }
    }else{
      if (worldModel->distanceToPoint(penaltyPosition) > 1){
	skillPool->goToPoint(penaltyPosition,1,100);
      }
    }
  }
}


// shouldnt it be named "isSideAttacked" or some fuck like that?
// Should also reside in the WorldModel
bool GoalieBehaviour::sideAttack(){

  const Vector2D &ballpos = worldModel->ball.getPosition();
  Vector2D intPoint;
  int noCycles;
  bool intable = worldModel->interceptionPoint(*myself, intPoint, noCycles,true);
  
  if(((ballpos.getX() < -45) && ((ballpos.getY() > 5)||(ballpos.getY() < -5))) && (intable && Field::OWN_PENALTY_AREA.isInside(ballpos))){
    return true;
  }else{
    return false;
  }
}

// Should be the same as always
void GoalieBehaviour::sideAttackBehaviour(){

  const Vector2D &ballpos = worldModel->ball.getPosition();
  Vector2D goPoint;

  if(ballpos.getY() > 0){
    goPoint = Vector2D(-50.0, 5.0);
  }else{
    goPoint = Vector2D(-50.0,-5.0);
  }
  skillPool->goToPoint(goPoint,1,100);
  
  if (ballpos.isInvalid()){
    skillPool->faceNeckAndBodyToBall();
  }
}

bool GoalieBehaviour::shouldTryToCatchBall(){
  if (!Field::OWN_PENALTY_TOLERANCE_AREA.isInside(ball->getPosition()))
    return false;
  if (!worldModel->mayKickBall()){
    return false;
  }

  static float reach = Configuration::catchableAreaL;
  float dist  = worldModel->distanceToBall();
  float diff  = dist - reach;
  float predDist = myself->predictedPosition().distanceTo(ball->predictedPosition());
  float predDiff = predDist - reach;

  if (diff <= 0.25) return true;
  if (diff >  0.50) return false;
  if (predDiff > diff) {
    //DEBUG1_HARLEKIN("Catching just to be sure!");
    return true;
  }
  return false;
}


void GoalieBehaviour::playOnBehaviour(){
  const Vector2D *ballPos = ball->getPositionPt();

  if(ballPos->isInvalid()){
    scanFieldWithBodyGoalie();
    return;
  }

  // Time *now = worldModel->getTime();
  oneTime = false;
  
  if(shouldTryToCatchBall()){
    skillPool->catchBall();
    lookAround(true);
    return;
  }
  
  Vector2D intPoint;
  int noCycles;
  
  bool considerBall = false;

  
  if(Field::OWN_PENALTY_TOLERANCE_AREA.isInside(*ballPos)){ 
    considerBall = true;
  }
  if (!considerBall){
    bool intable = worldModel->interceptionPoint(*myself, intPoint, noCycles, true);
    if (intable && Field::OWN_PENALTY_TOLERANCE_AREA.isInside(intPoint)){
      considerBall = true;
    }
  }

  if (considerBall){
    bool getBall = false;
    const Player *fastest = NULL;
    
    bool isGoalShot = worldModel->isGoalShot();
    if (isGoalShot){
      getBall = true;
    }
    
    if (!getBall){
      fastest = worldModel->fastestInSetToBall(PS_Players);
      if (fastest != NULL && *fastest == *myself){
	getBall = true;
      }
    }
    
    if (getBall){
      if (skillPool->getBall(100, true, true)){
	DEBUG1_HARLEKIN("Not allowed to catch: PassOrKick() called.");
	passOrKick();
      }
      lookAround(true);
      return;      
    }
  }
  
  //goaliePositioning(goaliePosBeh(*ballPos), *ballPos);
  doPositioning();
  //goaliePositioning(goaliePosBeh(ballPos),ballPos);
  return;
}

void GoalieBehaviour::playOnBehaviour2(){
  
  const Vector2D &ballPos = ball->getPosition();
  //Time *now = worldModel->getTime();
  oneTime = false;
  Vector2D intPoint;
  int noCycles;
  bool intable = worldModel->interceptionPoint(*myself, intPoint, noCycles, true);
  //float distToBall = Field::GOAL_CENTER.distanceTo(ballPos);
  GoaliePosBehaviour beh  = goaliePosBeh(ballPos);

    
  if(ballPos.isInvalid()){
    scanFieldWithBodyGoalie();
    //goaliePositioning(GOALIENOTHING,ballPos);
    //DEBUG1("IST UNBKANNT DER BALL");
    return;
  }
  
  if(shouldTryToCatchBall()){
    skillPool->catchBall();
    skillPool->faceOnlyNeckToBall();
    //DEBUG1("CATCHBALL in SHOULDTRYTOCATCH");
    return;
  }
	
	 
  if(Field::OWN_PENALTY_TOLERANCE_AREA.isInside(ballPos) || (intable && Field::OWN_PENALTY_TOLERANCE_AREA.isInside(intPoint))){
    if(Field::OWN_PENALTY_TOLERANCE_AREA.isInside(intPoint)) DEBUG1("INTPOINT IST DRIN");
    if(Field::OWN_PENALTY_TOLERANCE_AREA.isInside(ballPos)) DEBUG1("BALL IST DRIN");
    const Player *fastest = worldModel->fastestInSetToBall(PS_Players);
    //const Player *secondfastest = worldModel->secondClosestInSetToBall(PS_Players);
     
    if(fastest != NULL){
      if(!fastest->isOpponent()){
      	skillPool->getBall(100,true);
      	skillPool->faceOnlyNeckToBall();
	DEBUG2("ICh habe getBall, Mitspieler ist schnellster, intPoint an : ", intPoint);
      	return;
      }else{
      	Line oppRunLine = Line::makeLineFromTwoPoints(fastest->getPosition(), fastest->predictedPosition(2));
	Line goalLine = Line::makeLineFromTwoPoints(Field::OWN_GOAL_POST_TOP_POS, Field::OWN_GOAL_POST_BOTTOM_POS);
	Vector2D crossPoint = goalLine.getIntersection(oppRunLine);
	if(crossPoint.getX() < myself->getPosition().getX()){
	  skillPool->goToPoint(crossPoint,0.5,-100);
	  skillPool->faceOnlyNeckToBall();
	}else{
	  skillPool->goToPoint(crossPoint,0.5, 100);
	  skillPool->faceOnlyNeckToBall();
	}
	DEBUG2(" schnellster ist ein Gegner, ich renne zu Punkt : ", crossPoint);
	return;
      }
    return;
    }else{  
    DEBUG1("MOOOOEPER");
    if(myself->getPosition().distanceTo(ballPos) <= 5){
       skillPool->getBall(100,true);
       skillPool->faceOnlyNeckToBall();
     }else{
       Line ballLine = Line::makeLineFromTwoPoints(ballPos, intPoint);
       Line goalLine = Line::makeLineFromTwoPoints(Field::OWN_GOAL_POST_TOP_POS, Field::OWN_GOAL_POST_BOTTOM_POS);
       Vector2D crossPoint = goalLine.getIntersection(ballLine);
       if(crossPoint.getX() < myself->getPosition().getX()){
	  skillPool->goToPoint(crossPoint,0.5,-100);
	  skillPool->faceOnlyNeckToBall();
	}else{
	  skillPool->goToPoint(crossPoint,0.5, 100);
	  skillPool->faceOnlyNeckToBall();
	}
     }
     return;
    }
  }
  DEBUG1_TBOGON("ich mach GoaliePositioning");
  goaliePositioning(beh,ballPos);
}

bool GoalieBehaviour::doPenaltyGoalie(){
  static int  nextToShoot = 2;    // Starting Player, Goalies are unstable: 
                                  // Dont use them unless needed
  static bool penaltyPerformed = false;
  int uNum = myself->getUniformNumber();
  PenaltySide ps = worldModel->getPenaltySide();
  Playmode    pm = worldModel->getPlaymode();
  int posTol    = 1; // ConfigFile::positioningTolerance;
  // int tolerance = 3;
  Vector2D posGoalie, posWait, posPen, posCorner;
  if (ps == PS_OurGoal){
    posGoalie = Vector2D(-52,  0);
    posWait   = Vector2D(-53,-22);
    posPen    = Vector2D(-42,  0);
    posCorner = Vector2D(-50,-32);
  }else{
    posGoalie = Vector2D( 52,  0);
    posWait   = Vector2D( 53, 22);
    posPen    = Vector2D( 42,  0);
    posCorner = Vector2D( 50, 32);
  }
  switch(pm){
    // Their penalty
  case PM_TheirPenaltySetup: 
  case PM_TheirPenaltyReady:
  case PM_TheirPenaltyTaken:
    playOnBehaviour();
    // doPenaltyGoaliePositioning(posGoalie.getX()); // Get ball
    return false;
  case PM_TheirPenaltyScore:
  case PM_TheirPenaltyMiss:
    return false; // Wait
    // Our penalty
  case PM_OurPenaltyTaken:
  case PM_OurPenaltyScore:
  case PM_OurPenaltyMiss:
    if (penaltyPerformed == false){
      nextToShoot = (nextToShoot % 11) + 1;
      penaltyPerformed = true;
    }
    break;
  case PM_OurPenaltySetup:
    penaltyPerformed = false;
    if (nextToShoot == uNum){
      return skillPool->goToPoint(posPen, posTol, 50);      
    }else{
      if(!skillPool->goToPoint(posWait, 0.5, 80))
	return skillPool->faceOnlyNeckToPoint(posCorner); // Face ball 
      else
	return skillPool->faceNeckAndBodyToPoint(posCorner); // Face ball 
    }
    break;
  case PM_OurPenaltyReady:
    if (nextToShoot == uNum){
      return doPenaltyScore();
    }else{
      if(!skillPool->goToPoint(posWait, 0.5, 80))
	return skillPool->faceOnlyNeckToPoint(posCorner); // Face ball      
      else
	return skillPool->faceNeckAndBodyToPoint(posCorner); // Face ball 
    }
    break;
  default:
    if(!skillPool->goToPoint(posWait, 0.5, 80))
      return skillPool->faceOnlyNeckToPoint(posCorner); // Face ball      
    else
      return skillPool->faceNeckAndBodyToPoint(posCorner); // Face ball 
  }
  return skillPool->faceNeckAndBodyToBall(); // Face ball
}

void GoalieBehaviour::doPenaltyGoaliePositioning(float x){

  Playmode pm = worldModel->getPlaymode();
  Vector2D posGoalie = Vector2D(x, 1);
  
  const Vector2D &ballPos = worldModel->ball.getPosition();
  Vector2D predBallPos = ball->predictedPosition();
  Line goalieLine = Line::makeLineFromTwoPoints(Vector2D(x,-10), Vector2D(x, 10));
  Line ballLine = Line::makeLineFromTwoPoints(ballPos, predBallPos);
  Vector2D intPoint = goalieLine.getIntersection(ballLine);

  switch(pm){
  case PM_TheirPenaltySetup:
    if (skillPool->goToPoint(posGoalie,0.5,50)){
      skillPool->faceOnlyBodyToPoint(Vector2D(x,-10)); 
    }
    skillPool->faceOnlyNeckToBall();
    break;
  case PM_TheirPenaltyReady:
  case PM_TheirPenaltyTaken:
    if(!worldModel->isBallCatchable()){
      if (skillPool->goToPoint(intPoint, 1, 100))
      skillPool->faceNeckAndBodyToBall();
    }else{
      skillPool->catchBall();
    }
    skillPool->faceOnlyNeckToBall();
    break;
  default:
    return;
  }
}

/*******************************************************************************/

void GoalieBehaviour::scanFieldWithBodyGoalie(){
  // How about some comments?
  static float angle;
  if (oneTime == false){
    angle = worldModel->angleToBallRel2Head();
    //DEBUG1_TBOGON(angle);
    oneTime = true;
  }
 
  if (angle < 0 )
    skillPool->faceOnlyBodyToPoint(myself->getPosition()+Vector2D(1,normalizeTo180Deg(myself->getBodyDir()-myself->getViewAngle()),POLAR));
  else
    skillPool->faceOnlyBodyToPoint(myself->getPosition()+Vector2D(1,normalizeTo180Deg(myself->getBodyDir()+myself->getViewAngle()),POLAR));  
}

// Should be removed
GoaliePosBehaviour GoalieBehaviour::goaliePosBeh(const Vector2D &ballPos){
  
  float distGoalBall = Field::GOAL_CENTER.distanceTo(ballPos);
  //float angle = worldModel->angleFromPointToPoint(Field::GOAL_CENTER,ballPos);  
  GoaliePosBehaviour beh;
  
  DEBUG4_TBOGON("distance = ", distGoalBall, " angle = ", angle);
  
  
  if(distGoalBall >= 34){ 
    beh = GOALIENOTHING;
    return beh;
  }   
  if(distGoalBall >= 25){
    //if(angle <= -50) beh = GOALIELEFT;
    //else if(angle >= 50) beh = GOALIERIGHT;
    /*else*/ beh = GOALIENORM;
    return beh;
  }
  if(distGoalBall < 25){
    //if(angle <= -60) beh = GOALIELEFT;
    //else if(angle >= 60) beh = GOALIERIGHT;
    /*else*/ beh = GOALIENORM;
    return beh;
  }
  DEBUG1("GoalieBehaviour::goaliePosBeh(...): reached end of non-void method");
  return GOALIENOTHING;
}

// Should reside in doPositioning()
void GoalieBehaviour::goaliePositioning(GoaliePosBehaviour beh, 
					const Vector2D &ballPos) { 
  goalieNormBehaviour(ballPos);
  return;
  
  switch(beh){
    case GOALIELEFT: goalieLeftBehaviour(ballPos);
                    break;
    case GOALIERIGHT: goalieRightBehaviour(ballPos);
                    break;
    case GOALIENORM: goalieNormBehaviour(ballPos);
                    break;
    case GOALIENOTHING: goalieNothing();
                    break;
    default: goalieNormBehaviour(ballPos);
            break;
    }   
  }

  
// Not needed
void GoalieBehaviour::goalieLeftBehaviour(const Vector2D &ballPos){
  
  float catchArea = Configuration::catchableAreaL;
  Vector2D top = Field::OWN_GOAL_POST_TOP_POS + Vector2D(1,-2);
  Vector2D bottom = Field::OWN_GOAL_POST_TOP_POS + Vector2D(catchArea,1);
  //Vector2D mitte = ((bottom - top)*0.5) + top;
  Line mainLine = Line::makeLineFromTwoPoints(top , bottom);
  Line intersectionLine = Line::makeLineFromTwoPoints(ballPos, Field::GOAL_CENTER);
  Vector2D intPoint = mainLine.getIntersection(intersectionLine);
  
  if(worldModel->distanceToPoint(intPoint) > 0.5){ 
    skillPool->goToPoint(intPoint,0.5,100);
    skillPool->faceOnlyNeckToBall();
  }else{
    goalieTurnBody(top,bottom,ballPos,GOALIELEFT);  
  }
}

// Not needed
void GoalieBehaviour::goalieRightBehaviour(const Vector2D &ballPos){

  float catchArea = Configuration::catchableAreaL;
  Vector2D bottom = Field::OWN_GOAL_POST_BOTTOM_POS + Vector2D(catchArea,-1);
  Vector2D top = Field::OWN_GOAL_POST_BOTTOM_POS + Vector2D(1,2);
  //Vector2D mitte = ((bottom - top)*0.5) + top;
  Line mainLine = Line::makeLineFromTwoPoints(top , bottom);
  Line intersectionLine = Line::makeLineFromTwoPoints(ballPos, Field::GOAL_CENTER);
  Vector2D intPoint = mainLine.getIntersection(intersectionLine);
  
  if(skillPool->goToPoint(intPoint,0.5,100)){
     skillPool->faceOnlyNeckToBall();
  }else{
    goalieTurnBody(top,bottom,ballPos,GOALIERIGHT);
  }
}
  
// playOnBehaviour?
void GoalieBehaviour::goalieNormBehaviour(const Vector2D &ballPos){
  float catchArea = Configuration::catchableAreaL;
  //Vector2D bottom = Field::OWN_GOAL_POST_BOTTOM_POS + Vector2D((int)catchArea, -1);
  //Vector2D top    = Field::OWN_GOAL_POST_TOP_POS + Vector2D((int)catchArea, 1);
  //Vector2D mitte = ((bottom - top)*0.5) + top;
  float x = -(Field::LENGTH/2 - catchArea);
  //Line mainLine = Line::makeLineFromTwoPoints(Vector2D(x, -2), 
  //				      Vector2D(x,  2));
  float toTop    = Field::OWN_GOAL_POST_TOP_POS.distanceTo(&ballPos);
  float toBottom = Field::OWN_GOAL_POST_BOTTOM_POS.distanceTo(&ballPos);

  // between 0.00 and 1.00
  float fact = toTop/(toTop+toBottom);
  // between -GOAL_WIDTH/2 and +GOAL_WIDTH/2
  float y = (Field::GOAL_WIDTH * (fact - 0.5f));
  Vector2D pos = Vector2D(x, y);
  
  //Line intLine = mainLine;
  //Vector2D point = (Field::OWN_GOAL_POST_BOTTOM_POS + Vector2D(0,-6));
  //intLine = Line::makeLineFromTwoPoints(ballPos, point);
  
  //Line intersectionLine = Line::makeLineFromTwoPoints(ballPos, Field::GOAL_CENTER.getX());
  //Vector2D intPoint = mainLine.getIntersection(intLine);
  
  if(skillPool->goToPoint(pos /*intPoint*/,0.5,100)){
     skillPool->faceNeckToBall();
  }else{
    skillPool->faceOnlyNeckToBall();
  }
}
  
// moep?
void GoalieBehaviour::goalieNothing(){
   
  float catchArea = Configuration::catchableAreaL;
  //Vector2D bottom = Field::OWN_GOAL_POST_BOTTOM_POS;
  //Vector2D top = Field::OWN_GOAL_POST_TOP_POS;
  //Vector2D mitte = ((bottom - top)*0.5) + top;
  
  if(skillPool->goToPoint(worldModel->field.getMarkerPosition(GOAL_L) +  Vector2D(catchArea,0),1,100))
    skillPool->faceNeckAndBodyToBall();
}

  
void GoalieBehaviour::goalieTurnBody(const Vector2D &top, 
				     const Vector2D &bottom, 
				     const Vector2D &ballPos, 
				     GoaliePosBehaviour beh){
    
  //float bodyDir = myself->getBodyDir();
   float angle = worldModel->angleFromPointToPoint(myself->getPosition(),ballPos); 

   switch(beh){
   case GOALIELEFT: if(angle >= -100){
     skillPool->faceOnlyBodyToPoint(top);
   }else{
     skillPool->faceOnlyBodyToPoint(bottom);
   } 
     break;
   case GOALIERIGHT: if(angle <= 100){
     skillPool->faceOnlyBodyToPoint(top);
   }else{
     skillPool->faceOnlyBodyToPoint(bottom);
   }
     break;
   case GOALIENORM: {
     if(angle >= 0){
       skillPool->faceOnlyBodyToPoint(bottom);
     }else{
       skillPool->faceOnlyBodyToPoint(top);
     }
   }
     break;
   default: ;
     break;
   }   
   skillPool->faceOnlyNeckToBall();
}


