#include "Agent.hh"
#include "debug.hh"
#include "PlannerBehaviour.hh"


/** The Constructor initializes the (mostly) private Variables*/
PlannerBehaviour::PlannerBehaviour() : Behaviour(){
    /*
  usePriorityPassing = ConfigFile::usePriorityPassing;
  runsFree = false;
  currentlyDribbling = false;
  kickInMinDelay = 0; 
  kickInScanDelay = 0;
  kickInState  = 0; 
    */
}

/** Destructor destroying previously instantiated Variables
    Currently it does nothing, but this may change over Time
*/
PlannerBehaviour::~PlannerBehaviour(){
}


/** Returns if I am currently at a good position to score a goal
    @return the vector where on should shoot if it is a good Point
    or a Vector2D with UNDEFINED_FLOAT as parameters
*/
Vector2D PlannerBehaviour::isAGoodPointToScore(){
  static const int GOAL_DIST_THRESHOLD = 16;
  static const int GOALIE_DIST_THRESHOLD = 8;
  Vector2D nullVector(UNDEFINED_FLOAT, UNDEFINED_FLOAT);
  Vector2D goalMid = worldModel->field.getMarkerPosition(GOAL_R);
  Vector2D leftGoalCorner  = worldModel->field.getMarkerPosition(FLAG_GRT);
  Vector2D rightGoalCorner = worldModel->field.getMarkerPosition(FLAG_GRB);
  // dont kick to the posts
  leftGoalCorner.setY(leftGoalCorner.getY() + 1);
  rightGoalCorner.setY(rightGoalCorner.getY() - 1);
  Vector2D ballPos = ball->getPosition();
  const Player* goalie = worldModel->getOtherTeamPlayerByUnum(1);

  if(goalie == NULL){
    return nullVector;
  }
  Vector2D goaliePos = goalie->getPosition();
  float distanceToGoalie = worldModel->distanceFromPointToPoint(myself->getPosition(), goaliePos);
  float distToGoal = worldModel->distanceFromPointToPoint(ballPos, goalMid);
  
  if(distToGoal < GOAL_DIST_THRESHOLD || distanceToGoalie < GOALIE_DIST_THRESHOLD){
    // write something in a File for learning issues (to come later)
  }else{
    Vector2D nullVector(UNDEFINED_FLOAT, UNDEFINED_FLOAT);
    return nullVector;
  }
  float angleGoalieShootPointLeft = fabs(worldModel->angleFromPointToPoint(goaliePos, leftGoalCorner));
  float angleGoalieShootPointRight = fabs(worldModel->angleFromPointToPoint(goaliePos, rightGoalCorner));
  float angleGoalieShootPointMid = fabs(worldModel->angleFromPointToPoint(goaliePos, goalMid));

  double uLeft = (angleGoalieShootPointLeft - 26.1)*0.043 + (distanceToGoalie - 9)*0.09 -0.2;
  double uRight = (angleGoalieShootPointRight - 26.1)*0.043 + (distanceToGoalie - 9)*0.09 -0.2;
  double uMid = (angleGoalieShootPointMid - 26.1)*0.043 + (distanceToGoalie - 9)*0.09 -0.2;

  double probabilityLeft = (1 / (1 + exp((-9.5*uLeft))))*100;
  double probabilityRight = (1 / (1 + exp((-9.5*uRight))))*100;
  double probabilityMid = (1 / (1 + exp((-9.5*uMid))))*100;
  if(probabilityLeft >= probabilityRight && probabilityLeft >= probabilityMid){
    if(probabilityLeft > 99){
      return leftGoalCorner;
    }else{
      return nullVector;
    }
  }else if(probabilityRight >= probabilityLeft && probabilityRight >= probabilityMid){
    if(probabilityRight > 99){
      return rightGoalCorner;
    }else{
      return nullVector;
    }
  }else if(probabilityMid >= probabilityLeft && probabilityMid >= probabilityRight){
    if(probabilityMid > 99){
      return goalMid;
    }else{
      return nullVector;
    }
  }else{
    return nullVector;
  }
}
  


    
  /*  float angleToGoal = worldModel->angleFromPointToPoint(ballPos, goalMid);
  float distanceToLeftCorner  = worldModel->distanceFromPointToPoint(ballPos, leftGoalCorner);
  float distanceToRightCorner = worldModel->distanceFromPointToPoint(ballPos, rightGoalCorner);
  float distToBall = worldModel->distanceToPoint(ballPos);
  float angleToBall = worldModel->angleToBallRel2Body();
  float ballSpeed = Configuration::maxpower * 
                    ball->calculateKickRate(distToBall, angleToBall) * 
                    Configuration::kickPowerRate;*/
  /*
  bool intersectedRight = worldModel->isIntersected_NH(rightGoalCorner, ballPos, ballSpeed);
  bool intersectedLeft  = worldModel->isIntersected_NH(leftGoalCorner,  ballPos, ballSpeed);
  bool intersectedMid   = worldModel->isIntersected_NH(goalMid,         ballPos, ballSpeed);
  if(!intersectedRight){
    DEBUG1_HARLEKIN("Behaviour::isAGoodPointToScore: Shooting right");
    return rightGoalCorner;
  }else if(!intersectedLeft){
    DEBUG1_HARLEKIN("Behaviour::isAGoodPointToScore: Shooting left");
    return leftGoalCorner;
  }else if(!intersectedMid){
    DEBUG1_HARLEKIN("Behaviour::isAGoodPointToScore: Shooting mid");
    return goalMid;
  }else{
    DEBUG1_HARLEKIN("Behaviour::isAGoodPointToScore: Everything is intersected");
    Vector2D nullVector(UNDEFINED_FLOAT, UNDEFINED_FLOAT);
    return nullVector;  
  }
  */
  /*
  if (angleToGoal >= 80) return nullVector;
  if (distToGoal  >= 22) return nullVector;

  float intersectedRight = worldModel->isNotIntersected_NH(rightGoalCorner, ballPos, ballSpeed);
  float intersectedLeft  = worldModel->isNotIntersected_NH(leftGoalCorner,  ballPos, ballSpeed);
  float intersectedMid   = worldModel->isNotIntersected_NH(goalMid,         ballPos, ballSpeed);

  DEBUG6_HARLEKIN("'Risks', R: ", intersectedRight, " M: ", intersectedMid, " L: ", intersectedLeft);

  if (intersectedRight != 0 &&
      intersectedRight >= intersectedLeft &&
      intersectedRight >= intersectedMid)
    return rightGoalCorner;
  if (intersectedLeft != 0 &&
      intersectedLeft >= intersectedRight &&
      intersectedLeft >= intersectedMid)
    return leftGoalCorner;
  if (intersectedMid != 0 &&
      intersectedMid >= intersectedRight &&
      intersectedMid >= intersectedLeft)
    return goalMid;
  return nullVector;
  
 }*/
 
 bool PlannerBehaviour::getDribbleSide(){
     
     int opponents =  worldModel->numInSetInCone(PS_Opponents,myself->getPosition(), myself->getPosition() + Vector2D(8, myself->getBodyDir(), POLAR), 1);
     
     if (opponents == 0){
       if (worldModel->angleToBallRel2Body() < 0) return false;
       else return true;
       }
     
     const Player* nme = worldModel->closestInSetToBall(PS_Opponents);
     
      if(nme == NULL){
        if (worldModel->angleToBallRel2Body() < 0) return false;
       else return true;
       }else{
         if(worldModel->angleToPointRel2Body(nme->getPosition()) < 0) return true;
         else return false;
         }
} 

/** Returns the potential Pass Candidate
    @return the Pass Candidate
    @param offensive if this is an offensive pass, default value is true
*/
const Player* PlannerBehaviour::getPassCandidate(bool offensive){
  return worldModel->getPassCand(offensive);
}

/** Returns the Pass Candidate after the Priority based Model
    @param offensive is this an offensive Player, will be removed
    @return the Pass Candidate or NULL
    @todo this is working but needs further testing
    @todo clean up and remove the hardcoded stuff from here
*/
const Player* PlannerBehaviour::getPrioritisedPassCandidate(bool offensive){
  /* TODOS:
     1. The Hardcoded Numbers should be read from a Config File
     2. A PlayerSet should be given as parameter, so
     only the interesting Pass Partners will be searched for
  */
  int uniformNumber = myself->getUniformNumber();
  if(uniformNumber == 1){
    return getPassCandidate(offensive);
  }
  // this is currently hardcoded, and shouild be read from a Config (or Formation class/conf?)
  // File or something else, testing stage only

  int passCandBNumbers[][5] = {{ 2, 3, 4, 5, 0}, // Goalie
			       { 6, 9, 3, 0, 0},     // Wing Defender (left/true)
			       { 6, 7, 8, 2, 4},     // CenterBackDefender (left/true)
			       { 8, 7, 6, 5, 3},     // CenterBackDefender (right/false)
			       { 8,11, 4, 0, 0},     // Wing Defender (right/false)
			       { 9,10, 7, 0, 0},  // Wing Midfielder (left/true)
			       { 9,11,10, 6, 8},    // Center Midfielder
			       {11,10, 7, 0, 0},   //  Wing Midfielder (right/false)
			       {10, 6, 0, 0, 0}, // Wing Attacker (left/true)
			       { 9,11, 0, 0, 0}, // Center Attacker
			       {10, 8, 0, 0, 0}};  // Wing Attacker (right/false)
  int length = 5;

  int* passCandNumbers = NULL;
  passCandNumbers = &passCandBNumbers[Formations::getCurrentFormation().getBehaviour(uniformNumber) - 1][0];
  
  vector<Player>::iterator i;
  vector<Player> players = worldModel->myTeam.getPlayers();

  vector<Player> posPassCands;

  //map<int, Player> passCandidateMap;
  const Player* passCandidate = NULL;
  float ownX = myself->getPosition().getX();
  for(i=players.begin();i!=players.end();++i){
    int unum = i->getUniformNumber();
    if (unum == uniformNumber) continue; // Dont pass to yourself, fool! 
    // Player UNum not known or info too old
    if (unum == UNDEFINED_FLOAT || !i->isGoodConfident(worldModel->getTime())) continue;
    int beha = Formations::getCurrentFormation().getBehaviour(unum);

    // START FOOLISH TESTING
    // is behaviour of this guy in the passCandNumbers-Array? Do we want to pass to this sort of player?
    bool noContinue = true;
    int y = 0;
    while (y < length && passCandNumbers[y] != 0){
      if (beha == passCandNumbers[y]){
	noContinue = false;
	break;	
      }
      y++;
    }
    // sounds funny :)
    if(noContinue){
      continue;
    }
    // END FOOLISH TESTING
    float otherX = i->getPosition().getX();
    if(ownX > otherX){
      continue;
    }
    if(worldModel->isIntersected(i->getPosition())){
      continue;
    }
    // always?!
    if(passCandidate == NULL){
      posPassCands.push_back(*i);
    }
  }
  
  if (posPassCands.size() == 0) return NULL;
  else return &posPassCands[0];

}

/** Returns if stopping the Ball is necessary
    @return true, if the Ball needs to be stopped, false othewise
*/
bool PlannerBehaviour::isStopBallNeeded(){
  if(currentlyDribbling){
    currentlyDribbling = false;
    return false;
  }
  static float powerThr = ConfigFile::stopBallPowerThr;
  float angle = worldModel->angleToBallRel2Body();
  float dist  = worldModel->distanceToBall();
  Vector2D ballVel = ball->getVelocity();
  float power = ballVel.getLength() / ball->calculateKickRate(dist, angle) / Configuration::kickPowerRate;
  
  if(power <= powerThr){
    return false;
  }else{
    return true;
  }
}

/** Returns if running free is a good choice, for a Player at the overgiven Position
    A Good Choice is the following:
    1. we have the Ball
    2. I'm currently intersected from the Player with the Ball
    3. If I run 10m in front, i will be no longer intersected
    @param player the Vector of the Player that should run free
    @return true, if it is a good choice, false otherwise
    @todo this is currently in a testing Stage DO NOT USE YET
*/
bool PlannerBehaviour::shouldRunFree(Vector2D player){
  if(!worldModel->hasBallTeam()){
    return false;
  }
  if(!worldModel->isIntersected(player)){
    return false;
  }
  float playerX = player.getX();
  float playerY = player.getY();
  Vector2D freeRunVector(playerX+10, playerY);
  if(worldModel->isIntersected(freeRunVector)){
    return false;
  }
  return true;
}

/** Returns if the Player needs to turn to a Pass Reciever
    @param passCand the PassCandidate
    @return true, if a turn is needed, false otherwise
*/
bool PlannerBehaviour::isTurnToPassCandNeeded(const Player* passCand){
  Vector2D targetPos = passCand->getPosition();
  if ((fabs(worldModel->angleToPointRel2Head(targetPos))>15)||
      (fabs(worldModel->angleToPointRel2Head(targetPos))>15)){
    return true;
  }
  return false;
}

/** Returns if the currentSeeTime is equal to the currentTime
    @return true if the currentTime is equal to the last See
*/
bool PlannerBehaviour::isLastSeeAtCurrentTime(){
  if(worldModel->getLastSeeSSTime()!=worldModel->getTime()){
    return false;
  }
  return true;
}

/** Returns if a Dribbling by a Wing Attacker is possible, e.g. 
    his way is free to the Goal etc.
    @param dribblePoint the Point to which the wing Attacker would 
    dribble if it is possible
    @return true if a dribble is possible, false otherwise
*/
bool PlannerBehaviour::isWingAttackerDribblePossible(Vector2D dribblePoint){   
  /* TODOS:
     1. The Threshold should be read from a Config File
  */
  float distanceThreshold = 4.0f;
  vector<Player>::iterator i;
  vector<Player> players = worldModel->theirTeam.getPlayers();
  for(i=players.begin();i!=players.end();++i){
    if (!i->isGoodConfident(worldModel->getTime())) {
      continue;
    }
    float distanceToPlayer = worldModel->distanceFromPlayerToPoint(*i, dribblePoint);
    if(distanceToPlayer < distanceThreshold){
      return false;
    }
  }  
  return true;
}

/** Checks if a Pass to the Midfielder is possible
    @param empty Vector2D Pointer. If true is returned, this contains the Coordinates where one should pass to.
    If false is returned this will be a NULL Pointer !
    @return true if the Pass is possible, false otherwise
 */
bool PlannerBehaviour::isPassToMidfielderPossible(Vector2D* midfielderPosition){
  int uniformNumber = myself->getUniformNumber();
  float dribbleAreaX = ConfigFile::dribbleAreaX;
  float passAreaX = ConfigFile::passAreaX;
  float ballX = ball->getPosition().getX();
  //if im not the center attacker and am in the "pass area" then pass to
  //the estimated position of the center attacker if it is not inhterected
  if(Formations::getCurrentFormation().getBehaviour(uniformNumber) != B_CenterAttacker &&
     ballX >= passAreaX && ballX <= dribbleAreaX){
    //if(uniformNumber != UNUM_MIDDLEATTACKER && ballX >= passAreaX && ballX <= dribbleAreaX){
    Vector2D pos = worldModel->passToMidfielder(*myself);
    midfielderPosition->setX(pos.getX());
    midfielderPosition->setY(pos.getY());
    if(midfielderPosition->getX() != UNDEFINED_FLOAT && midfielderPosition->getY() != UNDEFINED_FLOAT){
      return true;
    }
  }
  midfielderPosition = NULL;
  return false;
}

/** Handles the Behaviour if the Agent has the Ball (the Ball is kickable)*/
void PlannerBehaviour::playOnWithBall(){
  int myBehaviour = Formations::getCurrentFormation().getBehaviour(myself->getUniformNumber());

  if(isStopBallNeeded()){
    APL_POINT3_MAJUNOR(myself->getPosition(), APL_L1, 255, 0, 0);
    skillPool->stopBall();
    return;
  }
  if(!ConfigFile::useIntersectedScoring){
    if (worldModel->isAtGoodPointToScore(*myself)){
      skillPool->scoreGoal();
      return;
    }
  }else{
    Vector2D kickVector = isAGoodPointToScore();
    if(kickVector.getX() != UNDEFINED_FLOAT && kickVector.getY() != UNDEFINED_FLOAT){
      APL_POINT3_MAJUNOR(kickVector, APL_L1, 255, 0, 0);
      skillPool->kickToPoint(kickVector, 100, true);
      return;
    }
  }
  const Player* passCand = NULL;
  if(usePriorityPassing){
    passCand = getPrioritisedPassCandidate();
    if(passCand != NULL)
      {
	APL_LINE3_MAJUNOR(myself->getPosition(), passCand->getPosition(),APL_L1, 0, 0, 255);
      }
  }else{
    passCand = getPassCandidate();
  }
  if(passCand != NULL){
    //    if(isLastSeeAtCurrentTime()){
      if(!isTurnToPassCandNeeded(passCand)){
	APL_LINE3_MAJUNOR(myself->getPosition(), passCand->getPosition(), APL_L1, 255, 0, 0);
	skillPool->passBallToPlayer(passCand);
	return;
      }else{
	skillPool->turnWithBallTo(worldModel->angleFromPointToPoint(myself->getPosition(), passCand->getPosition()));
	return;
      }
      // }
  }
  //  if(myself->getUniformNumber() == 9 || myself->getUniformNumber() == 11){
  if(myBehaviour == B_WingAttackerLeft  || 
     myBehaviour == B_WingAttackerRight ||
     myBehaviour == B_CenterAttacker){
    Vector2D whereToDribble;
    bool dribbleSide;
    //worldModel->getBestDribblePoint(*myself, whereToDribble);
    if (myself->getPosition().getY() >= 0){
      whereToDribble = Field::OPP_GOAL_POST_TOP_POS;
      dribbleSide = true;
    }else{
      whereToDribble = Field::OPP_GOAL_POST_BOTTOM_POS;
      dribbleSide =  false;
    } 
    skillPool->smartDribbleToPoint(whereToDribble, 100, dribbleSide);
    currentlyDribbling = true;
    return;    
  }
  if(myBehaviour != B_CenterAttacker){
    Vector2D midfielderPosition(UNDEFINED_FLOAT, UNDEFINED_FLOAT);
    if(isPassToMidfielderPossible(&midfielderPosition)){
      APL_LINE3_MAJUNOR(myself->getPosition(), midfielderPosition, APL_L1, 0, 255, 0);
      skillPool->passBallToPos(midfielderPosition);
      return;
    }
  }

  // START TESTING STUFF

  if(!worldModel->isAttacked(*myself)){
    float myBodyDir = myself->getBodyDir();
    if(myBodyDir > 90 || myBodyDir < -90){
      float distanceToPoint = worldModel->distanceToPoint(worldModel->field.getMarkerPosition(GOAL_L));
      if(distanceToPoint < 60 ){
	skillPool->turnWithBallTo(0);
	return;
      }
    }
  }

  // END TESTING STUFF

  // - otherwise attackers would kick the ball behind the goal...
  // - document it :-)
  if (myBehaviour >= B_WingMidfielderLeft){
    // skillPool->passBallToPos(Vector2D(30, 0));
    APL_LINE3_MAJUNOR(myself->getPosition(), worldModel->calcSBSPPosition(10), APL_L1, 255, 255, 255);
    skillPool->passBallToPos(worldModel->calcSBSPPosition(10));
    return;
  }
  APL_ELLIPSE_MAJUNOR(Vector2D((myself->getPosition().getX() -0.5), (myself->getPosition().getY() - 0.5)), 1, 1);
  skillPool->kickAway();
  //skillPool->faceOnlyNeckToBall();
  skillPool->lookAround();
  return;
}

/** Handles the behaviour if the Agent does not have the Ball (the Ball is not kickable)*/
void PlannerBehaviour::playOnWithoutBall(){
  // if the player looses the ball while dribbling he would
  // otherwise still think to be dribbling if he gets into the
  // reach of another ball... later... uhm... and would not stop it...
  currentlyDribbling = false;

  const Player *fastest = worldModel->fastestInSetToBall(PS_Teammates);
  if(fastest != NULL) {
    if(*fastest == *myself){
      if(skillPool->getBall()){
	skillPool->faceOnlyNeckToBall();
	skillPool->stopBall();
      }else{
	//skillPool->faceOnlyNeckToBall();
      skillPool->lookAround();
      }
      return;
    }
  }else{  //no one is fastest to ball
    if(worldModel->isClosestTeammateToBall(*myself)){
      if(skillPool->getBall()){
	skillPool->faceOnlyNeckToBall();
	skillPool->stopBall();
      }else{
	//skillPool->faceOnlyNeckToBall();
      skillPool->lookAround();
      }
      return;
    }
  }
  doPositioning();
}


// SHIT EINFAHC MAL NACHDENKEN!!!
void PlannerBehaviour::debugHowMuchPlayers(){
  vector<Player>::iterator i;
  int players = 0;
  if (worldModel->myTeam.getPlayers().size() < 2) return;

  for (i = worldModel->myTeam.getPlayers().begin(); i < worldModel->myTeam.getPlayers().end(); i++){
    if (i->getUniformNumber() == myself->getUniformNumber()) continue;
    if (!i->isGoodConfident(worldModel->getTime())) continue;
    players++;
  }
  players = 0;
  for (i = worldModel->theirTeam.getPlayers().begin(); i < worldModel->theirTeam.getPlayers().end(); i++){
    if (!i->isGoodConfident(worldModel->getTime())) continue;
    players++;
  }
}

/** The standard play On Behaviour calls specific Behaviours*/
void PlannerBehaviour::playOnBehaviour(){
  static Time now;
  now = worldModel->getTime();
  /*
  // debugHowMuchPlayers(); // what shit is that?!
  if(!ball->isGoodConfident(now)){
    skillPool->scanFieldWithBody();
    return;
  }
  if (worldModel->isBallKickable()){
    playOnWithBall();
    return;
  }else{
    playOnWithoutBall();
    return;
  }*/
  #ifdef USE_PLANNER
  Agent::getPlannerCore()->think();
  skillPool->lookAround();  
  #endif
}

/** The general Behaviour that tells the Agent what to do in the current Situation, e.g Free Kick, play on*/
void PlannerBehaviour::generalBehaviour(){
  //the following number are not cycles, but the number of calls of this
  //function, ratio to the cycles: 5:3 im default view mode
  static int passMinDelay  = ConfigFile::passMinDelay;
  static int passScanDelay = ConfigFile::passScanDelay;
  static int walkPower     = ConfigFile::walkPower;
  static bool fsUsed       = ConfigFile::useFullstate;
  static bool penaltyPerformed = false;
  static int nextToShoot = 2;

  static double prefferedDistance = Configuration::getPlayerSize(myself->getType()) +
                                    Configuration::getKickableMargin(myself->getType())/2 +
                                    Configuration::ballSize;
  static Vector2D penShootPoint = Vector2D(40, 0);
  static Vector2D opponent_goal_mid = worldModel->field.getMarkerPosition(GOAL_R);
  static Vector2D opponent_rc = worldModel->field.getMarkerPosition(FLAG_RB);
  static Vector2D opponent_lc = worldModel->field.getMarkerPosition(FLAG_RT);
  
  /*   Test fuer Anstoss */
  static Vector2D own_goal = worldModel->field.getMarkerPosition(FLAG_PLC);
  Playmode pm = worldModel->getPlaymode();
  Vector2D ballPos = ball->getPosition();

  int stuckedForCycles = myself->bodyInfo.getTackleExpires();
  if (stuckedForCycles > 0) {
    DEBUG1_HARLEKIN("generalBehaviour::Standing up...");
    return; 
  }

  switch(pm){
  case PM_BeforeKickOff:
    if( !fsUsed ){
      if(!worldModel->haveOptimalSeeTimes()){
	skillPool->doSSProfiling();
	return;
      }
    } 
    if(myself->getPosition().distanceTo(myself->getInitPosition()) > 1){
      skillPool->moveToInitialPosition();
    }else if(!ball->isPositionKnown()){
      skillPool->scanFieldWithBody();
    }else{
      skillPool->faceNeckAndBodyToBall();
    }
    break;
    case PM_PlayOn:
      //reset all flags
      resetKickIn();
      playOnBehaviour();
      break;
    case PM_OurKickOff:   //kick off in the middle
      if(!ball->isPositionKnown()){
	skillPool->scanFieldWithBody();
      }else if(worldModel->isClosestTeammateToBall(*myself)){
	if(skillPool->getBall(walkPower, false)){
	  skillPool->kickToPoint(Vector2D(-20,0), 90 * ball->calculateKickRate(worldModel->distanceToBall(), worldModel->angleToBallRel2Body()));
	}
      }else{
	skillPool->worldScan();
      }      
      break;
  case PM_TheirKickOff:
    if(myself->getPosition().distanceTo(myself->getInitPosition()) > 1){
      skillPool->moveToInitialPosition();
    }else{
      skillPool->faceNeckAndBodyToBall();
    }
    break;
  case PM_OurIndirectFreeKick:
  case PM_TheirFreeKickFault: // PM_OurFreeKick will follow
  case PM_TheirBackPass: // dito
  case PM_OurFreeKick:  //after the goalie has catched the ball,
    if(skillPool->worldScan(100)){
	//among other things
      if(worldModel->myTeam.getGoalie() != NULL){ // if I know the goalie
	if(worldModel->distanceFromPlayerToBall(*worldModel->myTeam.getGoalie()) < 2.0){ // and he is near the ball
	  if(Field::OWN_PENALTY_AREA.isInside(ballPos)){ // and the ball is inside the penalty area
	    if(myself->getPosition().getX() > (worldModel->getOffsideX()-3) ){ // and I am in a offside Position
	      doPositioning();
	      break;
	    }
	    if(!offerAsPassCandidate()){
	      doPositioning();
	    }
	    break;
	  }
	}
      }
      
      if( worldModel->isMyHomePosClosestToPos(ballPos) ){
	Vector2D freeKickPosition = ballPos + Vector2D(prefferedDistance,worldModel->angleFromPointToPoint(opponent_goal_mid, ballPos),POLAR); 
	if(skillPool->goToPoint(freeKickPosition, prefferedDistance/2, walkPower)){
	  if(skillPool->faceOnlyBodyToPoint(opponent_goal_mid)){
	    if(!performKickIn(true, passMinDelay, passScanDelay, 1.2)){
	      if(myself->getPosition().getX() > 0){
		kickToMidfielder();
	      }else{
		skillPool->kickAway();
	      }
	    }
	  }
	}
      }else{
	doPositioning();
      }
    }
    break;
  case PM_OurKickIn:    //unser Einwurf
    kickInBehaviour();
    break;
  case PM_TheirIndirectFreeKick:
  case PM_TheirFreeKick:
  case PM_OurBackPass: // PM_TheirFreeKick will follow
  case PM_OurFreeKickFault: // dito
  case PM_TheirKickIn:   //deren Einwurf
    doPositioning();
    break;
  case PM_OurCornerKick:
    if(skillPool->worldScan()){
      if( worldModel->isMyHomePosClosestToPos(ballPos) ){
	if(skillPool->getBall(walkPower, false)){
	  if(skillPool->faceOnlyBodyToPoint(penShootPoint)){
	    if(!performKickIn(true, passMinDelay, passScanDelay, 1.5)){
	      skillPool->kickToPoint(penShootPoint, 100);
	    }
	  }
	}
      }else if(skillPool->goToPositionOurCKick()){
	skillPool->faceNeckAndBodyToBall();
      }
    }
    break;
  case PM_TheirCornerKick:
    if(skillPool->goToPositionTheirCKick()){
      skillPool->faceNeckAndBodyToBall();
    }
    break;
  case PM_OurGoalKick:
    if(myself->getPosition().getX() > (worldModel->getOffsideX()-3) ){
      doPositioning();
      break;
    }
    if(!offerAsPassCandidate()){
      doPositioning();
    }
    break;
  case PM_TheirGoalKick:
    doPositioning();
    break;
  case PM_OurGoal:
  case PM_TheirGoal:
    if(myself->getPosition().distanceTo(myself->getInitPosition()) > 1){
      skillPool->moveToInitialPosition();
    }else{
      skillPool->faceNeckAndBodyToPoint(opponent_goal_mid);
    }
    break;
  case PM_OurOffside:
  case PM_TheirOffside:  //lasts a bit, after it comes freekick
    doPositioning();
    break;
  case PM_OurPenaltyKick:        //penalty kick
    if(skillPool->worldScan()){
      if( worldModel->isMyHomePosClosestToPos(ballPos) ){
	if(skillPool->getBall(walkPower, false)){
	  if(skillPool->faceOnlyBodyToPoint(opponent_goal_mid)){
	    if(!performKickIn(true, passMinDelay, passScanDelay, 1.2)){
	      skillPool->scoreGoal();
	    }
	  }
	}
      }else{
	if(myself->getPosition().getX() > (worldModel->getOffsideX()-3) ){
	  doPositioning();
	  break;
	}
	if(!offerAsPassCandidate()){
	  doPositioning();
	}
      }
    }
    break;
  case PM_TheirPenaltyKick:
    doPositioning();
    break;
  case PM_OurGoalieCatchBall:
    doPositioning ();
    break;
  case PM_TheirGoalieCatchBall:
    doPositioning ();
    break;    
  case PM_OurPenaltySetup:
    penaltyPerformed = false;
    if(myself->getUniformNumber() == nextToShoot)
      {
	skillPool->getBall();
      }
    else
      {
	skillPool->goToPoint(Vector2D(0, 0), 3.0,80.0);
      }
    break;
  case PM_OurPenaltyTaken:
  case PM_OurPenaltyReady:
    if(myself->getUniformNumber() == nextToShoot)
      {
	doPenalty();
      }
    //    skillPool->doPenalty();
    break;
  case PM_OurPenaltyScore:
  case PM_OurPenaltyMiss:
    if (penaltyPerformed == false)
      {
	nextToShoot = (nextToShoot % 11) + 1;
	penaltyPerformed = true;
      }
    break;
  case PM_TheirPenaltyTaken:
  case PM_TheirPenaltyScore:
  case PM_TheirPenaltyMiss:
  case PM_TheirPenaltyReady:
    break;
  case PM_TheirPenaltySetup:
    skillPool->goToPoint(Vector2D(0, 0), 3,80);
    break;
  default:
    DEBUG2("Behaviour::fieldPlayerGeneralBehaviour():\
             not supported playmode appeared: ", pm);
  }
  
  // report the ball data in every cycle!
  if (worldModel->isClosestToBall(*myself)  && 
      ball->isHighConfident(worldModel->getTime()) &&
      pm != PM_BeforeKickOff){
    skillPool->reportWorldModelStatus();
  }
}

/** Handles Penalty Shootout*/
void PlannerBehaviour::doPenalty()
{
  PenaltySide ps = worldModel->getPenaltySide();
  if(!skillPool->getBall())
    {
      return;
    }
  float myX = myself->getPosition().getX();
  Vector2D whereToDribble(UNDEFINED_FLOAT, UNDEFINED_FLOAT);
  if (isAGoodPenaltyShotPoint())
    {
      if(ps != PS_OurGoal)
	{
	  skillPool->scoreGoal();
	}
      else
	{
	  skillPool->scoreGoal(true);
	}
      return;
    }
  if(myX < 25)
    {
      whereToDribble.setX(myself->getPosition().getX() + 20);
      whereToDribble.setY(myself->getPosition().getY());
      if (ps == PS_OurGoal)
	{
	  whereToDribble.setX(whereToDribble.getX() - 40);
	}
      skillPool->simpleDribbleToPoint(whereToDribble, 30); 
      return;
    }
  else
    {
      whereToDribble.setX(myself->getPosition().getX() + 10);
      whereToDribble.setY(myself->getPosition().getY());
      if (ps == PS_OurGoal)
	{
	  whereToDribble.setX(whereToDribble.getX() - 20);
	}
      // do other dribbling here
      skillPool->simpleDribbleToPoint(whereToDribble, 30); 
      return;
    }
}

void PlannerBehaviour::kickInBehaviour( int wantedRecoveryDelay ) {
  // counter for stamina-recovery delay
  kickInMinDelay++;

  // returns true, if worldScan finished (up to 30 cycles in the past)
  if ( skillPool->worldScan(30) ) { 

    // do I have to perform the kick in?
    if ( worldModel->isMyHomePosClosestToPos( ball->getPosition() ) ) {

      Vector2D shootingPos = ball->getPosition();
      if ( shootingPos.getY() > 0 ) {
	shootingPos += Vector2D( 0, 0.5 );
      } else {
	shootingPos -= Vector2D( 0, 0.5);
      }
      // go to the ball - returns true, if we are close to the ball
      // tolerance is 0.2m
      if ( skillPool->goToPoint( shootingPos, 0.5, ConfigFile::walkPower ) ) {
	
	// calc the angle to turn to
	float posX = myself->getPosition().getX() + (Field::LENGTH / 2.0);
	//  at our corner -> -35, at their corner -> -125
	float angleToFaceTo = -35 + ((-90*posX) / Field::LENGTH);
	
	// if we kick in from the other side, negate the angle
	if ( myself->getPosition().getY() < 0 ) {
	  angleToFaceTo = -angleToFaceTo;
	}
	
	// calc the point to face to
	Vector2D pointToFace = myself->getPosition() + Vector2D(20, angleToFaceTo, POLAR);

	if ( skillPool->faceNeckAndBodyToPoint( pointToFace ) ) {
	  // we are now facing the right point
	  DEBUG2_PAGRA("kickInBehaviour: pointToFace: ", pointToFace );

	  // everything's ready for kickIn, wait till recovery is finished
	  if ( kickInMinDelay < wantedRecoveryDelay ) {
	    DEBUG1_PAGRA("kickInBehaviour: delaying game for recovery.");
	    return;
	  }
	  
	  // is our worldmodel up to date (see-message arrived this cycle)?
	  if ( isLastSeeAtCurrentTime() ) {
	    
	    // get the best passcand (backpasses are allowed, if necessary)
	    const Player *passCand = getPrioritisedPassCandidate();
	    
	    if ( passCand != NULL ) {
	      // passCand found
	      // make pass
	      Vector2D targetPos = passCand->getPosition();
	      //unused: Vector2D myPos = myself->getPosition();
	      skillPool->kickToPoint(targetPos, 100);
	      // skillPool->passBallToPlayer( passCand );
	      DEBUG4_PAGRA( "kickInBehaviour: passing ball to Player ", 
			    passCand->getUniformNumber(), " Pos: ", passCand->getPosition() );
	      
	    } else {
	      // if near to the enemy goal kick to midfielder 
	      // (kickAway doesn't work there correctly)
	      if ( myself->getPosition().getX() > 0 ) {
		skillPool->kickToPoint(Vector2D(31.5, 0), 100);
		// skillPool->kickToMidfielder();
		DEBUG1_PAGRA( "kickInBehaviour: passing ball to midfielder" );
	      } else {
		skillPool->kickAway();
		DEBUG1_PAGRA( "kickInBehaviour: kick away" );
	      }
	    }
	  }
	}
      }
    }else{
      if(myself->getPosition().getX() > (worldModel->getOffsideX()-3) ){
	doPositioning();
	return;
      }
      // I have to offer me as pass candidate
      if ( !offerAsPassCandidate() ) {
	// offering did not succeed -> go to standard position
	doPositioning();
      }
    }
  } // Worldscan finished
}


/** Determines whether the current Player Position is god to score a goal on penalties
    @return true if it is a good point, false otherwise
*/
bool PlannerBehaviour::isAGoodPenaltyShotPoint()
{
  PenaltySide ps = worldModel->getPenaltySide();
  if(ps != PS_OurGoal)
    {
      return worldModel->isAtGoodPointToScore(*myself);
    }
  else
    {
      return worldModel->isAtGoodPointToScore(*myself, true);
    }
  return false;
}

/*******************************************************************************/
bool PlannerBehaviour::goToStrategicPosition(){
  static float posTolerance = ConfigFile::positioningTolerance;

  Playmode p = worldModel->getPlaymode();
  Formation f = Formations::getCurrentFormation();
  Vector2D pos = myself->getHomePosition();
  Vector2D myPos = myself->getPosition();
  Vector2D ballPos = ball->getPosition();
  Vector2D relBallPos = ballPos - myself->getPosition();

  int unum = myself->getUniformNumber();


  //ball attraction factors
  float factorX = f.getAttractionX(unum);
  float factorY = f.getAttractionY(unum);

  //dependence on the ball position
  pos.setX(pos.getX() + relBallPos.getX()*factorX);
  pos.setY(pos.getY() + relBallPos.getY()*factorY);

  //avoid the boundaries of the field (width of field = 68.0)
  if (fabs(pos.getY()) > 30.0){ 
    pos.setY(pos.getY() - 0.5*factorY*relBallPos.getY());
  }

  // make sure the defence goes behind the ball
  // dont run behind the goalie
  if ((p != PM_OurFreeKick) && (p != PM_OurGoalKick)){
    if (f.getBehindBall(unum) && ballPos.getX() < pos.getX()){ 
      pos.setX(ballPos.getX());
    }
  }

  //make sure no one goes closer to the enemies goal as the ball if the enemy
  //is kicking an offside free kick
  if (p == PM_OurOffside) pos.setX(min(pos.getX(), ballPos.getX()));

  //monitor an maximum and a minimum of the x-coord of the strategic position
  pos.setX(min((double)pos.getX(), f.getMaxX(unum)));
  pos.setX(max((double)pos.getX(), f.getMinX(unum)));

  // Consider offside...
  pos.setX(min(pos.getX(), worldModel->getOffsideX()-3));

  /*** Hh?
  if(ConfigFile::useNearingMidfielder){
    if(myself->getUniformNumber() == 9){
      leftWingAttackerBehaviour->shouldNearMidfielderPosition(&pos);
    }else if(myself->getUniformNumber() == 11){
      rightWingAttackerBehaviour->shouldNearMidfielderPosition(&pos);
    }
  }
  */
  
  // try to cover a enemy play
  // stay away from the ball to save stamina
  if ((p == PM_TheirFreeKick)         || 
      (p == PM_TheirIndirectFreeKick) || 
      (p == PM_TheirKickOff)          || 
      (p == PM_TheirKickIn)){

    Vector2D posI1, posI2;
    Line myWay = Line::makeLineFromTwoPoints(myPos, pos);
    Circle boundary = Circle(ballPos, (float)10 /*radius*/);
    
    // Stay away if:
    //  -my way at least crosses the circle once and
    //  -the ball/circle lies on my way (is not behind me)
    //  -I am in the circle or I want to go into the circle
    if ((myWay.getCircleIntersectionPoints(boundary, &posI1, &posI2) != 0) &&
	((myWay.isInBetween(ballPos, myPos, pos)) || 
	 (boundary.isInside(pos))                 ||
	 (boundary.isInside(myPos)) )){
      if (myPos.distanceTo(posI1) < myPos.distanceTo(posI2))
	return skillPool->goToPoint(posI1, 2 /*posTolerance*/, 70);
      else
	return skillPool->goToPoint(posI2, 2 /*posTolerance*/, 70);
    }
  }

  //go to this determined position
  // speed depends on stamina-recovery 
  return skillPool->goToPoint(pos, posTolerance,  80); // REVISIT: Walk slower if you are tired 
                                           // or faster if you are close to the ball
}


/*******************************************************************************/
void PlannerBehaviour::doPositioning(){
  if(ball->isPositionKnown()){
    if(goToStrategicPosition()){
      if (worldModel->ball.isHighConfident(worldModel->getTime())){
	skillPool->faceOnlyBodyToBall();
	skillPool->faceOnlyNeckToPoint(Vector2D(myself->getPosition().getX(),0));
          skillPool->lookAround();
      }else{
	skillPool->faceNeckAndBodyToBall();
      }
    }else{
      if (worldModel->ball.isHighConfident(worldModel->getTime())){
	//faceOnlyNeckToPoint(Vector2D(myself->getPosition().getX(),0));
              skillPool->lookAround();
      }else{
	//faceOnlyNeckToBall();
              skillPool->lookAround();
      }
    }
  }
  else
    skillPool->scanFieldWithBody();
}

/*******************************************************************************/
static bool noPassCand = false;
bool PlannerBehaviour::passIfHaveACandidate(bool offensive, float endSpeed){
  // bool dopass = false;
  const Player *pc = worldModel->getPassCand(offensive);
  if ( pc != NULL ){
    skillPool->passBallToPlayer(pc, endSpeed);
    noPassCand = false;
    return true;
  }else{
    noPassCand = true;
    return false;
  }
}

void PlannerBehaviour::resetKickIn()
{
  kickInState = 0;
  kickInMinDelay = 0;
  kickInScanDelay = 0;
}

/*******************************************************************************/
bool PlannerBehaviour::performKickIn(bool offensive, int minDelay,
                              int scanDelay, float endSpeed){

  skillPool->scanFieldWithNeck();
  if( kickInMinDelay++ < minDelay )
    return true;

  if( kickInScanDelay++ >= scanDelay )
    return false;

  return passIfHaveACandidate(offensive, endSpeed);
}

/** Kicks the Ball to a good Position for the Midfielder
    This method should only be used as a failover, as it is 
    not as good as real passing
*/
void PlannerBehaviour::kickToMidfielder()
{
    Vector2D markerPosition = worldModel->field.getMarkerPosition(FLAG_PRC);
    float desiredX = markerPosition.getX() - 5;
    float desiredY = markerPosition.getY();
    Vector2D desiredPosition(desiredX, desiredY);
    skillPool->kickToPoint(desiredPosition, 90);
}


/*******************************************************************************/

void PlannerBehaviour::goaliePositioning(){
  //ermittle den positionierungspunkt
  //beispielsweise duch Schnittpunkt eines um das Tor gelegten Kreises mit
  //einer Geraden zu der Ballpos
  Vector2D ballPos   = ball->getPosition();
  float distGoalBall = Field::GOAL_CENTER.distanceTo(ballPos);
  int cycles = (int)(distGoalBall / Configuration::ballSpeedMax);
  Vector2D ballPredPos = ball->predictedPosition(cycles);


  Line line = Line::makeLineFromTwoPoints(ballPredPos, Field::GOAL_CENTER);
  Vector2D InterSecPoint_1;
  Vector2D InterSecPoint_2;
  Vector2D PointToGo;
  line.getCircleIntersectionPoints(Field::GOALIE_POSITIONING_CIRCLE,
				   &InterSecPoint_1, &InterSecPoint_2);


  //der positioning circle hat mit der (geraden zw. ball und tormitte) zwei
  //schnittpunkte; der goalie positioniert sich an dem, der auf dem feld
  //liegt. im unguenstigen falle liegen beide auf dem feld: dann positioniere
  //dich an dem punkt, der naeher zum ball ist
  if(Field::FIELD.isInside(InterSecPoint_1) && !Field::FIELD.isInside(InterSecPoint_2))
    PointToGo = InterSecPoint_1;
  else if(!Field::FIELD.isInside(InterSecPoint_1) && Field::FIELD.isInside(InterSecPoint_2))
    PointToGo = InterSecPoint_2;
  else if(Field::FIELD.isInside(InterSecPoint_1) && Field::FIELD.isInside(InterSecPoint_2)){
    if(worldModel->distanceFromPointToBall(InterSecPoint_1) <
       worldModel->distanceFromPointToBall(InterSecPoint_2))
      PointToGo = InterSecPoint_1;
    else
      PointToGo = InterSecPoint_2;
  }
                                    
  AngDeg ang1 = worldModel->angleToBall();
  AngDeg ang2 = worldModel->angleToPoint(PointToGo);

  AngDeg angDiff = getAbsAngleDifference(ang1, ang2);
  int dashPower;
  if(angDiff > 100.0) dashPower = -100;
  else                dashPower =  100;

  skillPool->goToPoint(PointToGo, 1, dashPower);
  skillPool->faceOnlyNeckToBall();
}

/********************************************************************************/
bool PlannerBehaviour::offerAsPassCandidate(){

  if(myself->getStamina() < 2500)
    {
      return false;
    }
  const Player* ballOwner=worldModel->getTMWithBall();
  
  if((ballOwner==NULL)||(ballOwner==myself)||(!worldModel->isIntersected(ballOwner->getPosition(), myself->getPosition()))){
    return(false);
  }
  
  Vector2D circleCenter = myself->getHomePosition();
  Circle actionArea = Circle(circleCenter, 10);
  Vector2D point1 = Vector2D();
  Vector2D point2 = Vector2D();
  if(actionArea.calcTangentIntersectionPoints(ballOwner->getPosition(),point1,point2)){
  DEBUG4_FRITSCHE(myself->getUniformNumber(), " : ", point1, point2);
  }else{

  DEBUG2_FRITSCHE(myself->getUniformNumber(), " : innerhalb.");
  }

  float angle1=worldModel->angleFromPointToPoint(ballOwner->getPosition(),point1);
  float angle2=worldModel->angleFromPointToPoint(ballOwner->getPosition(),point2);
  DEBUG2_FRITSCHE("Angle1: ", angle1);
  DEBUG2_FRITSCHE("Angle2: ", angle2);

  //old method !!! reimplement!!!
  Vector2D leftBorderPosition = ballOwner->getPosition() + Vector2D(10, -160, POLAR);
  Vector2D rightBorderPosition = ballOwner->getPosition() + Vector2D(10, 160, POLAR);
  float freeAngle=worldModel->getFreeAngle(ballOwner->getPosition(), leftBorderPosition, rightBorderPosition);
  DEBUG2_FRITSCHE("freeAngle: ", freeAngle);

  if(((angle1<freeAngle)&&(angle2>freeAngle))||((angle1>freeAngle)&&(angle2<freeAngle))){
    DEBUG2_FRITSCHE(myself->getUniformNumber(), "Ich mu vielleicht laufen...");
    if(skillPool->goToPoint(worldModel->findBestPassCandOfferPoint(ballOwner->getPosition(),freeAngle),ConfigFile::positioningTolerance/2,100)){
      DEBUG2_FRITSCHE(myself->getUniformNumber(), "Ich mu laufen...");
    }
  }else{
    return(false);
  }
  return(true);
}


























