/*********************************************************************************
 * SkillPool.hh
 * Copyright (c) 2001 Virtual Werder
 *
 *
 * implements:
 *   class SkillPool
 *
 * requires:
 *
 * Created on: 26.10.2001 by wenns
 * Maintainer: cniehaus
 * Group:
 * Co-authors: harlekin
 *
 * Changelog:
 *
 * $Id: SkillPool.hh,v 1.19 2004/04/22 16:30:43 harlekin Exp $
 *
 ********************************************************************************/

#ifndef SKILLPOOL_HH
#define SKILLPOOL_HH

#include "Geometry.hh"
#include "WorldModel.hh"
#include "Effectors.hh"
//#include "Agent.hh"
//#include "Behaviour.hh"
//#include "GoalieBehaviour.hh"
//#include "WingAttackerBehaviour.hh"

/**
 * The namespace SkillPool is a collection of agents skills.
 */
class SkillPool{
public:
  WorldModel *worldModel;
  Effectors *effectors;
  Myself *myself;
  Ball *ball;
  //Behaviour *behaviour;
  //GoalieBehaviour *goalieBehaviour;
  //WingAttackerBehaviour *leftWingAttackerBehaviour;
  //WingAttackerBehaviour *rightWingAttackerBehaviour;

  //references
  //in SkillPool.cc: WorldModel*, Effectors*, Myself&, MovingObject *ball

  /**
   * Intialize SkillPool -> get all relevant informations from worldModel
   * initializes the pointer and references used in this namespace
   */
  SkillPool();
  ~SkillPool(){};

/*------------------ bersicht Start ----------------------------------------------//
  face-Skills
  -----------
  bool faceOnlyBodyToPoint(Vector2D point)
  bool faceOnlyBackToPoint(Vector2D point)
  bool faceOnlyNeckToPoint(Vector2D point)
  bool faceNeckToPoint(Vector2D point)
  bool faceNeckAndBodyToPoint(Vector2D point)

  bool faceOnlyBodyToPlayer(const Player& p)
  bool faceOnlyNeckToPlayer(const Player& p)
  bool faceNeckToPlayer(const Player& p)
  bool faceNeckAndBodyToPlayer(const Player& p)

  bool faceOnlyBodyToBall()
  bool faceOnlyNeckToBall()
  bool faceNeckToBall()
  bool faceNeckAndBodyToBall()

  goTo-Skills
  -----------
  bool getBall(float power=100, bool forCatch=false, bool closeInterception=true)
  bool interceptClose()
  bool interceptCloseGoalie()
  bool goToPoint(Vector2D target, float tolerance, float dashPower)

  bool goToPositionOurCKick()
  bool goToPositionTheirCKick()

  kickTo-Skills
  -------------
  bool accelerateBallToVelocity(Vector2D wishedVelocity, bool kickSmallerToo = true)
  bool kickToPoint(Vector2D kickPoint, float pow, bool oneKick = false)

  bool kickBallCloseToBody(float angleToBody, float distanceToBody = 0.175)
  bool turnWithBallTo(float globalAngle)
  void holdBall()
  bool stopBall()

  void passBallToPos(Vector2D pos, float targetVel = 1.0, bool oneKick = false)
  void passBallToPlayer(const Player* passCand, float targetVel = 1.0, bool oneKick = false)

  bool simpleDribbleToPoint(Vector2D targetPos, float kickAhead)
  bool smartDribbleToPoint(Vector2D targetPos, int maxDashPower, bool dribbleSide, bool fuzzy = false)

  bool scoreGoal(bool oppGoal = false)
  void kickAway(bool defensive = false, bool oneKick = false)
  
  other-Skills
  ------------
  void moveToInitialPosition()
  void moveToPosition(Vector2D whereToMove)
  void scanFieldWithNeck()
  void scanFieldWithBody()

  Vector2D coverEnemy(Vector2D) 
  Vector2D coverGoal(Vector2D)
  void coverLineFromTo(Vector2D pos1, Vector2D pos2)
  void coverGoal()
  void coverEnemy(const Player* enemy, bool offensive = true)

  void reportWorldModelStatus()
  bool catchBall()
  void doSSProfiling()
  void lookAround()
  void tackle(int power)
  void pointToPos(Vector2D pos, bool turnOff)
  void attentionTo(const Player* speaker, bool turnOff)
  void getScore()
  
//------------------ bersicht Ende -----------------------------------------------*/


//------------------ face-Skills Start --------------------------------------------//
 
  /**
   * Turns the body of the agent in the direction of the given point
   * If the body can't be turned in one cycle because of the limitation from 
   * the agents speed, the body is turned as far as possible 
   * @param point to turn the body to
   * @return true, if no turning is needed (no command was send)... so if the agent
   *         body facing already the right direction         
   *         false, otherwise...
   */
  bool faceOnlyBodyToPoint(const Vector2D &point);
 
  /**
   * Turns the back of the agent in the direction of the given point
   * If the body can't be turned in one cycle because of the limitation from 
   * the agents speed, the body is turned as far as possible 
   * @param point to turn the back of the agent to
   * @return true, if no turning is needed (no command was send)... so if the agent
   *               body facing already the right direction         
   *         false, otherwise...
   */
  bool faceOnlyBackToPoint(const Vector2D &point);

  /**
   * Turns only the neck of the agent in the direction of the given point.
   * If the full turn is not possible because of the turn-neck-limit,
   * the turn will be executed with maximal angle. The body direction
   * is not influenced.
   * @param point the point to turn the neck to
   * @return true, if no turning neck is needed (no command was send),so if the agent
   *         head facing already the right direction         
   *         false, otherwise...
   */  
  bool faceOnlyNeckToPoint(const Vector2D &point);

  /**
   * Turns the neck to a given point.
   * If the neck can't be turned in one cycle as much as needed, 
   * because of the limitation from the agents speed, the body is turned as 
   * far as needed. ATTENTION: only use this skill, if no other skill sends
   * an TURN command in the current cycle.
   * @return true, if the agent already facing the given point 
   *         false, otherwise...
   */
  bool faceNeckToPoint(const Vector2D &point);        

  /**
   * Turns the body and the neck to a given point.
   * If the body and the neck can't be turned in one cycle as much as needed, 
   * because of the limitation from the agents speed, the body and the neck  
   * is turned as far as needed. 
   * @return true, if the agent already facing the given point 
   *         false, otherwise...
   */
  bool faceNeckAndBodyToPoint(const Vector2D &point);

  // see the description of the ...Point counterpart
  inline bool faceOnlyBodyToPlayer(const Player& p){
    if (p.predictedPosition().distanceTo(p.getPosition()) <= Configuration::playerSpeedMax)
      return faceOnlyBodyToPoint(p.predictedPosition());
    else
      return faceOnlyBodyToPoint(p.getPosition());      
  }

  // see the description of the ...Point counterpart
  inline bool faceOnlyNeckToPlayer(const Player& p){
    if (p.predictedPosition().distanceTo(p.getPosition()) <= Configuration::playerSpeedMax)
      return faceOnlyNeckToPoint(p.predictedPosition());
    else
      return faceOnlyNeckToPoint(p.getPosition());
  }

  // see the description of the ...Point counterpart
  inline bool faceNeckToPlayer(const Player& p){
    if (p.predictedPosition().distanceTo(p.getPosition()) <= Configuration::playerSpeedMax)
      return faceNeckToPoint(p.predictedPosition());
    else
      return faceNeckToPoint(p.getPosition());
  }

  // see the description of the ...Point counterpart
  inline bool faceNeckAndBodyToPlayer(const Player& p){
    if (p.predictedPosition().distanceTo(p.getPosition()) <= Configuration::playerSpeedMax)
      return faceNeckAndBodyToPoint(p.predictedPosition());
    else
      return faceNeckAndBodyToPoint(p.getPosition());
  }

  // see the description of the ...Point counterparts
  bool faceOnlyBodyToBall();
  bool faceOnlyNeckToBall();
  bool faceNeckToBall();
  bool faceNeckAndBodyToBall();
  
//------------------ face-Skills Ende --------------------------------------------//

//------------------ goTo-Skills Start -------------------------------------------//

  /**
   * The skill getBall tries to intercept the ball. Therefore it uses the lower level 
   * skill goToPoint with a tolerance useful for intercepting. In order to intercept
   * the ball at a right distance (kickableMargin/4) from the agent, it uses the skills 
   * interceptClose  and interceptCloseGoalie for the last two steps to the ball. 
   * @param power dashPower for Interception
   * @param forCatch true, if goalie intercepts the ball
   * @param closeInterception true, for using the close interception skill
   * @return true, if the agent got the ball. 
   *         false, otherwise...
   */
  bool getBall(float power=100, bool forCatch=false, bool closeInterception=true);

  /**
   * The skill interceptClose creates the last two steps to the ball. Therefore it 
   * checks all possible combinations of dash and turn commands. The goal is to bring
   * the agent within a good distance to the ball (kickableMargin/4). It checks  
   * the single dash, the combination of first turn when dash and the combination
   * of two dashes.
   * @return true, if the agent can intercept within two cycles. 
   *         false, otherwise...
   */
  bool interceptClose();

  /**
   * The skill interceptCloseGoalie creates the last two steps to the ball, if a 
   * goalie wants to intercept the ball. Therefore it checks all possible combinations 
   * of dash and turn commands. The goal is to bring the goalie within the 
   * catchable distance to the ball (Configuration::catchableAreaL). It checks  
   * the single dash, the combination of first turn when dash and the combination
   * of two dashes.
   * @return true, if the goalie can catch the ball within two cycles. 
   *         false, otherwise...
   */
  bool interceptCloseGoalie();

  /**
   * Lets the agent go to the specified point. The point is reached, if the
   * distance to them is smaller than the specified tolerance. 
   * @param target the point to go to
   * @param tolerance the aforesaid tolerance
   * @param dashPower the dash power to use
   * @return true, if no soccer server command is queued (job done) and the agent 
   *         is in the right place
   *         false, otherwise
   */
  bool goToPoint (const Vector2D &target, float tolerance, float dashPower);

  /**
   * Lets the agent go to a position assigned to him in the case of our
   * corner kick
   * KICK IT OUT OF THE SKILLPOOL AND PUT IT INTO A CONFIGURATIONFILE
   */
  bool goToPositionOurCKick();

  /**
   * Lets the agent go to a position assigned to him in the case of a opponent
   * corner kick
   * KICK IT OUT OF THE SKILLPOOL AND PUT IT INTO A CONFIGURATIONFILE
   */
  bool goToPositionTheirCKick();

//------------------ goTo-Skills Ende --------------------------------------------//

//------------------ kick-Skills Start -------------------------------------------//

  /**
   * This skill accelerate the ball to a wished velocity, but it only does so,if its
   * possible, else the ball is shot smaller, if variable kickSmallerToo is set. 
   * The skill also prevent the ball from colliding with the agent by shooting
   * the ball to a save distance from the body.
   * @param wishedVelocity the desired velocity
   * @param kickSmallerToo, if one shot cant get the ball to the wished 
   *        velocity, shot smaller but in the correct direction
   * @return true, if velocity can be reached
   *         false, otherwise ...
   */
  bool accelerateBallToVelocity(const Vector2D &wishedVelocity, bool kickSmallerToo = true);

 /**
  * This skill mainly uses the low level skill accelerateBallToVelocity and adds
  * the functionality of putting the ball in right place within the agent
  * kickArea in order to get the right velocity in the following cycle, if the
  * needed velocity cant be reached. The variable oneKick is needed to execute kickIns 
  * and freeKicks.
  * @param kickPoint target point
  * @param pow the kick power
  * @param oneKick one kick only, needed for kickIns
  * @return true if the shot can be execute
  *         false otherwise ...
  */
  bool kickToPoint(const Vector2D &kickPoint, float pow, bool oneKick = false);

 /**
  * kicks the ball to a positon in the agents kickableArea on the 
  * desired distance to the body and the desired angle according 
  * to the bodydirection.
  * the default value for distanceToBody should be
  * Configuration::kickableMargin/4, now it is set to 0.175.
  * @param angleToBody desired angle according the bodydirection
  * @param distanceToBody distance to the body, a value between 0 and kickableMargin
  * @return true if the shot can be execute
  *         false otherwise ...
  */
  bool kickBallCloseToBody(float angleToBody, float distanceToBody = 0.175);
    
 /**
  * allows the agent to turn with the ball to a desired global angle. 
  * this happens in three step, if necessary:
  * 1. kick the ball close to body in the right direction (use of min-threshold
  *    for ball angle to direction when to turn)
  * 2. stop the ball (use of min-threshold for ball velocity when to stopBall)
  * 3. face neck and body to the desired position, which is simple  
  *    the predicted ball position.
  * @param globalAngle angle to turn with ball to
  * @return true if server command is queued
  */
 bool turnWithBallTo(float globalAngle);

 /**
  * this skill allows the agent to hold the ball away from the nearest 
  * opponent to the ball, but if there is no nearest opponent, 
  * the agent simply stops the ball - the distance to hold the ball
  * away is written down in the ConfigFile
  */
 void holdBall();
 void holdBall_H();

 /**
  * Lets the agent stop the ball, i. e. kick the ball in such a manner, that
  * the ball looses all his velocity relativ to the agent velocity.
  * @return true if server command is queued (job done)
  */
 bool stopBall();

 /**
  * Let the agent pass the ball to a position using accelerateBallToVelocity
  * @param pos the position on the field to pass to. ATTENTION: please set the targetVel!
  * @param targetVel the speed, that the ball should have at the target
  *        position
  */
  bool passBallToPos(const Vector2D &pos, float targetVel = -1.0, bool oneKick = false);

 /**
  * Let the agent pass the ball to a player with a given targetVelocity
  * using the skill passBallToPos. ATTENTION: please set the targetVel!
  * @param passCand the position of the player on the field to pass to
  * @param targetVel the speed, that the ball should have when
  * the player intercepts the ball
  */
 bool passBallToPlayer(const Player* passCand, float targetVel = -1.0, bool oneKick = false);

 /**
  * The skill enables the agent to dribble with the ball to a given point. The agent
  * therefore kicks the ball in such a way that he can intercept it a given 
  * distance ahead (kickAhead). This dribbling is a subbehaviour in which the agent
  * changes from kickToPoint() to getBall() and back. 
  * @param targetPos point to dribble with ball to
  * @param kickAhead distance which the agent kicks the ball ahead
  * @return at the moment simply true 
  */
  bool simpleDribbleToPoint(const Vector2D &targetPos, float kickAhead);
    
 /**
  * Lets the agent perform a smart dribbling in the direction of the
  * targetPos. The dribbling goal is to get to the targetPos without 
  * losing the control over the ball, therefore the agent tries to hold
  * the ball kickable in every cycle by performing kicks or dashs.
  * The angle of the ball relativ to the body is controled over the parameter
  * dribbleAngle, so we can keep the ball away from opponents and still
  * dribble along.
  * @param targetPos the position in which direction should be dribbled
  * @param maxDashPower the dash power for dribbling
  * @param dribbleSide true for right side and false for left side
  * @param fuzzy use random function or not
  * @return true, if a turn, kick or dash is queued(so always true!)
  */
  bool smartDribbleToPoint(const Vector2D &targetPos, int maxDashPower, bool dribbleSide, bool fuzzy = false);

 /**
  * ROAD WORKS! please come around later for results :)
  * should be called scorePenalty no scoreGoal!
  * FOR the normal please use tryToScoreGoal which uses simple kickToPoint
  * @param oppGoal selection of the goal - true for former oppGoal - false for ownGoal
  * @return always true
  */
 bool scoreGoal(bool oppGoal = false);

 /**
  * Lets the agent kick the ball into the open space between the opponents, if
  * "defensive" is set, to the nearest side out
  * @param defensive if set the agent shoots the ball to the nearest side out
  * @param oneKick for kickIn and freeKick usage
  */
 void kickAway(bool defensive = false, bool oneKick = false);
  
//------------------ kick-Skills Ende --------------------------------------------//

//------------------ other-Skills Start ------------------------------------------//

 /**
  * Moves the agent to his initial position on the field 
  * before kickoffs and after goals
  */
 bool moveToInitialPosition();

 /** 
  * Moves the goalie to a specific position in non-playOn-modi 
  * @param whereToMove the position to whichthe agent should be moved
  */
 void moveToPosition(const Vector2D &whereToMove);

 /**
  * Turns the neck in steps to the right. If no turn to the right is
  * possible, the neck will be turned to the maximum left position.
  * Calling this function a couple of cycles in a row, the field in front
  * of the agent can be scanned
  */
 void scanFieldWithNeck();

 /**
  * Turns the body in steps to the right. Calling this function a couple of
  * cycles in a row, the field around of the agent can be scanned
  */
 void scanFieldWithBody();

 /**
  * allows the agent to cover a line between two given points. 
  * the agent covers the line at a given coverDistance from the from-point,   
  * but if the coverDistance is larger than the distance between the two point
  * than the half of the distance between the point is taken.
  * @param Vector2D pos1 - from-point 
  * @param Vector2D pos2 - to-point
  */
 void coverLineFromTo(const Vector2D &pos1, 
		      const Vector2D &pos2);
  
 /**
  * the skill allows the agent to cover the line from the ball to the goal.
  * it decides which point of the goal has to be covered according
  * to the agent position
  */
 void coverGoal();
  
 /**
  * this skill covers offensive: the line between the ball and a given enemy
  * or covers defensive: the line between the goal and a given enemy
  * @param const Player* enemy  
  * @param bool offensiv 
  */  
 void coverEnemy(const Player* enemy, bool offensive = true);


 /**
  * Lets the agent communicate with his teammates and inform about his
  * beliefs about the environment. Because of limited communication only
  * the ball position and ball velocity is reported.
  */
 void reportBallStatus(bool dribbling = false);
 void reportPosition();
 void reportPassPoint();


 /**
  * This is for goalie use only - it simple calculates the catchangle and queued the
  * catch command, but only if the ball is catchable.
  * @return true  if server catch command is queued
  *         false if the ball is not catchable
  */
 bool catchBall();

 /**
  * synchronise the agent SEEMessageIncomingTime by switching the viewangle
  * at the right time 
  */
 void doSSProfiling();

 /**
  * switching the viewangle at the right time from normal to narrow and back
  */
 void doSwitchView();

 /**
  * simply performs a tackle in order to kick away the ball in the body direction 
  * with given power - in the behaviour it is decided when to tackle
  * @param power tackle power, affect the ball speed in the next cycle
  */
 void tackle(int power);
    
 /**
  * points the arm of the agent to a given point on the field
  * @param pos point on the field to point 
  * @param turnOff finishing the pointing 
  */ 
 void pointToPos(const Vector2D &pos, bool turnOff);

 /** 
  * paying attention to a certain player in order to ignore the shit from
  * all other players on the field
  * @param speaker, player to pay attention to - for a while
  * @param turnOff finish paying attention to a certain player
  */
 void attentionTo(const Player* speaker, bool turnOff);

 /** 
  * simply gets the current score of the game from the server
  */
 void getScore();
 

/******************************************************************************/

  void moveLikeAnIdiot();

/**
 * calcAngleOffset is onlay used for scorePenalty h scoreGoal - KICK IT!
 */
 static float calcAngleOffset(float angle){
    int sgn = sign(angle);
    angle = fabs(angle);
    float offset = 0;
    if(angle >= 0 && angle < 60)
      offset = 0;
    else if(angle >=60 && angle < 75)
      offset = 2;
    else if(angle >=75 && angle < 82.5)
      offset = 4;
    else if(angle >=82.5 && angle <= 90)
      offset = 8;
    return offset * sgn;
  }
//------------------ other-Skills Ende -------------------------------------------//

};
#endif //SKILLPOOL_HH



