/******************************************************************************
 * Ball.cc
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   class Ball
 *
 * requires:
 *
 *
 * Created on: 15.11.2001 by wenns
 * Maintainer: wenns
 * Group:
 * Co-authors:
 *
 * Changelog:
 *
 ******************************************************************************/

#include "Ball.hh"
#include "Configuration.hh"
#include "debug.hh"

Ball::Ball(){
  lastKickedBy = NULL;
}


float Ball::calculateKickRate(const float distance,
			      const float angle) const {
  //the used formulas can be found in ss manual, section kick-model
  float angleFactor    = 0.25f * fabs(angle) / 180.0;
  float distanceFactor = 0.25f * ((distance - Configuration::ballSize - Configuration::playerSize) 
				/ Configuration::kickableMargin);
  return (1 - angleFactor - distanceFactor);
}


Vector2D Ball::predictedPosition(int cycles, float kickDir, float kickPower,
                                 float distance, float angle) const {
  static Vector2D acceleration;
  static Vector2D predPosition;
  static Vector2D predVelocity;
  predPosition.setVector2D(position);
  predVelocity.setVector2D(velocity);
  
  if (position.isInvalid() || velocity.isInvalid()){
    if (position.isInvalid()){
      DEBUG1("Ball::predictedPosition(): Can't estimate future if don't know present");
    }
    return position;
  }

  //if the ball is kicked by myself, compute the ball acceleration from kick
  if (kickDir != UNDEFINED_FLOAT && kickPower != UNDEFINED_FLOAT){
    acceleration.setVector2D(calculateKickRate(distance, angle) * 
			     kickPower * 
			     Configuration::kickPowerRate,
                             kickDir, POLAR);
    predVelocity += acceleration;
  }
  
  //reduce the ball velocity to ballSpeedMax to avoid error from noise
  if (predVelocity.getLength() > Configuration::ballSpeedMax)
    predVelocity.setLength(Configuration::ballSpeedMax);

  if (predVelocity.getLengthSq() == 0) 
    return predPosition;
  //calculate predicted position after nr of cycles
  double dDist = Geometry::getSumGeomSeries( predVelocity.getLength(),
					     Configuration::ballDecay,
					     cycles );
  predPosition += Vector2D( dDist, predVelocity.getDirection(), POLAR);
  predVelocity *= pow( Configuration::ballDecay, cycles );
  /*
  for (int i = 0; i < cycles; i++){
    predPosition += predVelocity;
    predVelocity *= Configuration::ballDecay;
    //if (predVelocity.getLength() < 0.001) break;
  } 
  */
  return predPosition;
}


Ball::~Ball(){
}

