/***********************************************************************
 * Configuration.hh
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   class Configuration.
 *
 * requires:
 *
 *
 * Created on: 25.03.2001 by wenns
 * Maintainer: wenns
 * Group:
 * Co-authors:
 *
 * Changelog:
 *
 **********************************************************************/
#ifndef CONFIGURATION_HH
#define CONFIGURATION_HH

#include <iostream>

#include "SenseObjects.hh"

/**
 * Contains the parameters of the simulation. The parameters are specified
 * by the robocup simulation commitee and are gained from the SERVERPARAM,
 * PLAYERPARAM and PLAYERTYPE messages. The PLAYERTYPE messages describes
 * several different player types, which can be deployed using online coach.
 * This last feature is called "heterogeneous players" and is not used yet.
 */
class Configuration {
private:

  static const int gameLogDir_Len       = 128;
  static const int gameLogFixedName_Len = 64;
  static const int landmarkFile_Len     = 128;
  static const int logDateFormat_Len    = 32;
  static const int textLogDir_Len       = 128;
  static const int textLogFixedName_Len = 64;

public:
  /** \name Parameters that comes from the SERVERPARAM msg */
  //@{
  static int catchBanCycle;              /**< number of cycles after catch in which the
					    goalie cannot catch again */
  static int clangAdviceWin;             /**< number of advice messages per window */
  static int clangDefineWin;             /**< number of define messages per window */
  static int clangDelWin;
  static int clangInfoWin;               /**< number of info messages per window */
  static int clangMessDelay;             /**< delay between receipt of message and send
					    to players*/
  static int clangMessPerCycle;          /**< maximum number of coach messages sent per
					    cycle*/
  static int clangMetaWin;               /**< number of meta messages per window */
  static int clangRuleWin;
  static int clangWinSize;           /**< time window which controls how many
					messages  can be sent */
  static int coachPort;
  static int dropBallTime;           /**< the number of cycles to wait until
					dropping the ball automatically for free
					kicks, corner kicks, etc */
  static int freeformSendPeriod;
  static int freeformWaitPeriod;
  static int gameLogCompression;
  static int gameLogVersion;
  static int goalieMaxMoves;        /**< maximum number of 'move' actions allowed
				       for the goalie after successfull catch */
  static int halfTime;              /**< length of game half in seconds */
  static int hearDecay;             /**< decay of hear messages */
  static int hearInc;               /**< hear capacity increment */
  static int hearMax;               /**< maximal hear capacity */
  static int maxGoalKicks;
  static int olcoachPort;
  static int pointToBan;
  static int pointToDuration;
  static int port;
  static int recvStep;              /**< time step of acception of command */
  static int sayCoachCntMax;        /**< maximum number of coach messages */
  static int sayCoachMsgSize;       /**< Upper limit of length of online
				       coach's message (default 128) */
  static int sayMsgSize;            /**< maximum string size of say message */
  static int sendStep;              /**< the length of the time interval between visual
				       messages to a player in the standard view mode */
  static int sendViStep;
  static int senseBodyStep;        /**< time intervall of sense body messages */
  static int simulatorStep;        /**< time step of simulation in ms */
  static int slowDownFactor;       /**< factor to slow down the simulation */
  static int startGoalL;
  static int startGoalR;
  static int synchMicroSleep;
  static int synchOffset;
  static int tackleCycles;
  static int textLogCompression;
  static char gameLogDir[];
  static char gameLogFixedName[];
  static char landmarkFile[];
  static char logDateFormat[];
  static char textLogDir[];
  static char textLogFixedName[];
  static bool coach;
  static bool coachWReferee;
  static bool oldCoachHear;
  static bool windNone;
  static float windRandom;
  static bool backPasses;
  static bool forbidKickOffOffside;    /**< forbid kick off offside. [on/off] */
  static bool freeKickFaults;
  static bool fullstateL;
  static bool fullstateR;
  static bool gameLogDated;
  static bool gameLogFixed;
  static bool gameLogging;
  static bool logTimes;
  static bool profile;
  static bool properGoalKicks;
  static bool recordMessages;
  static bool sendComms;
  static bool synchMode;
  static bool teamActuatorNoise;
  static bool textLogDated;
  static bool textLogFixed;
  static bool textLogging;
  static bool useOffside;              /**< flag for using off side rule. */
  static bool verbose;
  static float audioCutDist;           /**< maximum distance over which a spoken
					  message can be heard */
  static float ballAccelMax;           /**< maximum ball acceleration per cycle */
  static float ballDecay;              /**< ball speed decay per cycle */
  static float ballRand;               /**< random error in ball movement */
  static float ballSize;               /**< the size (=radius) of the ball */
  static float ballSpeedMax;           /**< maximum ball speed */
  static float ballWeight;             /**< the weight of the ball (for wind) */
  static float catchProbability;       /**< the probability for a goalie to catch the
					  ball when it's in the catch area */
  static float catchableAreaL;         /**< the length of the area in which the
					  goalie can catch the ball */
  static float catchableAreaW;         /**< the width of the area in which the
					  goalie can catch the ball */
  static float ckickMargin;            /**< the minimum distance to the ball
					  when a corner kick is taken */
  static float controlRadius;
  static float dashPowerRate;          /**< rate by which the 'power' argument in a
					  'dash' command is multiplied (thus
					  determining the amount of displacement of
					  the player as a result of the 'dash') */
  static float effortDec;              /**< decrement step per cycle for the effort */
  static float effortDecThr;           /**< percentage of staminaMax below
					  which player effort decreases */
  static float effortInc;              /**< increment step per cycle for the effort */
  static float effortIncThr;           /**< percentage of staminaMax above which
					  effort increases */
  static float effortInit;             /**< the initial value of effort */
  static float effortMin;              /**< minimum value of effort */
  static float goalWidth;              /**< width of the goal */
  static float inertiaMoment;          /**< inertia moment of a player: affects actual
					  turn angle depending on speed */
  static float kickPowerRate;          /**< rate by which the 'power' argument in a
					  'kick' command is multiplied (thus
					  determining the amount of displacement of
					  the ball as a result of the 'kick'*/
  static float kickRand;               /**< random error in kick direction */
  static float kickRandFactorL;
  static float kickRandFactorR;
  static float kickableMargin;         /**< margin around player in which ball is
					  kickable; kickable area thus equals
					  kickableMargin+ballSize+playerSize */
  static float maxmoment;             /**< maximum angle for turn and kick */
  static float maxneckang;            /**< maximum neck angle rel. to body */
  static float maxneckmoment;         /**< maximum angle for turn neck */
  static float maxpower;              /**< maximum power for dash and kick */
  static float minmoment;             /**< minimum angle for turn and kick */
  static float minneckang;            /**< minimum neck angle rel. to body */
  static float minneckmoment;         /**< minimum angle for turn neck */
  static float minpower;              /**< minimum power for dash and kick */
  static float offsideActiveAreaSize; /**< radius of circle around the ball in
					 which a player can be offside */
  static float offsideKickMargin;   /**< the minimum distance to the ball for
				       offending players when a free kick for
				       offside is taken*/
  static float playerAccelMax;      /**< maximum acceleration of a player per
				       cycle */
  static float playerDecay;         /**< player speed decay per cycle */
  static float playerRand;          /**< random error in player movement */

  static float playerSize;          /**< the size (=radius) of a player */

  static float playerSpeedMax;      /**< maximum speed of a player*/
  static float playerWeight;        /**< the weight of a player (for wind) */
  static float prandFactorL;
  static float prandFactorR;
  static float quantizeStep;       /**< quantization step for distance of moving
				      objects */
  static float quantizeStepL;      /**< quantization step for distance of
				      landmarks */
  static float recoverDec;         /**< decrement step per cycle for recovery */
  static float recoverDecThr;      /**< percentage of staminaMax below which
				      player recovery decreases */
  static float recoverMin;         /**< minimum of recovery */
  static float slownessOnTopForLeftTeam;
  static float slownessOnTopForRightTeam;
  static float staminaIncMax;      /**< maximum stamina increase per cycle */
  static float staminaMax;         /**< stamina maximum */
  static float stoppedBallVel;
  static float tackleBackDist;
  static float tackleDist;
  static float tackleExponent;
  static float tacklePowerRate;
  static float tackleWidth;
  static float visibleAngle;      /**< angle of the view cone of a
				     player in the standard view mode */
  static float visibleDistance;   /**< margin of the 'feel area': distance within
				     which objects are always 'visible' (even
				     when not in view cone) */
  static float windAng;
  static float windDir;           /**< direction of the wind */
  static float windForce;         /**< the force ot the wind */
  static float windRand;          /**< random factor in the wind direction */
  //@}


  /** \name Parameters that comes from the PLAYERPARAM msg */
  //@{
  static int playerTypes;                       /**< the number of player types
						   including the default player type */
  static int ptMax;
  static float randomSeed;
  static int subsMax;                           /**< the maximum number of substitutions
						   allowed during a game; the value also
						   indicates the maximum number of
						   players allowed for each type*/
  static float dashPowerRateDeltaMax;           /**< maximum delta for adjusting
						   dashPowerRate */
  static float dashPowerRateDeltaMin;           /**< minimum delta for adjusting
						   dashPowerRate */
  static float effortMaxDeltaFactor;            /**< amount delta is multiplied by for
						   effortMax */
  static float effortMinDeltaFactor;            /**< amount delta is multiplied by for
						   effortMin */
  static float extraStaminaDeltaMax;            /**< maximum delta for adjusting
						   extraStamina */
  static float extraStaminaDeltaMin;            /**< minimum delta for adjusting
						   extraStamina */
  static float inertiaMomentDeltaFactor;        /**< amount by which delta is multiplied
						   for inertia_moment */
  static float kickRandDeltaFactor;             /**< amount delta is multiplied by for
						   kickRandom */
  static float kickableMarginDeltaMax;          /**< maximum delta for adjusting
						   kickableMargin */
  static float kickableMarginDeltaMin;          /**< minimum delta for adjusting
						   kickableMargin */
  static float newDashPowerRateDeltaMax;
  static float newDashPowerRateDeltaMin;
  static float newStaminaIncMaxDeltaFactor;
  static float playerDecayDeltaMax;             /**< maximum delta for adjusting
						   playerDecay*/
  static float playerDecayDeltaMin;             /**< minimum delta for adjusting
						   playerDecay*/
  static float playerSizeDeltaFactor;           /**< amount delta is multiplied by for
						   player_size */
  static float playerSpeedMaxDeltaMax;          /**< maximum delta for adjusting
						   playerSpeedMax */
  static float playerSpeedMaxDeltaMin;          /**< minimum delta for adjusting
						   playerSpeedMax */
  static float staminaIncMaxDeltaFactor;        /**< amount by which delta is multiplied
						   for staminaInc */
  //@}


  //REVISIT: where is extraStamina parameter needed for heterogeneous players

  //------------------------   player types ------------------------------
  /**
   * stores the parameters of all heterogenous player types
   * (but not the default player type)
   */
  static vector<Sense::PlayerType> playerTypesV;
  static Sense::PlayerType playerTypesA[7];
  
  static Sense::PlayerType *getPlayerType(int pT);

  static float getPlayerSpeedMax(int pT);
  static float getStaminaIncMax(int pT);
  static float getStaminaMax(int pT);
  static float getPlayerDecay(int pT);
  static float getInertiaMoment(int pT);
  static float getDashPowerRate(int pT);
  static float getPlayerSize(int pT);
  static float getKickableMargin(int pT);
  static float getKickRand(int pT);
  static float getExtraStamina(int pT);
  static float getEffortMax(int pT);
  static float getEffortMin(int pT);
  static float getMaximumPlayerSpeed(int pT);
  
  
  
  
  Configuration();
  virtual ~Configuration();

  /**
   * sets the server parameters with the parsed contents of a SERVERPARAM
   * message
   * @param sp the parsed content of a SERVERPARAM message
   */
  void integrateServerParam(const Sense::ServerParam &sp);

  /**
   * sets the player parameters with the parsed contents of a PLAYERPARAM
   * message
   * @param pp the parsed content of a PLAYERPARAM message
   */
  void integratePlayerParam(const Sense::PlayerParam &pp);

  /** prints the values of the server parameters on standard output */
  void printServerParam();

  /** prints the values of the player parameters on standard output */
  void printPlayerParam();

  /**
   * adds a player type to the set (vector) of stored player types
   * @param type the player type to add
   */
  void addPlayerType(const Sense::PlayerType &type);

  /** prints the player parameters of the heterogeneous players */
  static void printPlayerTypes();
};

#endif //CONFIGURATION_HH





