/***********************************************************************
 * Formation.hh
 * Copyright (c) 2003 Virtual Werder
 *
 * implements:
 *   class Formation
 *
 * requires:
 *
 *
 * Created on: 19.09.2003 by harlekin
 * Maintainer: cniehaus, harlekin
 * Group:
 * Co-authors: cniehaus
 *
 * Changelog:
 *
 * $Id: Formation.hh,v 1.4 2004/04/03 07:58:16 harlekin Exp $
 *
 **********************************************************************/

#ifndef FORMATION_HH
#define FORMATION_HH

#include "Geometry.hh"
#include "debug.hh"
/*
#include "common.hh"
#include "Behaviour.hh"
*/
class Formation{
public:
  Formation(){};
  Formation(int id, 
	    const int *b, 
	    const double *hpx,  
	    const double *hpy,  
	    const double *ipx,  
	    const double *ipy,  
	    const double *aX, 
	    const double *aY,  
	    const double *maxX, 
	    const double *minX, 
	    const bool *bB);
    
  ~Formation(){};
  
  int       getBehaviour(int unum) const { if (unum == UNDEFINED_INT) return -1;
                                     else return (behaviour)[unum-1]; }
  const Vector2D  &getHomePos(int unum) const   { return homePos[unum-1]; }
  const Vector2D  &getHomePosition(int unum) const  { 
    return (homePos[unum-1]); 
  }
  const Vector2D  &getInitPos(int unum) const  { 
    return initPos[unum-1]; }
  const Vector2D  &getInitPosition(int unum) const  { 
    return initPos[unum-1]; }
  double    getAttrX(int unum) const    { return attrX[unum-1]; }
  double    getAttractionX(int unum) const    { return attrX[unum-1]; }
  double    getAttrY(int unum) const    { return attrY[unum-1]; }
  double    getAttractionY(int unum) const    { return attrY[unum-1]; }
  double    getMaxX(int unum) const     { return maxX[unum-1]; }
  double    getMinX(int unum) const     { return minX[unum-1]; }
  bool      getBehindBall(int unum) const { return behindBall[unum-1]; }
  int       getId() const { return id; } 
  void      print(){
    DEBUG2("Formation, ID: ", id);
    for (int i = 0; i < 11; i++){
      DEBUG2(" Player: ", (i+1));
      DEBUG2("  HomePos:    ", homePos[i]);
      DEBUG2("  InitPos:    ", initPos[i]);
      DEBUG2("  AttrX:      ", attrX[i]);
      DEBUG2("  AttrY:      ", attrY[i]);
      DEBUG2("  BehindBall: ", behindBall[i]);
      DEBUG2("  MaxX:       ", maxX[i]);
      DEBUG2("  MinX:       ", minX[i]);
    }
  }

  int       behaviour[11];
  Vector2D  homePos[11];
  Vector2D  initPos[11];
  double    attrX[11];
  double    attrY[11];
  double    maxX[11];
  double    minX[11];
  bool      behindBall[11];
  int       id;
};


#endif
