/*****************************************************************************
 * PotentialField.cc
 * Copyright (c) 2003 Virtual Werder
 *
 * implements:
 *   class PotentialField
 *
 * requires:
 *
 * Created on: 07.03.2003 by tekgeist
 * Maintainer:
 * Group:
 * Co-authors:
 *
 * Changelog:
 *
 ****************************************************************************/

#include "PotentialField.hh"
#include "debug.hh"
#include "common.hh"
#include "ConfigFile.hh"

PotentialField::PotentialField(){
   ;
}

PotentialField::~PotentialField(){
   ;
}

//begin<Nifty_Math_Stuff> ;)

/******** The Potential Functions of Game Objects ********/
/******** dunno whether we will ever use these... ********/

float PotentialField::playFieldBorderPotential      (const Vector2D pos){
   float x     = pos.getX();
   float y     = pos.getY();
  
   return (  20/(1+exp(2*x+LENGTH))   
            +20/(1+exp(LENGTH-2*x))
            +20/(1+exp(2*y+WIDTH))
            +20/(1+exp(WIDTH-2*y))  );
}

float PotentialField::theirGoalLogisticPotential    (const Vector2D pos){
   float x     = pos.getX();
   float y     = pos.getY();

   return(
         -50/(1+exp(1/2*sqrt(4*pow(x,(float)2)-4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/THEIRGOAL_SIGMA))
         );
}

float PotentialField::ourGoalLogisticPotential      (const Vector2D pos){
   float x     = pos.getX();
   float y     = pos.getY();
   
   return(
         50/(1+exp(1/2*sqrt(4*pow(x,(float)2)+4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/OURGOAL_SIGMA))
         );
}

float PotentialField::theirPlayerGaussPotential     (const Vector2D pos){
   float x     = pos.getX();
   float y     = pos.getY();
  
   return ( ENEMY_A/ENEMY_SIGMA*sqrt((float)2)/sqrt(Pi)*exp(-1/2*(pow(x,(float)2)+pow(y,(float)2))/pow(ENEMY_SIGMA,(float)2)) );
}

float PotentialField::ourPlayerGaussPotential       (const Vector2D pos){
   /*
    * ATTENTION: massive piece of bullshit ahead!
    */
   float x     = pos.getX();
   float y     = pos.getY();
   
   return (-5*TEAMMATE_A/TEAMMATE_SIGMA*sqrt((float)2)/sqrt(Pi)*exp(-1/2*(pow(x,(float)2)+pow(y,(float)2))/pow(TEAMMATE_SIGMA,(float)2)) );
}
            
/******** The Derivatives of the Potential Functions ********/

void PotentialField::playFieldBorderGradient    (const Vector2D pos, Vector2D &gradient){
   float x     = pos.getX();
   float y     = pos.getY();

   float gradX=(float)  (
                           -(double)20/
                           pow(((double)1+exp((double)x+(double)1/(double)2*(double)LENGTH)),(double)2)*exp((double)x+(double)1/(double)2*(double)LENGTH)
                           +(double)20/
                           pow(((double)1+exp((double)1/(double)2*(double)LENGTH-(double)x)),(double)2)*exp((double)1/(double)2*(double)LENGTH-(double)x)
                        );

   float gradY=(float)  (
                           -(double)20/
                           pow(((double)1+exp((double)1/(double)2*(double)WIDTH+(double)y)),(double)2)*exp((double)1/(double)2*(double)WIDTH+(double)y)
                           +(double)20/
                           pow(((double)1+exp((double)1/(double)2*(double)WIDTH-(double)y)),(double)2)*exp((double)1/(double)2*(double)WIDTH-(double)y)
                        );

   //DEBUG5_TEKGEIST("PlayFieldBorderGradient :(", gradX, ", ", gradY, ")");
   gradient.setVector2D(gradX, gradY);
}

void PotentialField::theirGoalLogisticGradient  (const Vector2D pos, Vector2D &gradient){
   float x     = pos.getX();
   float y     = pos.getY();
   float gradX = ( 50/pow((1+exp(1/2*sqrt(4*pow(x,(float)2)-4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/THEIRGOAL_SIGMA)),(float)2)*(2*x-LENGTH)/sqrt(4*pow(x,(float)2)-4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/THEIRGOAL_SIGMA*exp(1/2*sqrt(4*pow(x,(float)2)-4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/THEIRGOAL_SIGMA) );
   float gradY = ( 100/pow((1+exp(1/2*sqrt(4*pow(x,(float)2)-4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/THEIRGOAL_SIGMA)),(float)2)*y/sqrt(4*pow(x,(float)2)-4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/THEIRGOAL_SIGMA*exp(1/2*sqrt(4*pow(x,(float)2)-4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/THEIRGOAL_SIGMA) );
   
   //DEBUG5_TEKGEIST("TheirGoalGradient :(", gradX,", ", gradY, ")");
   gradient.setVector2D(gradX, gradY);
}

void PotentialField::ourGoalLogisticGradient    (const Vector2D pos, Vector2D &gradient){
   float x     = pos.getX();
   float y     = pos.getY();
   float gradX = ( -50/pow((1+exp(1/2*sqrt(4*pow(x,(float)2)+4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/OURGOAL_SIGMA) ),(float)2)*(2*x+LENGTH)/sqrt(4*pow(x,(float)2)+4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/OURGOAL_SIGMA*exp(1/2*sqrt(4*pow(x,(float)2)+4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/OURGOAL_SIGMA) );
   float gradY = ( -100/pow((1+exp(1/2*sqrt(4*pow(x,(float)2)+4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/OURGOAL_SIGMA)),(float)2)*y/sqrt(4*pow(x,(float)2)+4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/OURGOAL_SIGMA*exp(1/2*sqrt(4*pow(x,(float)2)+4*x*LENGTH+pow(LENGTH,(float)2)+4*pow(y,(float)2))/OURGOAL_SIGMA) );
   
   //DEBUG5_TEKGEIST("OurGoalGradient :(", gradX,", ", gradY, ")");
   gradient.setVector2D(gradX, gradY);
}

void PotentialField::theirPlayerGaussGradient   (const Vector2D pos, Vector2D &gradient){
   float x     = pos.getX();
   float y     = pos.getY();
#ifdef dribbleback
   float gradX = ( (float)-25*ENEMY_A*x*exp(-(pow(x,(float)2)+pow(y,(float)2))/((float)2*pow(ENEMY_SIGMA,(float)2)))/(pow(ENEMY_SIGMA,(float)3)*sqrt((float)2*Pi)) );
#else
   float gradX = 0;
#endif
   float gradY = ( (float)-25*ENEMY_A*y*exp(-(pow(x,(float)2)+pow(y,(float)2))/((float)2*pow(ENEMY_SIGMA,(float)2)))/(pow(ENEMY_SIGMA,(float)3)*sqrt((float)2*Pi)) );

   //DEBUG5_TEKGEIST("TheirPlayerGradient :(", gradX,", ", gradY, ")");
   gradient.setVector2D(gradX, gradY);
}

void PotentialField::ourPlayerGaussGradient     (const Vector2D pos, Vector2D &gradient){
   /*
    * ATTENTION: massive piece of bullshit ahead!
    */
   float x     = pos.getX();
   float y     = pos.getY();
   float gradX = ( (float)10*TEAMMATE_A*x*exp(-(pow(x,(float)2)+pow(y,(float)2))/((float)2*pow(TEAMMATE_SIGMA,(float)2)))/(pow(TEAMMATE_SIGMA,(float)3)*sqrt((float)2*Pi)) );
   float gradY = ( (float)10*TEAMMATE_A*y*exp(-(pow(x,(float)2)+pow(y,(float)2))/((float)2*pow(TEAMMATE_SIGMA,(float)2)))/(pow(TEAMMATE_SIGMA,(float)3)*sqrt((float)2*Pi)) );

   //DEBUG5_TEKGEIST("OurPlayerGradient :(", gradX,", ", gradY, ")");
   gradient.setVector2D(gradX, gradY);
}
                              
//end<Nifty_Math_Stuff>

// $Source: /home/robocup/virtualwerder/CVSRepository/VirtualWerder2004/worldmodel/PotentialField.cc,v $
